<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0"    
    xmlns:msxsl="urn:schemas-microsoft-com:xslt"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
  
<!-- (c) 2017, Trimble Inc. All rights reserved.                                               -->
<!-- Permission is hereby granted to use, copy, modify, or distribute this style sheet for any -->
<!-- purpose and without fee, provided that the above copyright notice appears in all copies   -->
<!-- and that both the copyright notice and the limited warranty and restricted rights notice  -->
<!-- below appear in all supporting documentation.                                             -->

<!-- TRIMBLE INC. PROVIDES THIS STYLE SHEET "AS IS" AND WITH ALL FAULTS.                       -->
<!-- TRIMBLE INC. SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF MERCHANTABILITY               -->
<!-- OR FITNESS FOR A PARTICULAR USE. TRIMBLE INC. DOES NOT WARRANT THAT THE                   -->
<!-- OPERATION OF THIS STYLE SHEET WILL BE UNINTERRUPTED OR ERROR FREE.                        -->

<xsl:output method="text" omit-xml-declaration="no" encoding="utf-8"/>

<!-- Set the numeric display details i.e. decimal point, thousands separator etc -->
<xsl:variable name="DecPt" select="'.'"/>    <!-- Change as appropriate for US/European -->
<xsl:variable name="GroupSep" select="','"/> <!-- Change as appropriate for US/European -->
<!-- Also change decimal-separator & grouping-separator in decimal-format below 
     as appropriate for US/European output -->
<xsl:decimal-format name="Standard" 
                    decimal-separator="."
                    grouping-separator=","
                    infinity="Infinity"
                    minus-sign="-"
                    NaN="?"
                    percent="%"
                    per-mille="&#2030;"
                    zero-digit="0" 
                    digit="#" 
                    pattern-separator=";" />

<xsl:variable name="DecPl0" select="'#0'"/>
<xsl:variable name="DecPl1" select="concat('#0', $DecPt, '0')"/>
<xsl:variable name="DecPl2" select="concat('#0', $DecPt, '00')"/>
<xsl:variable name="DecPl3" select="concat('#0', $DecPt, '000')"/>
<xsl:variable name="DecPl4" select="concat('#0', $DecPt, '0000')"/>
<xsl:variable name="DecPl5" select="concat('#0', $DecPt, '00000')"/>
<xsl:variable name="DecPl6" select="concat('#0', $DecPt, '000000')"/>
<xsl:variable name="DecPl8" select="concat('#0', $DecPt, '00000000')"/>

<xsl:variable name="fileExt" select="'crd'"/>

<!-- User variable definitions - Appropriate fields are displayed on the       -->
<!-- Survey Controller screen to allow the user to enter specific values       -->
<!-- which can then be used within the style sheet definition to control the   -->
<!-- output data.                                                              -->
<!--                                                                           -->
<!-- All user variables must be identified by a variable element definition    -->
<!-- named starting with 'userField' (case sensitive) followed by one or more  -->
<!-- characters uniquely identifying the user variable definition.             -->
<!--                                                                           -->
<!-- The text within the 'select' field for the user variable description      -->
<!-- references the actual user variable and uses the '|' character to         -->
<!-- separate the definition details into separate fields as follows:          -->
<!-- For all user variables the first field must be the name of the user       -->
<!-- variable itself (this is case sensitive) and the second field is the      -->
<!-- prompt that will appear on the Survey Controller screen.                  -->
<!-- The third field defines the variable type - there are four possible       -->
<!-- variable types: Double, Integer, String and StringMenu.  These variable   -->
<!-- type references are not case sensitive.                                   -->
<!-- The fields that follow the variable type change according to the type of  -->
<!-- variable as follow:                                                       -->
<!-- Double and Integer: Fourth field = optional minimum value                 -->
<!--                     Fifth field = optional maximum value                  -->
<!--   These minimum and maximum values are used by the Survey Controller for  -->
<!--   entry validation.                                                       -->
<!-- String: No further fields are needed or used.                             -->
<!-- StringMenu: Fourth field = number of menu items                           -->
<!--             Remaining fields are the actual menu items - the number of    -->
<!--             items provided must equal the specified number of menu items. -->
<!--                                                                           -->
<!-- The style sheet must also define the variable itself, named according to  -->
<!-- the definition.  The value within the 'select' field will be displayed in -->
<!-- the Survey Controller as the default value for the item.                  -->


<xsl:param name="leftInterfaceStringName" select="''"/>
<xsl:param name="rightInterfaceStringName" select="''"/>

<xsl:variable name="useLeftInterfaceStringName" select="translate($leftInterfaceStringName,'abcdefghijklmnopqrstuvwxyzáéíóúñü','ABCDEFGHIJKLMNOPQRSTUVWXYZÁÉÍÓÚÑÜ')"/>
<xsl:variable name="useRightInterfaceStringName" select="translate($rightInterfaceStringName,'abcdefghijklmnopqrstuvwxyzáéíóúñü','ABCDEFGHIJKLMNOPQRSTUVWXYZÁÉÍÓÚÑÜ')"/>

<xsl:param name="selectedAlignment" select="''"/>

<xsl:param name="specifiedStartStation"/>
<xsl:param name="specifiedEndStation"/>
<xsl:param name="regularStnInterval"/>  <!-- Value to use as the regular station interval -->

<xsl:variable name="includeRegularStations" select="'true'"/>

<!-- Create a node set variable containing the LandXML file contents without any namespace references. -->
<!-- This allows different LandXML file versions to be processed using the same style sheet.           -->
<xsl:variable name="noNamespaceLandXML">
  <xsl:element name="LandXML">
    <xsl:call-template name="CopyOf">
      <xsl:with-param name="select" select="child::*[1]"/>
    </xsl:call-template>
  </xsl:element>
</xsl:variable>

<xsl:template name="CopyOf">   <!-- Copies all the elements from the LandXML file without any namespace references -->
  <xsl:param name="select" />

  <xsl:for-each select="$select">
    <xsl:for-each select="node()|text()">
      <xsl:choose>
        <xsl:when test="self::*">   <!-- Node -->
          <xsl:element name="{local-name()}">
            <xsl:copy-of select="@*" />
            <xsl:call-template name="CopyOf">
              <xsl:with-param name="select" select="." />
            </xsl:call-template>
          </xsl:element>
        </xsl:when>

        <xsl:otherwise>  <!-- Text -->
          <xsl:if test="not(self::comment())">  <!-- Skip any comment elements -->
            <xsl:value-of select="." />
          </xsl:if>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:for-each>
  </xsl:for-each>
</xsl:template>

<!-- **************************************************************** -->
<!-- **************** Set up some conversion factors **************** -->  
<!-- **************************************************************** -->
<xsl:variable name="DistUnit" select="msxsl:node-set($noNamespaceLandXML)/LandXML/Units/*/@linearUnit"/>
  
<xsl:variable name="AngleUnit">
  <xsl:variable name="tempUnit" select="msxsl:node-set($noNamespaceLandXML)/LandXML/Units/*/@angularUnit"/>
  <xsl:choose>
    <xsl:when test="$tempUnit = ''">radians</xsl:when>  <!-- If angular units not defined default to radians (the LandXML default) -->
    <xsl:otherwise>
      <xsl:value-of select="$tempUnit"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:variable>

<xsl:variable name="AngleConvFactor" select="1.0"/>  <!-- Not actually used but required for included functions -->

<!-- Setup conversion factor for coordinate and distance values -->
<!-- Dist/coord values output to a RoadXML file are always in metres -->
<xsl:variable name="DistConvFactor">
  <xsl:choose>
    <xsl:when test="$DistUnit='millimeter'">0.001</xsl:when>      <!-- Convert to m -->
    <xsl:when test="$DistUnit='centimeter'">0.01</xsl:when>       <!-- Convert to m -->
    <xsl:when test="$DistUnit='meter'">1.0</xsl:when>             <!-- Output as m -->
    <xsl:when test="$DistUnit='kilometer'">1000.0</xsl:when>      <!-- Convert to m -->
    <xsl:when test="$DistUnit='foot'">1.0</xsl:when>              <!-- Output as int ft -->
    <xsl:when test="$DistUnit='USSurveyFoot'">1.0</xsl:when>      <!-- Output as survey ft -->
    <xsl:when test="$DistUnit='inch'">0.083333333333</xsl:when>   <!-- Convert to int ft -->
    <xsl:when test="$DistUnit='mile'">5280.0</xsl:when>           <!-- Convert to int ft -->
    <xsl:otherwise>1.0</xsl:otherwise>
  </xsl:choose>
</xsl:variable>

<xsl:variable name="alignmentToUse">
  <xsl:choose>
    <xsl:when test="$selectedAlignment != ''">
      <xsl:value-of select="$selectedAlignment"/>
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="msxsl:node-set($noNamespaceLandXML)/LandXML/Alignments[1]/Alignment[1]/@name"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:variable>

<xsl:variable name="Pi" select="3.14159265358979323846264"/>
<xsl:variable name="halfPi" select="$Pi div 2.0"/>

<!-- **************************************************************** -->
<!-- ************************** Main Loop *************************** -->
<!-- **************************************************************** -->
<xsl:template match="/" >

  <xsl:apply-templates select="msxsl:node-set($noNamespaceLandXML)/LandXML"/>

</xsl:template>

  
<!-- **************************************************************** -->
<!-- ****************** Process LandXML Element ********************* -->
<!-- **************************************************************** -->
<xsl:template match="LandXML" >

  <xsl:text>GENIO   </xsl:text>
  <xsl:value-of select="Surfaces/Surface[1]/@name"/>  <!-- Use the name of the surface as the road name -->
  <xsl:call-template name="NewLine"/>

  <xsl:text>017,RADI</xsl:text>
  <xsl:call-template name="NewLine"/>

  <xsl:if test="Alignments">  <!-- There is an Alignments element -->
    <!-- The GENIO format requires that all the easting value are positive - a negative easting  -->
    <!-- value is considered to indicate the start of a gap in the string and we don't want this -->
    <!-- to occur.  Determine the range of easting values for the master alignment and if the    -->
    <!-- values go negative determine an appropriate offset value to be applied.  The offset is  -->
    <!-- also included in the string definitions so the resultant values used in the field are   -->
    <!-- not affected.  A negative northing value indicates the end of a gap in the string so    -->
    <!-- also determine and appropriate northing offset value so that all the northing values in -->
    <!-- the GENIO strings will be positive.                                                     -->
    <xsl:variable name="posEastOffsetValue">
      <xsl:call-template name="DetermineEastingOffset"/>
    </xsl:variable>

    <xsl:variable name="posNorthOffsetValue">
      <xsl:call-template name="DetermineNorthingOffset"/>
    </xsl:variable>

    <xsl:for-each select="Alignments/Alignment[@name = $alignmentToUse]">
      <!-- The verticalDefn node-set variable holds the vertical alignment (profile) for -->
      <!-- the selected alignment (master alignment).                                    -->
      <xsl:variable name="verticalDefn">
        <xsl:call-template name="GetVerticalDefinition"/>
      </xsl:variable>

      <!-- The horizontalDefn node-set variable holds the horizontal alignment for the   -->
      <!-- selected alignment (master alignment).                                        -->
      <xsl:variable name="horizontalDefn">
        <xsl:element name="startStation">
          <xsl:call-template name="SixDecPlacesValue">
            <xsl:with-param name="value" select="@staStart"/>
          </xsl:call-template>
        </xsl:element>
        <xsl:element name="length">
          <xsl:call-template name="SixDecPlacesValue">
            <xsl:with-param name="value" select="@length"/>
          </xsl:call-template>
        </xsl:element>
        <xsl:call-template name="GetHorizontalDefinition"/>
      </xsl:variable>

      <!-- Create a stations variable that contains a sorted list of all the stations along  -->
      <!-- the alignment to be output. If the includeRegularStations variable is set to true -->
      <!-- then there will also be points at a regular 5m or 10ft interval included. The     -->
      <!-- elements in the stations node-set variable also include the hz and vt codes and   -->
      <!-- strings values to be included in the 12D string.                                  -->
      <xsl:variable name="stations">
        <xsl:call-template name="GetStations">
          <xsl:with-param name="verticalDefn" select="$verticalDefn"/>
          <xsl:with-param name="horizontalDefn" select="$horizontalDefn"/>
        </xsl:call-template>
      </xsl:variable>

      <!-- Output the 12D and 6D strings for the referenced alignment -->
      <xsl:call-template name="Output12DString">
        <xsl:with-param name="stations" select="$stations"/>
        <xsl:with-param name="verticalDefn" select="$verticalDefn"/>
        <xsl:with-param name="horizontalDefn" select="$horizontalDefn"/>
        <xsl:with-param name="posEastOffsetValue" select="$posEastOffsetValue"/>
        <xsl:with-param name="posNorthOffsetValue" select="$posNorthOffsetValue"/>
      </xsl:call-template>
    </xsl:for-each>

    <!-- Now output 3D strings for all the Surface/SourceData/Breaklines/Breakline elements -->
    <!-- We only want the strings that have a name attribute (even if it is empty) and we   -->
    <!-- don't want any string that has the same name as the selected alignment as it will  -->
    <!-- duplicate the master string.                                                       -->
    <xsl:for-each select="Surfaces/Surface[1]/SourceData/Breaklines/Breakline[@name][@name != $alignmentToUse]">
      <xsl:call-template name="OutputBreakline3DStrings">
        <xsl:with-param name="alignCount" select="position()"/>  <!-- Used as the string name index value -->
        <xsl:with-param name="posEastOffsetValue" select="$posEastOffsetValue"/>
        <xsl:with-param name="posNorthOffsetValue" select="$posNorthOffsetValue"/>
      </xsl:call-template>
    </xsl:for-each>

    <!-- Set up a node set variable referencing each Breakline string with at least 2 points (0 based) -->
    <xsl:variable name="stringRefsBreakline">
      <xsl:for-each select="Surfaces/Surface[1]/SourceData/Breaklines/Breakline[@name][@name != $alignmentToUse]">
        <xsl:variable name="points3D">
          <xsl:call-template name="Get3DListPoints">
            <xsl:with-param name="pointList3D" select="PntList3D"/>
          </xsl:call-template>
        </xsl:variable>

        <xsl:if test="count(msxsl:node-set($points3D)/point) &gt; 1">  <!-- There are at least 2 points in the string -->
          <xsl:element name="stringRef">
            <xsl:text>000 String=</xsl:text>
            <xsl:value-of select="position() - 1"/>
          </xsl:element>
        </xsl:if>
      </xsl:for-each>
    </xsl:variable>

    <xsl:variable name="breaklineStringsCount" select="count(msxsl:node-set($stringRefsBreakline)/stringRef)"/>

    <!-- Now output 3D strings for all the Contours/Contour elements (used for strings with points all at the same elevation) -->
    <xsl:for-each select="Surfaces/Surface[1]/SourceData/Contours/Contour">
      <xsl:call-template name="OutputContour3DStrings">
        <xsl:with-param name="alignCount" select="$breaklineStringsCount + position()"/>  <!-- Used as the string name index value -->
        <xsl:with-param name="posEastOffsetValue" select="$posEastOffsetValue"/>
        <xsl:with-param name="posNorthOffsetValue" select="$posNorthOffsetValue"/>
        <xsl:with-param name="elevation" select="@elev"/>
      </xsl:call-template>
    </xsl:for-each>

    <xsl:text>999</xsl:text>
    <xsl:call-template name="NewLine"/>

    <xsl:text>FINISH</xsl:text>
    <xsl:call-template name="NewLine"/>

    <!-- Set up the group definition -->
    <xsl:text>000 TRMB.NEW_GROUP</xsl:text>
    <xsl:call-template name="NewLine"/>
    <xsl:text>000 GroupID=</xsl:text>
    <xsl:value-of select="Surfaces/Surface[1]/@name"/>    <!-- Use the surface name -->
    <xsl:call-template name="NewLine"/>
    <xsl:text>000 ExcludeMasterFromXS=false</xsl:text>
    <xsl:call-template name="NewLine"/>

    <!-- Output a string reference for each Breakline string from the node set variable created earlier -->
    <xsl:for-each select="msxsl:node-set($stringRefsBreakline)/stringRef">  <!-- Output from node set -->
      <xsl:value-of select="."/>
      <xsl:call-template name="NewLine"/>
    </xsl:for-each>
    
    <!-- Now add a string reference for each Contour string with at least 2 points (0 based) -->
    <xsl:variable name="stringRefsContour">
      <xsl:for-each select="Surfaces/Surface[1]/SourceData/Contours/Contour">
        <xsl:variable name="points2D">
          <xsl:call-template name="Get2DListPoints">
            <xsl:with-param name="pointList2D" select="PntList2D"/>
          </xsl:call-template>
        </xsl:variable>

        <xsl:if test="count(msxsl:node-set($points2D)/point) &gt; 1">  <!-- There are at least 2 points in the string -->
          <xsl:element name="stringRef">
            <xsl:text>000 String=</xsl:text>
            <xsl:value-of select="number($breaklineStringsCount + position() - 1)"/>  <!-- Allow for already output Breakline strings -->
          </xsl:element>
        </xsl:if>
      </xsl:for-each>
    </xsl:variable>

    <xsl:for-each select="msxsl:node-set($stringRefsContour)/stringRef">  <!-- Output from node set -->
      <xsl:value-of select="."/>
      <xsl:call-template name="NewLine"/>
    </xsl:for-each>

    <xsl:variable name="totalStringCount" select="$breaklineStringsCount + count(msxsl:node-set($stringRefsContour)/stringRef)"/>
    
    <!-- Now add 2 further strings - 1 for the 12D string and one -->
    <!-- for the 6D string that will be created automatically     -->
    <xsl:text>000 String=</xsl:text>
    <xsl:value-of select="$totalStringCount"/>
    <xsl:call-template name="NewLine"/>
    <xsl:text>000 String=</xsl:text>
    <xsl:value-of select="$totalStringCount + 1"/>
    <xsl:call-template name="NewLine"/>
    
    <xsl:text>000 StationingMaster=</xsl:text>
    <xsl:value-of select="$totalStringCount + 1"/>
    <xsl:call-template name="NewLine"/>

    <xsl:text>000 End Group</xsl:text>
    <xsl:call-template name="NewLine"/>
  </xsl:if>

</xsl:template>


<!-- **************************************************************** -->
<!-- ********** Output MX 12D strings for master alignment ********** -->
<!-- **************************************************************** -->
<xsl:template name="Output12DString">
  <xsl:param name="stations"/>
  <xsl:param name="verticalDefn"/>
  <xsl:param name="horizontalDefn"/>
  <xsl:param name="posEastOffsetValue"/>
  <xsl:param name="posNorthOffsetValue"/>

  <xsl:text>001FORMAT(3D23.8/3D23.8/2D23.8,4A4)</xsl:text>
<!--  <xsl:text>001FORMAT(3D23.17/3D23.17/2D23.17,4A4)</xsl:text> -->
  <xsl:call-template name="NewLine"/>

  <xsl:choose>
    <xsl:when test="string-length(@name) &lt;= 4">
      <xsl:text>080,</xsl:text>
      <xsl:value-of select="@name"/>
      <xsl:text>,MCG1,5= </xsl:text>
    </xsl:when>
    <xsl:otherwise>080,GCG1,MCG1,5= </xsl:otherwise>
  </xsl:choose>
  <xsl:value-of select="format-number($posEastOffsetValue * -1.0, $DecPl1, 'Standard')"/>
  <xsl:text>, </xsl:text>
  <xsl:value-of select="format-number($posNorthOffsetValue * -1.0, $DecPl1, 'Standard')"/>
  <xsl:text>,  777712</xsl:text>
  <xsl:call-template name="NewLine"/>

  <!-- Hold the 12D string lines in a node-set variable so that the first and -->
  <!-- second lines for each point can be used again for the 6D string        -->
  <xsl:variable name="output12DLines">
    <xsl:for-each select="msxsl:node-set($stations)/station">
      <!-- If this station is within the horizontal alignment station range -->
      <xsl:if test="(. &gt;= msxsl:node-set($horizontalDefn)/startStation) and
                    (. &lt;= msxsl:node-set($horizontalDefn)/startStation + msxsl:node-set($horizontalDefn)/length)">
        <xsl:element name="lineGroup">
          <xsl:element name="line1">
            <xsl:variable name="coords">
              <xsl:call-template name="PositionAtStation">
                <xsl:with-param name="horizontalDefn" select="$horizontalDefn"/>
                <xsl:with-param name="inStation" select="."/>
              </xsl:call-template>
            </xsl:variable>

            <!-- Output the point easting value -->
            <xsl:call-template name="OutputFileValue">
              <xsl:with-param name="value" select="msxsl:node-set($coords)/east + $posEastOffsetValue"/>
            </xsl:call-template>

            <!-- Output the point northing value -->
            <xsl:call-template name="OutputFileValue">
              <xsl:with-param name="value" select="msxsl:node-set($coords)/north + $posNorthOffsetValue"/>
            </xsl:call-template>

            <!-- Output the elevation at this station -->
            <xsl:call-template name="OutputFileValue">
              <xsl:with-param name="value">
                <xsl:call-template name="ElevationAtStation">
                  <xsl:with-param name="verticalDefn" select="$verticalDefn"/>
                  <xsl:with-param name="inStation" select="."/>
                </xsl:call-template>
              </xsl:with-param>
            </xsl:call-template>
          </xsl:element>

          <xsl:element name="line2">
            <!-- Output the station value -->
            <xsl:call-template name="OutputFileValue">
              <xsl:with-param name="value" select="."/>
              <xsl:with-param name="useCoordNullVal">false</xsl:with-param>
            </xsl:call-template>

            <!-- Output the instantaneous azimuth value (in radians) -->
            <xsl:variable name="azimuth">
              <xsl:call-template name="InstantaneousAzimuth">
                <xsl:with-param name="horizontalDefn" select="$horizontalDefn"/>
                <xsl:with-param name="inStation" select="."/>
              </xsl:call-template>
            </xsl:variable>
            <xsl:call-template name="OutputFileValue">
              <xsl:with-param name="value" select="$azimuth * $Pi div 180.0"/>
              <xsl:with-param name="useCoordNullVal">false</xsl:with-param>
              <xsl:with-param name="applyDistUnitsConv">false</xsl:with-param>
            </xsl:call-template>

            <!-- Output the instantaneous radius value -->
            <xsl:call-template name="OutputFileValue">
              <xsl:with-param name="value">
                <xsl:call-template name="InstantaneousRadius">
                  <xsl:with-param name="horizontalDefn" select="$horizontalDefn"/>
                  <xsl:with-param name="inStation" select="."/>
                </xsl:call-template>
              </xsl:with-param>
              <xsl:with-param name="useCoordNullVal">false</xsl:with-param>
            </xsl:call-template>
          </xsl:element>

          <xsl:element name="line3">
            <!-- Output the instantaneous grade value -->
            <xsl:call-template name="OutputFileValue">
              <xsl:with-param name="value">
                <xsl:call-template name="GradeAtStation">
                  <xsl:with-param name="verticalDefn" select="$verticalDefn"/>
                  <xsl:with-param name="inStation" select="."/>
                </xsl:call-template>
              </xsl:with-param>
              <xsl:with-param name="useCoordNullVal">false</xsl:with-param>
              <xsl:with-param name="applyDistUnitsConv">false</xsl:with-param>
            </xsl:call-template>

            <!-- Output the MFactor value -->
            <xsl:call-template name="OutputFileValue">
              <xsl:with-param name="value">
                <xsl:call-template name="MFactorAtStation">
                  <xsl:with-param name="verticalDefn" select="$verticalDefn"/>
                  <xsl:with-param name="inStation" select="."/>
                </xsl:call-template>
              </xsl:with-param>
              <xsl:with-param name="useCoordNullVal">false</xsl:with-param>
              <xsl:with-param name="applyDistUnitsConv">false</xsl:with-param>
            </xsl:call-template>

            <!-- Now output the horizontal code -->
            <xsl:call-template name="PadRight">
              <xsl:with-param name="stringWidth" select="4"/>
              <xsl:with-param name="theString" select="@hzCode"/>
            </xsl:call-template>

            <!-- Now output the vertical code -->
            <xsl:call-template name="PadRight">
              <xsl:with-param name="stringWidth" select="4"/>
              <xsl:with-param name="theString" select="@vtCode"/>
            </xsl:call-template>

            <!-- Now output the horizontal string number - new string number for each new horizontal element -->
            <xsl:choose>
              <xsl:when test="not(@hzString) or @hzString = ''">
                <xsl:value-of select="preceding-sibling::*[@hzString != ''][1]/@hzString"/>
              </xsl:when>
              <xsl:otherwise>
                <xsl:value-of select="@hzString"/>
              </xsl:otherwise>
            </xsl:choose>

            <!-- Now output the vertical string number - new string number for each new vertical element -->
            <xsl:choose>
              <xsl:when test="not(@vtString) or @vtString = ''">
                <xsl:value-of select="preceding-sibling::*[@vtString != ''][1]/@vtString"/>
              </xsl:when>
              <xsl:otherwise>
                <xsl:value-of select="@vtString"/>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:element>
        </xsl:element>
      </xsl:if>
    </xsl:for-each>
  </xsl:variable>

  <!-- Write out all the actual 12D string lines -->
  <xsl:for-each select="msxsl:node-set($output12DLines)/lineGroup">
    <xsl:value-of select="line1"/>
    <xsl:call-template name="NewLine"/>
    <xsl:value-of select="line2"/>
    <xsl:call-template name="NewLine"/>
    <xsl:value-of select="line3"/>
    <xsl:call-template name="NewLine"/>
  </xsl:for-each>

  <!-- Now output the 'end of string' record for the 12D string -->
  <xsl:call-template name="OutputFileValue">
    <xsl:with-param name="value">0</xsl:with-param>
  </xsl:call-template>
  <xsl:call-template name="OutputFileValue">
    <xsl:with-param name="value">0</xsl:with-param>
  </xsl:call-template>
  <xsl:call-template name="OutputFileValue">
    <xsl:with-param name="value"></xsl:with-param>   <!-- Null elevation -->
  </xsl:call-template>
  <xsl:call-template name="NewLine"/>

  <xsl:call-template name="OutputFileValue">
    <xsl:with-param name="value">0</xsl:with-param>
  </xsl:call-template>
  <xsl:call-template name="OutputFileValue">
    <xsl:with-param name="value">0</xsl:with-param>
  </xsl:call-template>
  <xsl:call-template name="OutputFileValue">
    <xsl:with-param name="value">0</xsl:with-param>
  </xsl:call-template>
  <xsl:call-template name="NewLine"/>

  <xsl:call-template name="OutputFileValue">
    <xsl:with-param name="value">0</xsl:with-param>
  </xsl:call-template>
  <xsl:call-template name="OutputFileValue">
    <xsl:with-param name="value">0</xsl:with-param>
  </xsl:call-template>
  <xsl:text>                </xsl:text>    <!-- 16 spaces -->
  <xsl:call-template name="NewLine"/>

</xsl:template>


<!-- **************************************************************** -->
<!-- ********* Output MX 3D sub-strings (Breakline elements) ******** -->
<!-- **************************************************************** -->
<xsl:template name="OutputBreakline3DStrings">
  <xsl:param name="alignCount"/>
  <xsl:param name="posEastOffsetValue"/>
  <xsl:param name="posNorthOffsetValue"/>

  <!-- Get the points from the PntList3D element -->
  <xsl:variable name="points3D">
    <xsl:call-template name="Get3DListPoints">
      <xsl:with-param name="pointList3D" select="PntList3D"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:if test="count(msxsl:node-set($points3D)/point) &gt; 1">  <!-- There are at least 2 points in the string -->
    <xsl:text>001FORMAT(3D23.8)</xsl:text>
    <!--  <xsl:text>001FORMAT(3D23.17)</xsl:text> -->
    <xsl:call-template name="NewLine"/>

    <xsl:text>003,ORDR,4=1,1,2,3,</xsl:text>
    <xsl:call-template name="NewLine"/>

    <!-- Start a new string -->
    <xsl:variable name="isInterfaceString">
      <xsl:variable name="name" select="translate(@name,'abcdefghijklmnopqrstuvwxyzáéíóúñü','ABCDEFGHIJKLMNOPQRSTUVWXYZÁÉÍÓÚÑÜ')"/>
      <xsl:choose>
        <xsl:when test="($useLeftInterfaceStringName != '') and ($name != '') and ($name = $useLeftInterfaceStringName)">1</xsl:when>
        <xsl:when test="($useRightInterfaceStringName != '') and ($name != '') and ($name = $useRightInterfaceStringName)">2</xsl:when>
      </xsl:choose>
    </xsl:variable>

    <xsl:variable name="stringName">
      <xsl:choose>
        <xsl:when test="$isInterfaceString != ''">
          <xsl:text>INT</xsl:text>
          <xsl:choose>
            <xsl:when test="$isInterfaceString = 1">L</xsl:when>
            <xsl:otherwise>R</xsl:otherwise>
          </xsl:choose>
        </xsl:when>
        <xsl:otherwise>
          <xsl:choose>
            <xsl:when test="string-length(@name) &lt;= 4">
              <xsl:value-of select="@name"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:text>ST</xsl:text>
              <xsl:value-of select="format-number($alignCount, '00')"/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:text>080,</xsl:text>
    <xsl:value-of select="$stringName"/>
    <xsl:text>,5= </xsl:text>
    <xsl:value-of select="format-number($posEastOffsetValue * -1.0, $DecPl1, 'Standard')"/>
    <xsl:text>, </xsl:text>
    <xsl:value-of select="format-number($posNorthOffsetValue * -1.0, $DecPl1, 'Standard')"/>
    <xsl:text>,    7703</xsl:text>
    <xsl:call-template name="NewLine"/>

    <!-- Now write out the combined coordinates to the file -->
    <xsl:for-each select="msxsl:node-set($points3D)/point">
      <xsl:call-template name="OutputFileValue">
        <xsl:with-param name="value" select="east + $posEastOffsetValue"/>
      </xsl:call-template>
      <xsl:call-template name="OutputFileValue">
        <xsl:with-param name="value" select="north + $posNorthOffsetValue"/>
      </xsl:call-template>
      <xsl:call-template name="OutputFileValue">
        <xsl:with-param name="value" select="elev"/>
      </xsl:call-template>
      <xsl:call-template name="NewLine"/>
    </xsl:for-each>

    <!-- Now write the final end of string record -->
    <xsl:call-template name="OutputFileValue">
      <xsl:with-param name="value">0</xsl:with-param>
    </xsl:call-template>
    <xsl:call-template name="OutputFileValue">
      <xsl:with-param name="value">0</xsl:with-param>
    </xsl:call-template>
    <xsl:call-template name="OutputFileValue">
      <xsl:with-param name="value">0</xsl:with-param>
    </xsl:call-template>
    <xsl:call-template name="NewLine"/>
  </xsl:if>
</xsl:template>


<!-- **************************************************************** -->
<!-- ********** Output MX 3D sub-strings (Contour elements) ********* -->
<!-- **************************************************************** -->
<xsl:template name="OutputContour3DStrings">
  <xsl:param name="alignCount"/>
  <xsl:param name="posEastOffsetValue"/>
  <xsl:param name="posNorthOffsetValue"/>
  <xsl:param name="elevation"/>

  <!-- Get the points from the PntList3D element -->
  <xsl:variable name="points2D">
    <xsl:call-template name="Get2DListPoints">
      <xsl:with-param name="pointList2D" select="PntList2D"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:if test="count(msxsl:node-set($points2D)/point) &gt; 1">  <!-- There are at least 2 points in the string -->
    <xsl:text>001FORMAT(3D23.8)</xsl:text>
    <!--  <xsl:text>001FORMAT(3D23.17)</xsl:text> -->
    <xsl:call-template name="NewLine"/>

    <xsl:text>003,ORDR,4=1,1,2,3,</xsl:text>
    <xsl:call-template name="NewLine"/>

    <!-- Start a new string -->
    <xsl:variable name="isInterfaceString">
      <xsl:variable name="name" select="translate(@name,'abcdefghijklmnopqrstuvwxyzáéíóúñü','ABCDEFGHIJKLMNOPQRSTUVWXYZÁÉÍÓÚÑÜ')"/>
      <xsl:choose>
        <xsl:when test="($useLeftInterfaceStringName != '') and ($name != '') and ($name = $useLeftInterfaceStringName)">1</xsl:when>
        <xsl:when test="($useRightInterfaceStringName != '') and ($name != '') and ($name = $useRightInterfaceStringName)">2</xsl:when>
      </xsl:choose>
    </xsl:variable>

    <xsl:variable name="stringName">
      <xsl:choose>
        <xsl:when test="$isInterfaceString != ''">
          <xsl:text>INT</xsl:text>
          <xsl:choose>
            <xsl:when test="$isInterfaceString = 1">L</xsl:when>
            <xsl:otherwise>R</xsl:otherwise>
          </xsl:choose>
        </xsl:when>
        <xsl:otherwise>
          <xsl:choose>
            <xsl:when test="string-length(@name) &lt;= 4">
              <xsl:value-of select="@name"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:text>ST</xsl:text>
              <xsl:value-of select="format-number($alignCount, '00')"/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:text>080,</xsl:text>
    <xsl:value-of select="$stringName"/>
    <xsl:text>,5= </xsl:text>
    <xsl:value-of select="format-number($posEastOffsetValue * -1.0, $DecPl1, 'Standard')"/>
    <xsl:text>, </xsl:text>
    <xsl:value-of select="format-number($posNorthOffsetValue * -1.0, $DecPl1, 'Standard')"/>
    <xsl:text>,    7703</xsl:text>
    <xsl:call-template name="NewLine"/>

    <!-- Now write out the combined coordinates to the file -->
    <xsl:for-each select="msxsl:node-set($points2D)/point">
      <xsl:call-template name="OutputFileValue">
        <xsl:with-param name="value" select="east + $posEastOffsetValue"/>
      </xsl:call-template>
      <xsl:call-template name="OutputFileValue">
        <xsl:with-param name="value" select="north + $posNorthOffsetValue"/>
      </xsl:call-template>
      <xsl:call-template name="OutputFileValue">
        <xsl:with-param name="value" select="$elevation"/>
      </xsl:call-template>
      <xsl:call-template name="NewLine"/>
    </xsl:for-each>

    <!-- Now write the final end of string record -->
    <xsl:call-template name="OutputFileValue">
      <xsl:with-param name="value">0</xsl:with-param>
    </xsl:call-template>
    <xsl:call-template name="OutputFileValue">
      <xsl:with-param name="value">0</xsl:with-param>
    </xsl:call-template>
    <xsl:call-template name="OutputFileValue">
      <xsl:with-param name="value">0</xsl:with-param>
    </xsl:call-template>
    <xsl:call-template name="NewLine"/>
  </xsl:if>
</xsl:template>


<!-- **************************************************************** -->
<!-- ************* Return 3D List Points As A Node Set ************** -->
<!-- **************************************************************** -->
<xsl:template name="Get3DListPoints">
  <xsl:param name="pointList3D"/>

  <xsl:variable name="north" select="substring-before(normalize-space($pointList3D), ' ')"/>
  <xsl:variable name="temp1" select="substring-after(normalize-space($pointList3D), ' ')"/>
  <xsl:variable name="east" select="substring-before(normalize-space($temp1), ' ')"/>
  <xsl:variable name="temp2" select="substring-after(normalize-space($temp1), ' ')"/>
  <xsl:variable name="elev">
    <xsl:choose>
      <xsl:when test="contains($temp2, ' ')"><xsl:value-of select="substring-before($temp2, ' ')"/></xsl:when>
      <xsl:otherwise><xsl:value-of select="$temp2"/></xsl:otherwise>
    </xsl:choose>
  </xsl:variable>
  <xsl:variable name="remainder" select="substring-after($temp2, ' ')"/>

  <xsl:if test="string(number($north)) != 'NaN'">  <!-- We have an coordinate pair -->
    <xsl:element name="point">
      <xsl:element name="north">
        <xsl:value-of select="$north"/>
      </xsl:element>
      <xsl:element name="east">
        <xsl:value-of select="$east"/>
      </xsl:element>
      <xsl:element name="elev">
        <xsl:value-of select="$elev"/>
      </xsl:element>
    </xsl:element>

    <!-- Now use a recursive call to get the next pair of values -->
    <xsl:call-template name="Get3DListPoints">
      <xsl:with-param name="pointList3D" select="$remainder"/>
    </xsl:call-template>
  </xsl:if>
</xsl:template>


<!-- **************************************************************** -->
<!-- ************* Return 2D List Points As A Node Set ************** -->
<!-- **************************************************************** -->
<xsl:template name="Get2DListPoints">
  <xsl:param name="pointList2D"/>

  <xsl:variable name="north" select="substring-before(normalize-space($pointList2D), ' ')"/>
  <xsl:variable name="temp" select="substring-after(normalize-space($pointList2D), ' ')"/>
  <xsl:variable name="east">
    <xsl:choose>
      <xsl:when test="contains($temp, ' ')"><xsl:value-of select="substring-before($temp, ' ')"/></xsl:when>
      <xsl:otherwise><xsl:value-of select="$temp"/></xsl:otherwise>
    </xsl:choose>
  </xsl:variable>
  <xsl:variable name="remainder" select="substring-after($temp, ' ')"/>

  <xsl:if test="string(number($north)) != 'NaN'">  <!-- We have an coordinate pair -->
    <xsl:element name="point">
      <xsl:element name="north">
        <xsl:value-of select="$north"/>
      </xsl:element>
      <xsl:element name="east">
        <xsl:value-of select="$east"/>
      </xsl:element>
    </xsl:element>

    <!-- Now use a recursive call to get the next pair of values -->
    <xsl:call-template name="Get2DListPoints">
      <xsl:with-param name="pointList2D" select="$remainder"/>
    </xsl:call-template>
  </xsl:if>
</xsl:template>


<!-- **************************************************************** -->
<!-- ******* Return an appropriate easting offset if required ******* -->
<!-- **************************************************************** -->
<xsl:template name="DetermineEastingOffset">

  <xsl:variable name="breaklinePoints">
    <xsl:for-each select="Surfaces/Surface/SourceData/Breaklines/Breakline[@name][@name != $alignmentToUse]">
      <xsl:call-template name="Get3DListPoints">
        <xsl:with-param name="pointList3D" select="PntList3D"/>
      </xsl:call-template>
    </xsl:for-each>
  </xsl:variable>

  <xsl:variable name="eastings">
    <xsl:for-each select="Alignments/Alignment[@name = $alignmentToUse]/CoordGeom/*/*">
      <xsl:element name="east">
        <xsl:call-template name="EastVal">
          <xsl:with-param name="coordStr" select="."/>
        </xsl:call-template>
      </xsl:element>
    </xsl:for-each>

    <xsl:for-each select="msxsl:node-set($breaklinePoints)/point">
      <xsl:element name="east">
        <xsl:value-of select="east"/>
      </xsl:element>
    </xsl:for-each>
  </xsl:variable>

  <xsl:variable name="sortedVals">
    <xsl:for-each select="msxsl:node-set($eastings)/east">
      <xsl:sort data-type="number" order="ascending" select="."/>
      <xsl:copy-of select="."/>
    </xsl:for-each>
  </xsl:variable>


  <xsl:choose>
    <xsl:when test="msxsl:node-set($sortedVals)/east[1] &lt; 0.0">
      <xsl:variable name="minEast" select="(msxsl:node-set($sortedVals)/east[1] - 5000.0) * -1.0"/>  <!-- Allow an extra 5000 just in case -->
      <xsl:value-of select="round($minEast div 1000.0) * 1000.0"/>
    </xsl:when>
    <xsl:otherwise>0</xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!-- **************************************************************** -->
<!-- ******* Return an appropriate northing offset if required ****** -->
<!-- **************************************************************** -->
<xsl:template name="DetermineNorthingOffset">

  <xsl:variable name="breaklinePoints">
    <xsl:for-each select="Surfaces/Surface/SourceData/Breaklines/Breakline[@name][@name != $alignmentToUse]">
      <xsl:call-template name="Get3DListPoints">
        <xsl:with-param name="pointList3D" select="PntList3D"/>
      </xsl:call-template>
    </xsl:for-each>
  </xsl:variable>

  <xsl:variable name="northings">
    <xsl:for-each select="Alignments/Alignment[@name = $alignmentToUse]/CoordGeom/*/*">
      <xsl:element name="north">
        <xsl:call-template name="NorthVal">
          <xsl:with-param name="coordStr" select="."/>
        </xsl:call-template>
      </xsl:element>
    </xsl:for-each>

    <xsl:for-each select="msxsl:node-set($breaklinePoints)/point">
      <xsl:element name="north">
        <xsl:value-of select="north"/>
      </xsl:element>
    </xsl:for-each>
  </xsl:variable>

  <xsl:variable name="sortedVals">
    <xsl:for-each select="msxsl:node-set($northings)/north">
      <xsl:sort data-type="number" order="ascending" select="."/>
      <xsl:copy-of select="."/>
    </xsl:for-each>
  </xsl:variable>


  <xsl:choose>
    <xsl:when test="msxsl:node-set($sortedVals)/north[1] &lt; 0.0">
      <xsl:variable name="minNorth" select="(msxsl:node-set($sortedVals)/north[1] - 5000.0) * -1.0"/>  <!-- Allow an extra 5000 just in case -->
      <xsl:value-of select="round($minNorth div 1000.0) * 1000.0"/>
    </xsl:when>
    <xsl:otherwise>0</xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!-- **************************************************************** -->
<!-- ****** Return Vertical Alignment Elements In Node Variable ***** -->
<!-- **************************************************************** -->
<xsl:template name="GetVerticalDefinition">

  <!-- Now we need to locate the design profile that goes with the selected -->
  <!-- alignment.  This may be based on the name attribute associated with  -->
  <!-- the <Profile> element or the name attribute associated with the      -->
  <!-- <ProfAlign> element.                                                 -->
  <!-- If there is only one ProfAlign available then this must be the       -->
  <!-- profile to use.                                                      -->
  <xsl:variable name="profileName">
    <xsl:choose>
      <xsl:when test="count(Profile/ProfAlign) = 1">
        <xsl:value-of select="Profile/ProfAlign[1]/@name"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$alignmentToUse"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <!-- Vertical definition node set ends up with the structure:              -->
  <!-- <VertElementName> (PVI, ParaCurve, UnsymParaCurve, CircCurve)         -->
  <!--   <VertElementName/>                                                  -->
  <!--     Contains VIP station and elevation pair and associated attributes -->
  <!--   <gradeIn/>                                                          -->
  <!--   <gradeOut/>                                                         -->
  <!--   <startStn/>                                                         -->
  <!--   <endStn/>                                                           -->
  <!--   <startElev/>                                                        -->
  <!--   <endElev/>                                                          -->
  <!--   <sagSummitType>             (sag or summit)                         -->
  <!--   <circVCCentreStation/>      for circular vertical curve             -->
  <!--   <circVCCentreElevation/>    for circular vertical curve             -->
  <!--   <sagSummitStn>              for vertical curve                      -->
  <!--   <sagSummitElev>             for vertical curve                      -->
  <!-- </VertElementName>                                                    -->

  <xsl:for-each select="Profile[(@name = $profileName) or
                                   (ProfAlign/@name = $profileName)]/ProfAlign[1]/*">
    <xsl:copy>
      <xsl:copy-of select="."/>
      <xsl:variable name="gradeIn">
        <xsl:call-template name="GradeInValue"/>
      </xsl:variable>

      <xsl:variable name="gradeOut">
        <xsl:call-template name="GradeOutValue"/>
      </xsl:variable>

      <!-- Add the grade in as an element -->
      <xsl:element name="gradeIn">
        <xsl:value-of select="$gradeIn"/>
      </xsl:element>
      <!-- Add the grade out as an element -->
      <xsl:element name="gradeOut">
        <xsl:value-of select="$gradeOut"/>
      </xsl:element>

      <xsl:variable name="inLen">
        <xsl:choose>
          <xsl:when test="name(.) = 'PVI'">0</xsl:when>
          <xsl:when test="name(.) = 'ParaCurve'"><xsl:value-of select="@length div 2.0"/></xsl:when>
          <xsl:when test="name(.) = 'UnsymParaCurve'"><xsl:value-of select="@lengthIn"/></xsl:when>
          <xsl:otherwise>  <!-- Circular vertical curve -->
            <xsl:call-template name="CircVertCurveInOutLen">
              <xsl:with-param name="gradeIn" select="$gradeIn"/>
              <xsl:with-param name="gradeOut" select="$gradeOut"/>
              <xsl:with-param name="radius" select="@radius"/>
              <xsl:with-param name="returnInLen" select="'true'"/>
            </xsl:call-template>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:variable>

      <xsl:variable name="outLen">
        <xsl:choose>
          <xsl:when test="name(.) = 'PVI'">0</xsl:when>
          <xsl:when test="name(.) = 'ParaCurve'"><xsl:value-of select="@length div 2.0"/></xsl:when>
          <xsl:when test="name(.) = 'UnsymParaCurve'"><xsl:value-of select="@lengthOut"/></xsl:when>
          <xsl:otherwise>  <!-- Circular vertical curve -->
            <xsl:call-template name="CircVertCurveInOutLen">
              <xsl:with-param name="gradeIn" select="$gradeIn"/>
              <xsl:with-param name="gradeOut" select="$gradeOut"/>
              <xsl:with-param name="radius" select="@radius"/>
              <xsl:with-param name="returnInLen" select="'false'"/>
            </xsl:call-template>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:variable>

      <xsl:variable name="stationIP" select="substring-before(normalize-space(.), ' ')"/>
      <xsl:variable name="elevIP" select="substring-after(normalize-space(.), ' ')"/>

      <!-- Add the start station as an element -->
      <xsl:element name="startStn">
        <xsl:value-of select="$stationIP - $inLen"/>
      </xsl:element>

      <!-- Add the station IP as an element -->
      <xsl:element name="stationIP">
        <xsl:value-of select="$stationIP"/>
      </xsl:element>

      <!-- Add the end station as an element -->
      <xsl:element name="endStn">
        <xsl:value-of select="$stationIP + $outLen"/>
      </xsl:element>

      <!-- Add the start elevation as an element -->
      <xsl:element name="startElev">
        <xsl:value-of select="$elevIP - ($gradeIn * $inLen)"/>
      </xsl:element>

      <!-- Add the end elevation as an element -->
      <xsl:element name="endElev">
        <xsl:value-of select="$elevIP + ($gradeOut * $outLen)"/>
      </xsl:element>

      <xsl:element name="sagSummitType">
        <xsl:choose>
          <xsl:when test="$gradeOut &gt; $gradeIn">sag</xsl:when>
          <xsl:otherwise>summit</xsl:otherwise>
        </xsl:choose>
      </xsl:element>

      <!-- Add the sag/summit position if appropriate for parabolic and circular vertical curves -->
      <xsl:choose>
        <xsl:when test="(name(.) = 'ParaCurve') or (name(.) = 'UnsymParaCurve')">
          <xsl:call-template name="SagSummitPointParabola">
            <xsl:with-param name="stationIP" select="$stationIP"/>
            <xsl:with-param name="gradeIn" select="$gradeIn"/>
            <xsl:with-param name="gradeOut" select="$gradeOut"/>
            <xsl:with-param name="startStn" select="$stationIP - $inLen"/>
            <xsl:with-param name="endStn" select="$stationIP + $outLen"/>
            <xsl:with-param name="startElev" select="$elevIP - ($gradeIn * $inLen)"/>
            <xsl:with-param name="endElev" select="$elevIP + ($gradeOut * $outLen)"/>
          </xsl:call-template>
        </xsl:when>
        <xsl:when test="name(.) = 'CircCurve'">
          <!-- If a circular vertical curve compute and record the centre point station and elevation -->
          <xsl:variable name="centrePt">
            <xsl:call-template name="CircVertCurveCentrePoint">
              <xsl:with-param name="gradeIn" select="$gradeIn"/>
              <xsl:with-param name="gradeOut" select="$gradeOut"/>
              <xsl:with-param name="radius" select="@radius"/>
              <xsl:with-param name="stationIP" select="$stationIP"/>
              <xsl:with-param name="startElev" select="$elevIP - ($gradeIn * $inLen)"/>
            </xsl:call-template>
          </xsl:variable>

          <!-- Add the circular vertical curve centre station as an element -->
          <xsl:element name="circVCCentreStation">
            <xsl:value-of select="msxsl:node-set($centrePt)/centreStn"/>
          </xsl:element>

          <!-- Add the circular vertical curve centre elevation as an element -->
          <xsl:element name="circVCCentreElevation">
            <xsl:value-of select="msxsl:node-set($centrePt)/centreElev"/>
          </xsl:element>

          <xsl:call-template name="SagSummitPointCircularVC">
            <xsl:with-param name="centrePtStn" select="msxsl:node-set($centrePt)/centreStn"/>
            <xsl:with-param name="centrePtElev" select="msxsl:node-set($centrePt)/centreElev"/>
            <xsl:with-param name="radius" select="@radius"/>
            <xsl:with-param name="startStn" select="$stationIP - $inLen"/>
            <xsl:with-param name="endStn" select="$stationIP + $outLen"/>
            <xsl:with-param name="gradeIn" select="$gradeIn"/>
            <xsl:with-param name="gradeOut" select="$gradeOut"/>
          </xsl:call-template>
        </xsl:when>
      </xsl:choose>

    </xsl:copy>
  </xsl:for-each>

</xsl:template>


<!-- **************************************************************** -->
<!-- ***** Return Horizontal Alignment Elements In Node Variable **** -->
<!-- **************************************************************** -->
<xsl:template name="GetHorizontalDefinition">

  <!-- Horizontal definition node set ends up with the structure:              -->
  <!-- <startStation> element - start station for horizontal alignment         -->
  <!-- <length> element - length of horizontal alignment                       -->
  <!-- multiple <Line>, <Curve> or <Spiral> elements - these elements are      -->
  <!--    copies of the CoordGeom elements for the selected alignment          -->
  <!-- in addition each <Line>, <Curve> or <Spiral> element has the following  -->
  <!-- elements added to it:                                                   -->
  <!--   <length>    length of the element                                     -->
  <!--   <startStn>  start station for the element                             -->
  <!--   <endStn>    end station for the element                               -->
  <!-- the <Curve> and <Spiral> elements also have the following elements      -->
  <!-- added to them:                                                          -->
  <!--   <inAzimuth>  start azimuth for element in decimal degrees             -->
  <!--   <outAzimuth> end azimuth for element in decimal degrees               -->
  <!-- the <Spiral> elements also have the following element added to them:    -->
  <!--   <spiralType> set to either EntrySpiral or ExitSpiral                  -->

  <!-- Set up a node-set variable with all the individual element lengths -->
  <xsl:variable name="elementLengths">
    <xsl:element name="lengths">
      <xsl:attribute name="startStn"><xsl:value-of select="@staStart"/></xsl:attribute>
      <xsl:for-each select="CoordGeom/*">
        <xsl:element name="length">
          <xsl:choose>
            <xsl:when test="string(number(@length)) = 'NaN'">
              <xsl:choose>
                <xsl:when test="name(.) = 'Line'">
                  <xsl:variable name="coordDeltas">
                    <xsl:call-template name="ComputeDeltaCoords">
                      <xsl:with-param name="startCoord" select="Start"/>
                      <xsl:with-param name="endCoord" select="End"/>
                    </xsl:call-template>
                  </xsl:variable>
                  <xsl:call-template name="InverseDistance">
                    <xsl:with-param name="deltaN" select="msxsl:node-set($coordDeltas)/deltaN"/>
                    <xsl:with-param name="deltaE" select="msxsl:node-set($coordDeltas)/deltaE"/>
                  </xsl:call-template>
                </xsl:when>
                <xsl:when test="name(.) = 'Curve'">
                  <xsl:call-template name="ComputedArcLength">
                    <xsl:with-param name="startCoord" select="Start"/>
                    <xsl:with-param name="centreCoord" select="Center"/>
                    <xsl:with-param name="endCoord" select="End"/>
                    <xsl:with-param name="direction" select="@rot"/>
                  </xsl:call-template>
                </xsl:when>
              </xsl:choose>
            </xsl:when>
            <xsl:otherwise>
              <xsl:value-of select="@length"/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:element>
      </xsl:for-each>
    </xsl:element>
  </xsl:variable>

  <xsl:for-each select="CoordGeom/*">
    <xsl:variable name="currPos" select="position()"/>
    <!-- Don't include zero length lines -->
    <xsl:if test="(name(.) != 'Line') or ((name(.) = 'Line') and (msxsl:node-set($elementLengths)/lengths/length[$currPos] != 0))">
      <xsl:copy>
        <xsl:copy-of select="* | @*"/>

        <xsl:if test="name(.) = 'Curve'">
          <xsl:element name="inAzimuth">
            <xsl:variable name="azToStart">
              <xsl:variable name="coordDeltas">
                <xsl:call-template name="ComputeDeltaCoords">
                  <xsl:with-param name="startCoord" select="Center"/>
                  <xsl:with-param name="endCoord" select="Start"/>
                </xsl:call-template>
              </xsl:variable>
              <xsl:call-template name="InverseAzimuth">
                <xsl:with-param name="deltaN" select="msxsl:node-set($coordDeltas)/deltaN"/>
                <xsl:with-param name="deltaE" select="msxsl:node-set($coordDeltas)/deltaE"/>
              </xsl:call-template>
            </xsl:variable>
            <xsl:choose>
              <xsl:when test="@rot = 'cw'">
                <xsl:call-template name="NormalisedAngle">
                  <xsl:with-param name="angle" select="$azToStart + 90.0"/>
                </xsl:call-template>
              </xsl:when>
              <xsl:otherwise>
                <xsl:call-template name="NormalisedAngle">
                  <xsl:with-param name="angle" select="$azToStart - 90.0"/>
                </xsl:call-template>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:element>

          <xsl:element name="outAzimuth">
            <xsl:variable name="azToEnd">
              <xsl:variable name="coordDeltas">
                <xsl:call-template name="ComputeDeltaCoords">
                  <xsl:with-param name="startCoord" select="Center"/>
                  <xsl:with-param name="endCoord" select="End"/>
                </xsl:call-template>
              </xsl:variable>
              <xsl:call-template name="InverseAzimuth">
                <xsl:with-param name="deltaN" select="msxsl:node-set($coordDeltas)/deltaN"/>
                <xsl:with-param name="deltaE" select="msxsl:node-set($coordDeltas)/deltaE"/>
              </xsl:call-template>
            </xsl:variable>
            <xsl:choose>
              <xsl:when test="@rot = 'cw'">
                <xsl:call-template name="NormalisedAngle">
                  <xsl:with-param name="angle" select="$azToEnd + 90.0"/>
                </xsl:call-template>
              </xsl:when>
              <xsl:otherwise>
                <xsl:call-template name="NormalisedAngle">
                  <xsl:with-param name="angle" select="$azToEnd - 90.0"/>
                </xsl:call-template>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:element>
        </xsl:if>

        <xsl:if test="name(.) = 'Spiral'">
          <xsl:element name="spiralType">
            <xsl:choose>
              <xsl:when test="@radiusStart = 'INF'">EntrySpiral</xsl:when>
              <xsl:when test="@radiusEnd = 'INF'">ExitSpiral</xsl:when>
              <xsl:otherwise>
                <xsl:choose>
                  <xsl:when test="@radiusStart &gt; @radiusEnd">EntrySpiral</xsl:when>
                  <xsl:otherwise>ExitSpiral</xsl:otherwise>
                </xsl:choose>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:element>
          <xsl:element name="inAzimuth">
            <xsl:variable name="coordDeltas">
              <xsl:call-template name="ComputeDeltaCoords">
                <xsl:with-param name="startCoord" select="Start"/>
                <xsl:with-param name="endCoord" select="PI"/>   <!-- Note: The spiral PI is the intersection point of the actual start and end azimuths of the spiral -->
              </xsl:call-template>
            </xsl:variable>
            <xsl:call-template name="InverseAzimuth">
              <xsl:with-param name="deltaN" select="msxsl:node-set($coordDeltas)/deltaN"/>
              <xsl:with-param name="deltaE" select="msxsl:node-set($coordDeltas)/deltaE"/>
            </xsl:call-template>
          </xsl:element>
          <xsl:element name="outAzimuth">
            <xsl:variable name="coordDeltas">
              <xsl:call-template name="ComputeDeltaCoords">
                <xsl:with-param name="startCoord" select="PI"/> <!-- Note: The spiral PI is the intersection point of the actual start and end azimuths of the spiral -->
                <xsl:with-param name="endCoord" select="End"/>
              </xsl:call-template>
            </xsl:variable>
            <xsl:call-template name="InverseAzimuth">
              <xsl:with-param name="deltaN" select="msxsl:node-set($coordDeltas)/deltaN"/>
              <xsl:with-param name="deltaE" select="msxsl:node-set($coordDeltas)/deltaE"/>
            </xsl:call-template>
          </xsl:element>
        </xsl:if>

        <!-- Now insert the element length and the start and end stations for the element -->
        <xsl:variable name="length">
          <xsl:value-of select="msxsl:node-set($elementLengths)/lengths/length[$currPos]"/>
        </xsl:variable>

        <xsl:variable name="startStn">
          <xsl:value-of select="msxsl:node-set($elementLengths)/lengths/@startStn + sum(msxsl:node-set($elementLengths)/lengths/length[position() &lt; $currPos])"/>
        </xsl:variable>

        <xsl:element name="length">
          <xsl:call-template name="SixDecPlacesValue">
            <xsl:with-param name="value" select="$length"/>
          </xsl:call-template>
        </xsl:element>

        <xsl:element name="startStn">
          <xsl:call-template name="SixDecPlacesValue">
            <xsl:with-param name="value" select="$startStn"/>
          </xsl:call-template>
        </xsl:element>

        <xsl:element name="endStn">
          <xsl:call-template name="SixDecPlacesValue">
            <xsl:with-param name="value" select="$startStn + $length"/>
          </xsl:call-template>
        </xsl:element>
      </xsl:copy>
    </xsl:if>
  </xsl:for-each>
</xsl:template>


<!-- **************************************************************** -->
<!-- ******** Return the value truncated to 6 decimal places ******** -->
<!-- **************************************************************** -->
<xsl:template name="SixDecPlacesValue">
  <xsl:param name="value"/>
  
  <xsl:choose>
    <xsl:when test="contains($value, '.')">
      <xsl:value-of select="concat(substring-before($value, '.'), '.', substring(substring-after($value, '.'), 1, 6))"/>
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="concat($value, '.000000')"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!-- **************************************************************** -->
<!-- ********* Return sorted list of all stations for output ******** -->
<!-- **************************************************************** -->
<xsl:template name="GetStations">
  <xsl:param name="verticalDefn"/>
  <xsl:param name="horizontalDefn"/>
  
  <xsl:variable name="rawStations">
    <xsl:for-each select="msxsl:node-set($horizontalDefn)/*[position() &gt; 2]">
      <!-- Deal with the startStn for the element -->
      <xsl:element name="station">
        <!-- Add an attribute indicating the horizontal code - effectively the horizontal point type -->
        <xsl:attribute name="hzCode">
          <xsl:choose>
            <xsl:when test="position() = 1">
              <xsl:choose>
                <xsl:when test="name(.) = 'Curve'">PBC</xsl:when>
                <xsl:otherwise>PBT</xsl:otherwise>
              </xsl:choose>
            </xsl:when>
            <xsl:when test="(name(.) = 'Line') and (name(preceding-sibling::*[1]) = 'Curve')">PT</xsl:when>
            <xsl:when test="(name(.) = 'Line') and (name(preceding-sibling::*[1]) = 'Spiral') and (preceding-sibling::*[1]/spiralType = 'ExitSpiral')">ST</xsl:when>
            <xsl:when test="(name(.) = 'Curve') and (name(preceding-sibling::*[1]) = 'Line')">PC</xsl:when>
            <xsl:when test="(name(.) = 'Curve') and (name(preceding-sibling::*[1]) = 'Spiral')">SC</xsl:when>
            <xsl:when test="(name(.) = 'Curve') and (name(preceding-sibling::*[1]) = 'Curve') and (@rot = preceding-sibling::*[1]/@rot)">PCC</xsl:when>
            <xsl:when test="(name(.) = 'Curve') and (name(preceding-sibling::*[1]) = 'Curve') and (@rot != preceding-sibling::*[1]/@rot)">PRC</xsl:when>
            <xsl:when test="(name(.) = 'Spiral') and (spiralType = 'EntrySpiral') and (name(preceding-sibling::*[1]) = 'Line')">TS</xsl:when>
            <xsl:when test="(name(.) = 'Spiral') and (name(preceding-sibling::*[1]) = 'Curve')">CS</xsl:when>
            <xsl:when test="(name(.) = 'Spiral') and (name(preceding-sibling::*[1]) = 'Spiral')">SS</xsl:when>
            <xsl:otherwise>    </xsl:otherwise>
          </xsl:choose>
        </xsl:attribute>
        <xsl:attribute name="hzString">
          <xsl:call-template name="PadLeft">
            <xsl:with-param name="stringWidth" select="4"/>
            <xsl:with-param name="theString" select="position()"/>
          </xsl:call-template>
        </xsl:attribute>
        <xsl:call-template name="SixDecPlacesValue">
          <xsl:with-param name="value" select="startStn"/>
        </xsl:call-template>
      </xsl:element>

      <!-- Deal with the endStn for the element -->
      <xsl:element name="station">
        <!-- Add an attribute indicating the horizontal code - effectively the horizontal point type -->
        <xsl:attribute name="hzCode">
          <xsl:choose>
            <xsl:when test="position() = last()">
              <xsl:choose>
                <xsl:when test="name(.) = 'Curve'">PAC </xsl:when>
                <xsl:otherwise>PAT </xsl:otherwise>
              </xsl:choose>
            </xsl:when>
            <xsl:when test="(name(.) = 'Line') and (name(following-sibling::*[1]) = 'Curve')">PC</xsl:when>
            <xsl:when test="(name(.) = 'Curve') and (name(following-sibling::*[1]) = 'Line')">PT</xsl:when>
            <xsl:when test="(name(.) = 'Curve') and (name(following-sibling::*[1]) = 'Spiral')">CS</xsl:when>
            <xsl:when test="(name(.) = 'Curve') and (name(following-sibling::*[1]) = 'Curve') and (@rot = following-sibling::*[1]/@rot)">PCC</xsl:when>
            <xsl:when test="(name(.) = 'Curve') and (name(following-sibling::*[1]) = 'Curve') and (@rot != following-sibling::*[1]/@rot)">PCC</xsl:when>
            <xsl:when test="(name(.) = 'Spiral') and (spiralType = 'ExitSpiral') and (name(following-sibling::*[1]) = 'Line')">ST</xsl:when>
            <xsl:when test="(name(.) = 'Spiral') and (name(following-sibling::*[1]) = 'Curve')">SC</xsl:when>
            <xsl:when test="(name(.) = 'Spiral') and (name(following-sibling::*[1]) = 'Spiral')">ST</xsl:when>
            <xsl:otherwise>    </xsl:otherwise>
          </xsl:choose>
        </xsl:attribute>
        <xsl:attribute name="hzString">
          <xsl:call-template name="PadLeft">
            <xsl:with-param name="stringWidth" select="4"/>
            <xsl:with-param name="theString" select="position()"/>
          </xsl:call-template>
        </xsl:attribute>
        <xsl:call-template name="SixDecPlacesValue">
          <xsl:with-param name="value" select="endStn"/>
        </xsl:call-template>
      </xsl:element>
    </xsl:for-each>
    
    <xsl:for-each select="msxsl:node-set($verticalDefn)/*">
      <xsl:variable name="prevElementDist" select="concat(substring('-',2 - ((startStn - preceding-sibling::*[1]/endStn) &lt; 0)), '1') * (startStn - preceding-sibling::*[1]/endStn)"/>
      <xsl:variable name="nextElementDist" select="concat(substring('-',2 - ((endStn - following-sibling::*[1]/startStn) &lt; 0)), '1') * (endStn - following-sibling::*[1]/startStn)"/>
      <!-- Only include PVI elements if they are not coincident with the start or end of another element -->
      <xsl:if test="(name(.) != 'PVI') or (position() = 1) or (position() = last()) or
                    ((name(.) = 'PVI') and ($nextElementDist &gt; 0.0001) and ($prevElementDist &gt; 0.0001))">
        <!-- Output startStn element -->
        <xsl:element name="station">
          <xsl:attribute name="vtCode">
            <xsl:choose>
              <xsl:when test="position() = 1">
                <xsl:choose>
                  <xsl:when test="name(.) = 'PVI'">PBT</xsl:when>
                  <xsl:otherwise>PBC</xsl:otherwise>
                </xsl:choose>
              </xsl:when>
              <xsl:when test="position() = last()">
                <xsl:choose>
                  <xsl:when test="name(.) = 'PVI'">PAT</xsl:when>
                  <xsl:otherwise>PAC</xsl:otherwise>
                </xsl:choose>
              </xsl:when>
              <xsl:when test="($prevElementDist &lt; 0.0001) and
                              (name(.) != 'PVI') and (name(preceding-sibling::*[1]) != 'PVI')"> <!-- Adjacent vert curves -->
                <xsl:choose>
                  <xsl:when test="sagSummitType = preceding-sibling::*[1]/sagSummitType">PCC</xsl:when>
                  <xsl:otherwise>PRC</xsl:otherwise>
                </xsl:choose>
              </xsl:when>
              <xsl:when test="(name(.) = 'PVI') and ($nextElementDist &gt; 0.0001)">TT</xsl:when>   <!-- Just a vertical IP point -->
              <xsl:when test="name(.) != 'PVI'">PC</xsl:when>  <!-- Start of vertical curve -->
            </xsl:choose>
          </xsl:attribute>

          <xsl:attribute name="vtString">
            <xsl:call-template name="PadLeft">
              <xsl:with-param name="stringWidth" select="4"/>
              <xsl:with-param name="theString" select="position()"/>
            </xsl:call-template>
          </xsl:attribute>

          <xsl:call-template name="SixDecPlacesValue">
            <xsl:with-param name="value" select="startStn"/>
          </xsl:call-template>
        </xsl:element>
      </xsl:if>

      <xsl:if test="name(.) != 'PVI'">  <!-- Output the Vertical IP and end point (plus sag/summit point if required -->
        <xsl:element name="station">
          <xsl:attribute name="vtCode">  <!-- Vertical IP for a vertical curve -->
            <xsl:choose>
              <xsl:when test="stationIP = sagSummitStn">VMX</xsl:when>
              <xsl:otherwise>VM</xsl:otherwise>
            </xsl:choose>
          </xsl:attribute>

          <xsl:attribute name="vtString">
            <xsl:call-template name="PadLeft">
              <xsl:with-param name="stringWidth" select="4"/>
              <xsl:with-param name="theString" select="position()"/>
            </xsl:call-template>
          </xsl:attribute>

          <xsl:call-template name="SixDecPlacesValue">
            <xsl:with-param name="value" select="stationIP"/>
          </xsl:call-template>
        </xsl:element>

        <xsl:if test="sagSummitStn and sagSummitElev and (stationIP != sagSummitStn)">
          <xsl:element name="station">
            <xsl:attribute name="vtCode">VX</xsl:attribute>

            <xsl:attribute name="vtString">
              <xsl:call-template name="PadLeft">
                <xsl:with-param name="stringWidth" select="4"/>
                <xsl:with-param name="theString" select="position()"/>
              </xsl:call-template>
            </xsl:attribute>

            <xsl:call-template name="SixDecPlacesValue">
              <xsl:with-param name="value" select="sagSummitStn"/>
            </xsl:call-template>
          </xsl:element>
        </xsl:if>

        <xsl:element name="station">
          <xsl:attribute name="vtCode">PT</xsl:attribute>
          <xsl:attribute name="vtString">
            <xsl:call-template name="PadLeft">
              <xsl:with-param name="stringWidth" select="4"/>
              <xsl:with-param name="theString" select="position()"/>
            </xsl:call-template>
          </xsl:attribute>

          <xsl:call-template name="SixDecPlacesValue">
            <xsl:with-param name="value" select="endStn"/>
          </xsl:call-template>
        </xsl:element>
      </xsl:if>
    </xsl:for-each>
    
    <!-- Create an element for the specified start station if needed -->
    <xsl:if test="(string(number($specifiedStartStation)) != 'NaN') and ($specifiedStartStation &gt; msxsl:node-set($horizontalDefn)/startStation)">
      <xsl:element name="station">
        <xsl:value-of select="$specifiedStartStation"/>
      </xsl:element>
    </xsl:if>

    <!-- Create an element for the specified end station if needed -->
    <xsl:if test="(string(number($specifiedEndStation)) != 'NaN') and ($specifiedEndStation &lt; (msxsl:node-set($horizontalDefn)/startStation + msxsl:node-set($horizontalDefn)/length))">
      <xsl:element name="station">
        <xsl:value-of select="$specifiedEndStation"/>
      </xsl:element>
    </xsl:if>

    <xsl:if test="$includeRegularStations = 'true'">
      <!-- Add in regular stations along alignment -->
      <!-- First determine the appropriate regular start station based on the interval -->
      <xsl:variable name="startStn">
        <xsl:call-template name="GetInitialStation">
          <xsl:with-param name="startStn" select="msxsl:node-set($horizontalDefn)/*[1]"/>
          <xsl:with-param name="interval" select="$regularStnInterval"/>
        </xsl:call-template>
      </xsl:variable>
      
      <xsl:call-template name="GenerateRegularStations">
        <xsl:with-param name="startStn" select="$startStn"/>
        <xsl:with-param name="endStn" select="msxsl:node-set($horizontalDefn)/*[1] + msxsl:node-set($horizontalDefn)/*[2]"/>
        <xsl:with-param name="interval" select="$regularStnInterval"/>  <!-- Add at specified interval -->
      </xsl:call-template>
    </xsl:if>
  </xsl:variable>

  <xsl:variable name="sortedStations">
    <xsl:for-each select="msxsl:node-set($rawStations)/station">
      <xsl:sort data-type="number" order="ascending" select="."/>
      <xsl:sort data-type="number" order="descending" select="number(@hzString)"/>
      <xsl:if test="string(number(.)) != 'NaN'">
        <xsl:element name="station">
          <xsl:copy-of select="@*"/>
          <xsl:value-of select="."/>
        </xsl:element>
      </xsl:if>
    </xsl:for-each>
  </xsl:variable>
  
  <xsl:for-each select="msxsl:node-set($sortedStations)/station">
    <xsl:variable name="stnVal" select="."/>
    <xsl:variable name="stnVtCode">
      <xsl:choose>
        <xsl:when test="@vtCode">
          <xsl:value-of select="@vtCode"/>
        </xsl:when>
        <xsl:otherwise>  <!-- Grab code from following record if same station -->
          <xsl:value-of select="following-sibling::*[number(.) = $stnVal][@vtCode][1]/@vtCode"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="stnHzCode">
      <xsl:choose>
        <xsl:when test="@hzCode">
          <xsl:value-of select="@hzCode"/>
        </xsl:when>
        <xsl:otherwise>  <!-- Grab code from following record if same station -->
          <xsl:value-of select="following-sibling::*[number(.) - $stnVal][@hzCode][1]/@hzCode"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="stnVtString">
      <xsl:choose>
        <xsl:when test="@vtString">
          <xsl:value-of select="@vtString"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:choose>  <!-- Try to get the preceding vtString but if none get the next -->
            <xsl:when test="preceding-sibling::*[@vtString][1]/@vtString != ''">
              <xsl:value-of select="preceding-sibling::*[@vtString][1]/@vtString"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:value-of select="following-sibling::*[@vtString][1]/@vtString"/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:if test="count(preceding-sibling::*[number(.) = $stnVal]) = 0">
      <xsl:element name="station">
        <xsl:attribute name="hzCode">
          <xsl:value-of select="$stnHzCode"/>
        </xsl:attribute>
        <xsl:attribute name="vtCode">
          <xsl:value-of select="$stnVtCode"/>
        </xsl:attribute>
        <xsl:attribute name="hzString">
          <xsl:value-of select="@hzString"/>
        </xsl:attribute>
        <xsl:attribute name="vtString">
          <xsl:value-of select="$stnVtString"/>
        </xsl:attribute>
        <xsl:value-of select="."/>
      </xsl:element>
    </xsl:if>
  </xsl:for-each>
</xsl:template>


<!-- **************************************************************** -->
<!-- **** Return 'rounded' interval station based on supplied stn *** -->
<!-- **************************************************************** -->
<xsl:template name="GetInitialStation">
  <xsl:param name="startStn"/>
  <xsl:param name="interval"/>

  <xsl:variable name="temp" select="$startStn mod $interval"/>
  <xsl:choose>
    <xsl:when test="$temp = 0"><xsl:value-of select="$startStn"/></xsl:when>
    <xsl:otherwise>
      <xsl:choose>
        <xsl:when test="$startStn &gt; 0">
          <xsl:value-of select="$startStn - $temp"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$startStn - ($interval + $temp)"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!-- **************************************************************** -->
<!-- **************** Return a list of regular stations ************* -->
<!-- **************************************************************** -->
<xsl:template name="GenerateRegularStations">
  <xsl:param name="startStn"/>
  <xsl:param name="endStn"/>
  <xsl:param name="interval"/>

  <xsl:if test="$startStn &lt; $endStn">
    <xsl:element name="station">
      <xsl:value-of select="$startStn"/>
    </xsl:element>
    
    <!-- Recurse the function to step along alignment -->
    <xsl:call-template name="GenerateRegularStations">
      <xsl:with-param name="startStn" select="$startStn + $interval"/>
      <xsl:with-param name="endStn" select="$endStn"/>
      <xsl:with-param name="interval" select="$interval"/>
    </xsl:call-template>
  </xsl:if>
    
</xsl:template>


<!-- **************************************************************** -->
<!-- ******************* Compute the Grade In Value ***************** -->
<!-- **************************************************************** -->
<xsl:template name="GradeInValue">
  <xsl:choose>
    <xsl:when test="position() = 1"></xsl:when>  <!-- Return null value -->
    <xsl:otherwise>
      <xsl:variable name="prevStn">
        <xsl:for-each select="preceding-sibling::*[1]">
          <xsl:value-of select="substring-before(normalize-space(.), ' ')"/>
        </xsl:for-each>
      </xsl:variable>
      <xsl:variable name="prevElev">
        <xsl:for-each select="preceding-sibling::*[1]">
          <xsl:value-of select="substring-after(normalize-space(.), ' ')"/>
        </xsl:for-each>
      </xsl:variable>
      <xsl:value-of select="(substring-after(normalize-space(.), ' ') - $prevElev) div (substring-before(normalize-space(.), ' ') - $prevStn)"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!-- **************************************************************** -->
<!-- ****************** Compute the Grade Out Value ***************** -->
<!-- **************************************************************** -->
<xsl:template name="GradeOutValue">

  <xsl:choose>
    <xsl:when test="position() = last()"></xsl:when>  <!-- Return null value -->
    <xsl:otherwise>
      <xsl:variable name="nextStn">
        <xsl:for-each select="following-sibling::*[1]">
          <xsl:value-of select="substring-before(normalize-space(.), ' ')"/>
        </xsl:for-each>
      </xsl:variable>
      <xsl:variable name="nextElev">
        <xsl:for-each select="following-sibling::*[1]">
          <xsl:value-of select="substring-after(normalize-space(.), ' ')"/>
        </xsl:for-each>
      </xsl:variable>
      <xsl:value-of select="($nextElev - substring-after(normalize-space(.), ' ')) div ($nextStn - substring-before(normalize-space(.), ' '))"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

  
<!-- **************************************************************** -->
<!-- ** Return The Sag/Summit Point For A Parabolic Vertical Curve ** -->
<!-- **************************************************************** -->
<xsl:template name="SagSummitPointParabola">
  <xsl:param name="stationIP"/>
  <xsl:param name="gradeIn"/>
  <xsl:param name="gradeOut"/>
  <xsl:param name="startStn"/>
  <xsl:param name="endStn"/>
  <xsl:param name="startElev"/>
  <xsl:param name="endElev"/>

  <xsl:variable name="curveLen" select="$endStn - $startStn"/>
  <xsl:variable name="lenIn" select="$stationIP - $startStn"/>
  <xsl:variable name="lenOut" select="$endStn - $stationIP"/>
  
  <xsl:variable name="sagSummitStn">
    <xsl:choose>
      <xsl:when test="$lenIn = $lenOut"> <!-- Symmetric vertical curve -->
        <xsl:value-of select="$startStn - $curveLen * ($gradeIn div ($gradeOut - $gradeIn))"/>
      </xsl:when>

      <xsl:when test="$lenIn != $lenOut">  <!-- Asymmetric vertical curve -->
        <xsl:variable name="symCurveLen">
          <xsl:value-of select="($lenIn + $lenOut) * ($lenIn div $lenOut)"/>
        </xsl:variable>

        <xsl:variable name="tempStnSagSum">
          <xsl:value-of select="$startStn - $symCurveLen * ($gradeIn div ($gradeOut - $gradeIn))"/>
        </xsl:variable>

        <!-- Is SagSummit station in the entry or exit curve? -->
        <xsl:choose>
          <xsl:when test="$tempStnSagSum &gt; $stationIP">
            <xsl:variable name="newSymCurveLen">
              <xsl:value-of select="($lenIn + $lenOut) * ($lenOut div $lenIn)"/>
            </xsl:variable>

            <xsl:value-of select="$endStn - $newSymCurveLen * ($gradeOut div ($gradeOut - $gradeIn))"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="$tempStnSagSum"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
    </xsl:choose>
  </xsl:variable>
  
  <!-- If the computed sag/summit station is outside the range of the vertical curve return an empty string -->
  <xsl:if test="($sagSummitStn &lt;= $endStn) and ($sagSummitStn &gt;= $startStn)">
    <xsl:variable name="sagSummitElev">
      <xsl:call-template name="ParabolaPointElevation">
        <xsl:with-param name="stationIP" select="$stationIP"/>
        <xsl:with-param name="gradeIn" select="$gradeIn"/>
        <xsl:with-param name="gradeOut" select="$gradeOut"/>
        <xsl:with-param name="startStn" select="$startStn"/>
        <xsl:with-param name="endStn" select="$endStn"/>
        <xsl:with-param name="startElev" select="$startElev"/>
        <xsl:with-param name="endElev" select="$endElev"/>
        <xsl:with-param name="lenIn" select="$lenIn"/>
        <xsl:with-param name="lenOut" select="$lenOut"/>
        <xsl:with-param name="ptStn" select="$sagSummitStn"/>
      </xsl:call-template>
    </xsl:variable>
      
    <xsl:element name="sagSummitStn">
      <xsl:value-of select="$sagSummitStn"/>
    </xsl:element>

    <xsl:element name="sagSummitElev">
      <xsl:value-of select="$sagSummitElev"/>
    </xsl:element>
  </xsl:if>
</xsl:template>


<!-- **************************************************************** -->
<!-- ********* Return The Elevation Of A Point On A Parabola ******** -->
<!-- **************************************************************** -->
<xsl:template name="ParabolaPointElevation">
  <xsl:param name="stationIP"/>
  <xsl:param name="gradeIn"/>
  <xsl:param name="gradeOut"/>
  <xsl:param name="startStn"/>
  <xsl:param name="endStn"/>
  <xsl:param name="startElev"/>
  <xsl:param name="endElev"/>
  <xsl:param name="lenIn"/>
  <xsl:param name="lenOut"/>
  <xsl:param name="ptStn"/>

  <xsl:variable name="aOne">
    <xsl:choose>
      <xsl:when test="$lenIn + $lenOut &gt; 0.0">
        <xsl:value-of select="($gradeOut - $gradeIn) * $lenOut div ($lenIn + $lenOut)"/>
      </xsl:when>
      <xsl:otherwise>0.0</xsl:otherwise>  <!-- avoid divide by zero -->
    </xsl:choose>
  </xsl:variable>
  
  <xsl:variable name="aTwo">
    <xsl:choose>
      <xsl:when test="$lenIn + $lenOut &gt; 0.0">
        <xsl:value-of select="($gradeOut - $gradeIn) * $lenIn div ($lenIn + $lenOut)"/>
      </xsl:when>
      <xsl:otherwise>0.0</xsl:otherwise>  <!-- avoid divide by zero -->
    </xsl:choose>
  </xsl:variable>

  <xsl:variable name="xE" select="$ptStn - $startStn"/>
  <xsl:variable name="xX" select="$ptStn - $endStn"/>

  <xsl:choose>
    <xsl:when test="$ptStn &lt; $stationIP">
      <xsl:choose>
        <xsl:when test="$lenIn != 0.0">
          <xsl:value-of select="$startElev + $gradeIn * $xE + $aOne * $xE * $xE div (2.0 * $lenIn)"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$startElev + $gradeIn * $xE"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:when>

    <xsl:otherwise>
      <xsl:choose>
        <xsl:when test="$lenOut != 0.0">
          <xsl:value-of select="$endElev + $gradeOut * $xX + $aTwo * $xX * $xX div (2.0 * $lenOut)"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$endElev + $gradeOut * $xX"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!-- **************************************************************** -->
<!-- *** Return The Elevation Of A Point On A Circular Vert Curve *** -->
<!-- **************************************************************** -->
<xsl:template name="CircularVertCurvePointElevation">
  <xsl:param name="startStn"/>
  <xsl:param name="startElev"/>
  <xsl:param name="centreStn"/>
  <xsl:param name="centreElev"/>
  <xsl:param name="radius"/>
  <xsl:param name="ptStn"/>

  <xsl:variable name="incAngle" select="($ptStn - $startStn) div $radius"/>
  
  <xsl:variable name="azToStart">
    <xsl:variable name="degVal">
      <xsl:variable name="coordDeltas">
        <xsl:call-template name="ComputeDeltaCoords">
          <xsl:with-param name="startCoord" select="concat($centreStn, ' ', $centreElev)"/>
          <xsl:with-param name="endCoord" select="concat($startStn, ' ', $startElev)"/>
        </xsl:call-template>
      </xsl:variable>
      <xsl:call-template name="InverseAzimuth">
        <xsl:with-param name="deltaN" select="msxsl:node-set($coordDeltas)/deltaN"/>
        <xsl:with-param name="deltaE" select="msxsl:node-set($coordDeltas)/deltaE"/>
      </xsl:call-template>
    </xsl:variable>
    <!-- Convert returned degrees value to radians -->
    <xsl:value-of select="$degVal * $Pi div 180.0"/>
  </xsl:variable>
  
  <xsl:variable name="azToPt">
    <xsl:choose>
      <xsl:when test="$centreElev &gt; $startElev">
        <xsl:value-of select="$azToStart + $incAngle"/>
      </xsl:when>
      <xsl:otherwise><xsl:value-of select="$azToStart - $incAngle"/></xsl:otherwise>
    </xsl:choose>
  </xsl:variable>
  
  <xsl:variable name="sinAzToPt">
    <xsl:call-template name="Sine">
      <xsl:with-param name="theAngle" select="$azToPt"/>
    </xsl:call-template>
  </xsl:variable>
  
  <xsl:value-of select="$centreElev + $radius * $sinAzToPt"/> <!-- Return the computed elevation value -->
</xsl:template>


<!-- **************************************************************** -->
<!-- ************ Return The Computed In Vertical Angle ************* -->
<!-- **************************************************************** -->
<xsl:template name="ComputedInVertAngle">
  <xsl:param name="gradeIn"/>
  
  <!-- Compute the vertical angle turned from vertically up -->
  <xsl:variable name="arcTanGradeIn">
    <xsl:call-template name="ArcTanSeries">
      <xsl:with-param name="tanVal" select="$gradeIn"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:value-of select="$Pi div 2.0 - $arcTanGradeIn"/>
</xsl:template>


<!-- **************************************************************** -->
<!-- ************ Return The Computed Out Vertical Angle ************ -->
<!-- **************************************************************** -->
<xsl:template name="ComputedOutVertAngle">
  <xsl:param name="gradeOut"/>
  
  <!-- Compute the vertical angle turned from vertically up -->
  <xsl:variable name="arcTanGradeOut">
    <xsl:call-template name="ArcTanSeries">
      <xsl:with-param name="tanVal" select="$gradeOut"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:value-of select="$Pi div 2.0 - $arcTanGradeOut"/>
</xsl:template>


<!-- **************************************************************** -->
<!-- ****** Return In/Out Length For A Circular Vertical Curve ****** -->
<!-- **************************************************************** -->
<xsl:template name="CircVertCurveInOutLen">
  <xsl:param name="gradeIn"/>
  <xsl:param name="gradeOut"/>
  <xsl:param name="radius"/>
  <xsl:param name="returnInLen"/>
  
  <xsl:variable name="inVertAngle">
    <xsl:call-template name="ComputedInVertAngle">
      <xsl:with-param name="gradeIn" select="$gradeIn"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="outVertAngle">
    <xsl:call-template name="ComputedOutVertAngle">
      <xsl:with-param name="gradeOut" select="$gradeOut"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="absDeltaAngle" select="concat(substring('-',2 - (($inVertAngle - $outVertAngle) &lt; 0)), '1') * ($inVertAngle - $outVertAngle)"/>
  
  <xsl:choose>
    <xsl:when test="$absDeltaAngle &lt; 0.000001">0</xsl:when> <!-- No significant change in grade - return length of 0 --> 
    <xsl:otherwise>
      <xsl:variable name="includedAngle" select="$Pi - ($inVertAngle - $outVertAngle)"/>
      <xsl:variable name="tanHalfIncAngle">
        <xsl:call-template name="Tan">
          <xsl:with-param name="TheAngle" select="$includedAngle div 2.0"/>
        </xsl:call-template>
      </xsl:variable>

      <xsl:variable name="tanLength" select="concat(substring('-',2 - (($radius div $tanHalfIncAngle) &lt; 0)), '1') * ($radius div $tanHalfIncAngle)"/>

      <xsl:variable name="sinVal">
        <xsl:choose>
          <xsl:when test="$returnInLen = 'true'"> <!-- The in length is required -->
            <xsl:call-template name="Sine">
              <xsl:with-param name="theAngle" select="$inVertAngle"/>
            </xsl:call-template>
          </xsl:when>
          <xsl:otherwise>
            <xsl:call-template name="Sine">
              <xsl:with-param name="theAngle" select="$outVertAngle"/>
            </xsl:call-template>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:variable>
      
      <xsl:value-of select="$tanLength * $sinVal"/> <!-- Return the computed in/out length -->
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!-- **************************************************************** -->
<!-- ******** Return The Circular Vertical Curve Centre Point ******* -->
<!-- **************************************************************** -->
<xsl:template name="CircVertCurveCentrePoint">
  <xsl:param name="gradeIn"/>
  <xsl:param name="gradeOut"/>
  <xsl:param name="radius"/>
  <xsl:param name="stationIP"/>
  <xsl:param name="startElev"/>
  
  <xsl:variable name="inVertAngle">
    <xsl:call-template name="ComputedInVertAngle">
      <xsl:with-param name="gradeIn" select="$gradeIn"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="outVertAngle">
    <xsl:call-template name="ComputedOutVertAngle">
      <xsl:with-param name="gradeOut" select="$gradeOut"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="absDeltaAngle" select="concat(substring('-',2 - (($inVertAngle - $outVertAngle) &lt; 0)), '1') * ($inVertAngle - $outVertAngle)"/>
  
  <xsl:if test="$absDeltaAngle &gt; 0.000001"> <!-- Significant change in grade - return values -->
    <xsl:variable name="includedAngle" select="$Pi - ($inVertAngle - $outVertAngle)"/>
    <xsl:variable name="tanHalfIncAngle">
      <xsl:call-template name="Tan">
        <xsl:with-param name="TheAngle" select="$includedAngle div 2.0"/>
      </xsl:call-template>
    </xsl:variable>

    <xsl:variable name="tanLength" select="concat(substring('-',2 - (($radius div $tanHalfIncAngle) &lt; 0)), '1') * ($radius div $tanHalfIncAngle)"/>

    <!-- Centre is to the left (before) of the intersection point if the absolute -->
    <!-- value of the in grade is less than the absolute value of the out grade.  -->
    <xsl:variable name="absInAngle" select="concat(substring('-',2 - (($Pi div 2.0 - $inVertAngle) &lt; 0)), '1') * ($Pi div 2.0 - $inVertAngle)"/>
    <xsl:variable name="absOutAngle" select="concat(substring('-',2 - (($Pi div 2.0 - $outVertAngle) &lt; 0)), '1') * ($Pi div 2.0 - $outVertAngle)"/>
    <xsl:variable name="cosMeanAngle">
      <xsl:call-template name="Cosine">
        <xsl:with-param name="theAngle" select="($inVertAngle + $outVertAngle) div 2.0"/>
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="absCosMeanAngle" select="concat(substring('-',2 - ($cosMeanAngle &lt; 0)), '1') * $cosMeanAngle"/>
    <xsl:variable name="hypotenuse">
      <xsl:call-template name="Sqrt">
        <xsl:with-param name="num" select="$radius * $radius + $tanLength * $tanLength"/> 
      </xsl:call-template>
    </xsl:variable>

    <xsl:variable name="centreStation">
      <xsl:choose>
        <xsl:when test="$absInAngle &lt; $absOutAngle"> <!-- Left of centre -->
          <xsl:value-of select="$stationIP - $hypotenuse * $absCosMeanAngle"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$stationIP + $hypotenuse * $absCosMeanAngle"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    
    <xsl:variable name="sinInVertAngle">
      <xsl:call-template name="Sine">
        <xsl:with-param name="theAngle" select="$inVertAngle"/>
      </xsl:call-template>
    </xsl:variable>

    <!-- The Centre is above the road if the out grade is greater than the in grade. -->
    <!-- ie: (Pi / 2.0 - inVertAngle) < (Pi / 2.0 - outVertAngle)                    -->
    <!-- Due to the angles being turned from the vertical this is then rearranged    -->
    <!-- to be $outVertAngle < $inVertAngle.                                         -->
    <xsl:variable name="centreElev">
      <xsl:choose>
        <xsl:when test="$outVertAngle &lt; $inVertAngle">
          <xsl:value-of select="$startElev + $radius * $sinInVertAngle"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$startElev - $radius * $sinInVertAngle"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <!-- Now return the station/elevation pair -->
    <xsl:element name="centreStn">
      <xsl:value-of select="$centreStation"/>
    </xsl:element>
    <xsl:element name="centreElev">
      <xsl:value-of select="$centreElev"/>
    </xsl:element>
  </xsl:if>

</xsl:template>


<!-- **************************************************************** -->
<!-- ** Return The Sag/Summit Point For A Circular Vertical Curve *** -->
<!-- **************************************************************** -->
<xsl:template name="SagSummitPointCircularVC">
  <xsl:param name="centrePtStn"/>
  <xsl:param name="centrePtElev"/>
  <xsl:param name="radius"/>
  <xsl:param name="startStn"/>
  <xsl:param name="endStn"/>
  <xsl:param name="gradeIn"/>
  <xsl:param name="gradeOut"/>
  
  <xsl:if test="($centrePtStn &gt;= $startStn) and ($centrePtStn &lt;= $endStn)">
    <xsl:variable name="inVertAngle">
      <xsl:call-template name="ComputedInVertAngle">
        <xsl:with-param name="gradeIn" select="$gradeIn"/>
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="outVertAngle">
      <xsl:call-template name="ComputedOutVertAngle">
        <xsl:with-param name="gradeOut" select="$gradeOut"/>
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="sagSummitElev">
      <xsl:choose>
        <xsl:when test="$outVertAngle &lt; $inVertAngle"> <!-- Centre above the road -->
          <xsl:value-of select="$centrePtElev - $radius"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$centrePtElev + $radius"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:element name="sagSummitStn">
      <xsl:value-of select="$centrePtStn"/>
    </xsl:element>

    <xsl:element name="sagSummitElev">
      <xsl:value-of select="$sagSummitElev"/>
    </xsl:element>
  </xsl:if>
</xsl:template>


<!-- **************************************************************** -->
<!-- *** Return Design Elevation At Station Along Vert Alignment **** -->
<!-- **************************************************************** -->
<xsl:template name="ElevationAtStation">
  <xsl:param name="verticalDefn"/>
  <xsl:param name="inStation"/>
  
  <!-- Vertical definition node set has the structure:                       -->
  <!-- <VertElementName> (PVI, ParaCurve, UnsymParaCurve, CircCurve)         -->
  <!--   <VertElementName/>                                                  -->
  <!--     Contains VIP station and elevation pair and associated attributes -->
  <!--   <gradeIn/>                                                          -->
  <!--   <gradeOut/>                                                         -->
  <!--   <startStn/>                                                         -->
  <!--   <endStn/>                                                           -->
  <!--   <startElev/>                                                        -->
  <!--   <endElev/>                                                          -->
  <!--   <circVCCentreStation/>                                              -->
  <!--   <circVCCentreElevation/>                                            -->
  <!-- </VertElementName>                                                    -->

  <xsl:variable name="startStn" select="substring-before(normalize-space(msxsl:node-set($verticalDefn)/*[1]/child::*[1]), ' ')"/>
  <xsl:variable name="endStn" select="substring-before(normalize-space(msxsl:node-set($verticalDefn)/*[last()]/child::*[1]), ' ')"/>

  <!-- Set to start or end station if very close to either -->
  <xsl:variable name="station">
    <xsl:variable name="absStartDelta" select="concat(substring('-',2 - (($inStation - $startStn) &lt; 0)), '1') * ($inStation - $startStn)"/>
    <xsl:variable name="absEndDelta" select="concat(substring('-',2 - (($inStation - $endStn) &lt; 0)), '1') * ($inStation - $endStn)"/>
    <xsl:choose>
      <xsl:when test="$absStartDelta &lt; 0.0001">
        <xsl:value-of select="$startStn"/>
      </xsl:when>
      <xsl:when test="$absEndDelta &lt; 0.0001">
        <xsl:value-of select="$endStn"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$inStation"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <xsl:for-each select="msxsl:node-set($verticalDefn)/*">
    <xsl:variable name="stationIP" select="substring-before(normalize-space(child::*[1]), ' ')"/>
    <xsl:variable name="nextStnIP" select="substring-before(normalize-space(following-sibling::*[1]/child::*[1]), ' ')"/>
    <xsl:if test="($station &gt;= $stationIP) and (($station &lt; $nextStnIP) or (string(number($nextStnIP)) = 'NaN'))">
      <xsl:variable name="elevIP" select="substring-after(normalize-space(child::*[1]), ' ')"/>
      <!-- We have located the vertical IP before or equal to the required station -->
      <xsl:choose>
        <!-- When at a vertical point simply return its elevation -->
        <xsl:when test="(number($station) = number($stationIP)) and (name(.) = 'PVI')">
          <xsl:value-of select="$elevIP"/>
        </xsl:when>

        <!-- When on a grade compute the elevation along the grade -->
        <xsl:when test="($station &gt;= endStn) and ($station &lt;= following-sibling::*[1]/startStn)">
          <xsl:value-of select="$elevIP + ($station - $stationIP) * gradeOut"/>
        </xsl:when>
        
        <!-- When on the lead out section of this parabolic vertical curve compute the elevation on the curve -->
        <xsl:when test="((name(.) = 'ParaCurve') or (name(.) = 'UnsymParaCurve')) and 
                        ($station &lt; endStn)">
          <xsl:call-template name="ParabolaPointElevation">
            <xsl:with-param name="stationIP" select="$stationIP"/>
            <xsl:with-param name="gradeIn" select="gradeIn"/>
            <xsl:with-param name="gradeOut" select="gradeOut"/>
            <xsl:with-param name="startStn" select="startStn"/>
            <xsl:with-param name="endStn" select="endStn"/>
            <xsl:with-param name="startElev" select="startElev"/>
            <xsl:with-param name="endElev" select="endElev"/>
            <xsl:with-param name="lenIn" select="$stationIP - startStn"/>
            <xsl:with-param name="lenOut" select="endStn - $stationIP"/>
            <xsl:with-param name="ptStn" select="$station"/>
          </xsl:call-template>
        </xsl:when>
        
        <!-- When on the lead in section of next parabolic vertical curve compute the elevation on the curve -->
        <xsl:when test="((name(following-sibling::*[1]) = 'ParaCurve') or 
                         (name(following-sibling::*[1]) = 'UnsymParaCurve')) and 
                        ($station &gt; following-sibling::*[1]/startStn)">
          <xsl:call-template name="ParabolaPointElevation">
            <xsl:with-param name="stationIP" select="$nextStnIP"/>
            <xsl:with-param name="gradeIn" select="following-sibling::*[1]/gradeIn"/>
            <xsl:with-param name="gradeOut" select="following-sibling::*[1]/gradeOut"/>
            <xsl:with-param name="startStn" select="following-sibling::*[1]/startStn"/>
            <xsl:with-param name="endStn" select="following-sibling::*[1]/endStn"/>
            <xsl:with-param name="startElev" select="following-sibling::*[1]/startElev"/>
            <xsl:with-param name="endElev" select="following-sibling::*[1]/endElev"/>
            <xsl:with-param name="lenIn" select="$nextStnIP - following-sibling::*[1]/startStn"/>
            <xsl:with-param name="lenOut" select="following-sibling::*[1]/endStn - $nextStnIP"/>
            <xsl:with-param name="ptStn" select="$station"/>
          </xsl:call-template>
        </xsl:when>
        
        <!-- When on the lead out section of this circular vertical curve compute the elevation on the curve -->
        <xsl:when test="(name(.) = 'CircCurve') and ($station &lt; endStn)">
          <xsl:call-template name="CircularVertCurvePointElevation">
            <xsl:with-param name="startStn" select="startStn"/>
            <xsl:with-param name="startElev" select="startElev"/>
            <xsl:with-param name="centreStn" select="circVCCentreStation"/>
            <xsl:with-param name="centreElev" select="circVCCentreElevation"/>
            <xsl:with-param name="radius" select="child::*[1]/@radius"/>
            <xsl:with-param name="ptStn" select="$station"/>
          </xsl:call-template>
        </xsl:when>
        
        <!-- When on the lead in section of next circular vertical curve compute the elevation on the curve -->
        <xsl:when test="(name(following-sibling::*[1]) = 'CircCurve') and 
                        ($station &gt; following-sibling::*[1]/startStn)">
          <xsl:call-template name="CircularVertCurvePointElevation">
            <xsl:with-param name="startStn" select="following-sibling::*[1]/startStn"/>
            <xsl:with-param name="startElev" select="following-sibling::*[1]/startElev"/>
            <xsl:with-param name="centreStn" select="following-sibling::*[1]/circVCCentreStation"/>
            <xsl:with-param name="centreElev" select="following-sibling::*[1]/circVCCentreElevation"/>
            <xsl:with-param name="radius" select="following-sibling::*[1]/child::*[1]/@radius"/>
            <xsl:with-param name="ptStn" select="$station"/>
          </xsl:call-template>
        </xsl:when>
        
      </xsl:choose>
    </xsl:if>
  </xsl:for-each>
</xsl:template>  

  
<!-- **************************************************************** -->
<!-- ******* Return the grade at Station along Vert Alignment ******* -->
<!-- **************************************************************** -->
<xsl:template name="GradeAtStation">
  <xsl:param name="verticalDefn"/>
  <xsl:param name="inStation"/>

  <xsl:variable name="startStn" select="substring-before(normalize-space(msxsl:node-set($verticalDefn)/*[1]/child::*[1]), ' ')"/>
  <xsl:variable name="endStn" select="substring-before(normalize-space(msxsl:node-set($verticalDefn)/*[last()]/child::*[1]), ' ')"/>

  <!-- Set to start or end station if very close to either -->
  <xsl:variable name="station">
    <xsl:variable name="absStartDelta" select="concat(substring('-',2 - (($inStation - $startStn) &lt; 0)), '1') * ($inStation - $startStn)"/>
    <xsl:variable name="absEndDelta" select="concat(substring('-',2 - (($inStation - $endStn) &lt; 0)), '1') * ($inStation - $endStn)"/>
    <xsl:choose>
      <xsl:when test="$absStartDelta &lt; 0.0001">
        <xsl:value-of select="$startStn"/>
      </xsl:when>
      <xsl:when test="$absEndDelta &lt; 0.0001">
        <xsl:value-of select="$endStn"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$inStation"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <xsl:for-each select="msxsl:node-set($verticalDefn)/*">
    <xsl:variable name="stationIP" select="substring-before(normalize-space(child::*[1]), ' ')"/>
    <xsl:variable name="nextStnIP" select="substring-before(normalize-space(following-sibling::*[1]/child::*[1]), ' ')"/>
    <xsl:if test="(number($station) &gt;= $stationIP) and ((number($station) &lt; $nextStnIP) or (string(number($nextStnIP)) = 'NaN'))">
      <xsl:choose>
        <xsl:when test="(name(.) = 'PVI') and (number($station) = $stationIP)">
          <xsl:choose>
            <xsl:when test="position() = 1">
              <xsl:value-of select="gradeOut"/> <!-- Return the gradeOut value for the first PVI -->
            </xsl:when>
            <xsl:otherwise>
              <xsl:value-of select="gradeIn"/>  <!-- Return the gradeIn value for a PVI -->
            </xsl:otherwise>
          </xsl:choose>
        </xsl:when>

        <xsl:when test="$station &lt; endStn">  <!--  Position is within the second half of the vertical curve at the this vertical IP -->
          <xsl:choose>
            <xsl:when test="(name(.) = 'ParaCurve') or (name(.) = 'UnsymParaCurve')">
              <xsl:variable name="changeOfGrade" select="gradeOut - gradeIn"/>
              <xsl:value-of select="gradeIn + $changeOfGrade div (endStn - startStn) * ($station - startStn)"/>
            </xsl:when>
            <xsl:otherwise>  <!-- Must be in second half of a circular vertical curve -->
              <xsl:call-template name="VerticalCurveGradeAtStation">
                <xsl:with-param name="curveDefn" select="."/>
                <xsl:with-param name="station" select="$station"/>
                <xsl:with-param name="inRadius" select="child::*[1]/@radius"/>
              </xsl:call-template>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:when>

        <xsl:when test="(position() &lt; last()) and (number($station) &gt; following-sibling::*[1]/startStn)">
          <!-- Position is in the first half of the next vertical curve -->
          <xsl:choose>
            <xsl:when test="(name(following-sibling::*[1]) = 'ParaCurve') or (name(following-sibling::*[1]) = 'UnsymParaCurve')">
              <xsl:for-each select="following-sibling::*[1]">  <!-- Get the next element -->
                <xsl:variable name="changeOfGrade" select="gradeOut - gradeIn"/>
                <xsl:value-of select="gradeIn + $changeOfGrade div (endStn - startStn) * ($station - startStn)"/>
              </xsl:for-each>
            </xsl:when>
            <xsl:otherwise>  <!-- Must be in first half of a circular vertical curve -->
              <xsl:call-template name="VerticalCurveGradeAtStation">
                <xsl:with-param name="curveDefn" select="."/>
                <xsl:with-param name="station" select="$station"/>
                <xsl:with-param name="inRadius" select="child::*[1]/@radius"/>
              </xsl:call-template>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:when>

        <xsl:otherwise>  <!-- Position is on the grade between vertical curves or points -->
          <xsl:value-of select="gradeOut"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:if>
  </xsl:for-each>

</xsl:template>


<!-- **************************************************************** -->
<!-- *** Return the grade at station on circular vertical curve ***** -->
<!-- **************************************************************** -->
<xsl:template name="VerticalCurveGradeAtStation">
  <xsl:param name="curveDefn"/>
  <xsl:param name="station"/>
  <xsl:param name="inRadius"/>

  <xsl:for-each select="msxsl:node-set($curveDefn)">
    <xsl:variable name="stationIP" select="substring-before(normalize-space(child::*[1]), ' ')"/>
    <xsl:variable name="deltaStation" select="endStn - startStn"/>
    
    <xsl:variable name="gradeInAsAngle">
      <xsl:call-template name="ArcTanSeries">
        <xsl:with-param name="tanVal" select="gradeIn"/>
      </xsl:call-template>
    </xsl:variable>
    
    <xsl:variable name="gradeOutAsAngle">
      <xsl:call-template name="ArcTanSeries">
        <xsl:with-param name="tanVal" select="gradeOut"/>
      </xsl:call-template>
    </xsl:variable>

    <xsl:variable name="inVertAngle" select="$Pi div 2.0 - $gradeInAsAngle"/>
    <xsl:variable name="outVertAngle" select="$Pi div 2.0 - $gradeOutAsAngle"/>
    <xsl:variable name="absDeltaGrade" select="concat(substring('-',2 - (($inVertAngle - $outVertAngle) &lt; 0)), '1') * ($inVertAngle - $outVertAngle)"/>

    <xsl:choose>
      <xsl:when test="$absDeltaGrade &lt; 0.000001">  <!-- Effectively no change in grade therefore no arc length -->
        <xsl:choose>
          <xsl:when test="$station &lt;= $stationIP">
            <xsl:value-of select="gradeIn"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="gradeOut"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      
      <xsl:otherwise>
        <xsl:variable name="radius" select="concat(substring('-',2 - ($inRadius &lt; 0)), '1') * $inRadius"/>
        <xsl:variable name="arcLength" select="$absDeltaGrade * $radius"/>
        <xsl:variable name="distAlongArc" select="($station - startStn) * $arcLength div $deltaStation"/>
        <xsl:variable name="includedAngle" select="$distAlongArc div (2.0 * $Pi * $inRadius)"/>
        <!-- Add the included angle on to the in vertical angle then convert back to grade -->
        <xsl:call-template name="Tan">
          <xsl:with-param name="TheAngle" select="$Pi div 2.0 - $inVertAngle + $includedAngle"/>
        </xsl:call-template>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:for-each>
</xsl:template>


<!-- **************************************************************** -->
<!-- ****** Return the MFactor at Station along Vert Alignment ****** -->
<!-- **************************************************************** -->
<xsl:template name="MFactorAtStation">
  <xsl:param name="verticalDefn"/>
  <xsl:param name="inStation"/>

  <xsl:variable name="startStn" select="substring-before(normalize-space(msxsl:node-set($verticalDefn)/*[1]/child::*[1]), ' ')"/>
  <xsl:variable name="endStn" select="substring-before(normalize-space(msxsl:node-set($verticalDefn)/*[last()]/child::*[1]), ' ')"/>

  <!-- Set to start or end station if very close to either -->
  <xsl:variable name="station">
    <xsl:variable name="absStartDelta" select="concat(substring('-',2 - (($inStation - $startStn) &lt; 0)), '1') * ($inStation - $startStn)"/>
    <xsl:variable name="absEndDelta" select="concat(substring('-',2 - (($inStation - $endStn) &lt; 0)), '1') * ($inStation - $endStn)"/>
    <xsl:choose>
      <xsl:when test="$absStartDelta &lt; 0.0001">
        <xsl:value-of select="$startStn"/>
      </xsl:when>
      <xsl:when test="$absEndDelta &lt; 0.0001">
        <xsl:value-of select="$endStn"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$inStation"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <xsl:variable name="mFactor">
    <xsl:for-each select="msxsl:node-set($verticalDefn)/*[number($station) = number(substring-before(normalize-space(child::*[1]), ' ')) and
                                                          (name(.) != 'PVI') and (endStn - startStn != 0)]">
      <xsl:value-of select="100.0 * ((100.0 * gradeOut - 100.0 * gradeIn) div (endStn - startStn))"/>
    </xsl:for-each>
  </xsl:variable>
  
  <xsl:choose>
    <xsl:when test="string(number($mFactor)) != 'NaN'">
      <xsl:value-of select="$mFactor"/>
    </xsl:when>
    <xsl:otherwise>0</xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!-- **************************************************************** -->
<!-- *** Return Design Position At Station Along Horiz Alignment **** -->
<!-- **************************************************************** -->
<xsl:template name="PositionAtStation">
  <xsl:param name="horizontalDefn"/>
  <xsl:param name="inStation"/>

  <xsl:variable name="startStn" select="msxsl:node-set($horizontalDefn)/startStation"/>
  <xsl:variable name="endStn" select="msxsl:node-set($horizontalDefn)/startStation + msxsl:node-set($horizontalDefn)/length"/>

  <!-- Set to start or end station if very close to either -->
  <xsl:variable name="station">
    <xsl:variable name="absStartDelta" select="concat(substring('-',2 - (($inStation - $startStn) &lt; 0)), '1') * ($inStation - $startStn)"/>
    <xsl:variable name="absEndDelta" select="concat(substring('-',2 - (($inStation - $endStn) &lt; 0)), '1') * ($inStation - $endStn)"/>
    <xsl:choose>
      <xsl:when test="$absStartDelta &lt; 0.0001">
        <xsl:value-of select="$startStn"/>
      </xsl:when>
      <xsl:when test="$absEndDelta &lt; 0.0001">
        <xsl:value-of select="$endStn"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$inStation"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>
  
  <xsl:for-each select="msxsl:node-set($horizontalDefn)/*">
    <!-- If the $station value is actually the staStart value for an element then simply return -->
    <!-- the available Start coordinates otherwise determine the coordinates along the element. -->
    <xsl:choose>
      <xsl:when test="(position() &gt; 2) and (concat(substring('-',2 - ((startStn - $station) &lt; 0)), '1') * (startStn - $station) &lt; 0.00001)">
        <xsl:element name="north">
          <xsl:call-template name="NorthVal">
            <xsl:with-param name="coordStr" select="Start"/>
          </xsl:call-template>
        </xsl:element>
        <xsl:element name="east">
          <xsl:call-template name="EastVal">
            <xsl:with-param name="coordStr" select="Start"/>
          </xsl:call-template>
        </xsl:element>
      </xsl:when>
      <xsl:otherwise>
        <!-- Skip the startStation and length elements (first 2 elements), ensure station is greater than -->
        <!-- or equal to the start station of element and less than the end station of element or, if the -->
        <!-- last element then allow for match with end station of element.                               -->
        <xsl:if test="(position() &gt; 2) and (number(startStn) &lt;= number($station)) and
                      (number(startStn + length - 0.000001) &gt; number($station)) or
                      ((number($station) = number($endStn)) and (position() = last()))">
          <xsl:variable name="distAlong" select="$station - startStn"/>
          <xsl:choose>
            <xsl:when test="name(.) = 'Line'">
              <xsl:call-template name="InterpolatedCoords">
                <xsl:with-param name="distAlong" select="$distAlong"/>
                <xsl:with-param name="startCoord" select="Start"/>
                <xsl:with-param name="endCoord" select="End"/>
              </xsl:call-template>
            </xsl:when>

            <xsl:when test="name(.) = 'Curve'">
              <xsl:variable name="deflectionAngle" select="$distAlong div @radius * 180.0 div $Pi"/>

              <xsl:variable name="azToStart">
                <xsl:variable name="coordDeltas">
                  <xsl:call-template name="ComputeDeltaCoords">
                    <xsl:with-param name="startCoord" select="Center"/>
                    <xsl:with-param name="endCoord" select="Start"/>
                  </xsl:call-template>
                </xsl:variable>
                <xsl:call-template name="InverseAzimuth">
                  <xsl:with-param name="deltaN" select="msxsl:node-set($coordDeltas)/deltaN"/>
                  <xsl:with-param name="deltaE" select="msxsl:node-set($coordDeltas)/deltaE"/>
                </xsl:call-template>
              </xsl:variable>

              <xsl:variable name="azToPoint">
                <xsl:call-template name="NormalisedAngle">
                  <xsl:with-param name="angle">
                    <xsl:choose>
                      <xsl:when test="@rot = 'cw'">
                        <xsl:value-of select="$azToStart + $deflectionAngle"/>
                      </xsl:when>
                      <xsl:otherwise>
                        <xsl:value-of select="$azToStart - $deflectionAngle"/>
                      </xsl:otherwise>
                    </xsl:choose>
                  </xsl:with-param>
                </xsl:call-template>
              </xsl:variable>

              <xsl:variable name="startN">
                <xsl:call-template name="NorthVal">
                  <xsl:with-param name="coordStr" select="Center"/>
                </xsl:call-template>
              </xsl:variable>

              <xsl:variable name="startE">
                <xsl:call-template name="EastVal">
                  <xsl:with-param name="coordStr" select="Center"/>
                </xsl:call-template>
              </xsl:variable>

              <xsl:call-template name="FollowAzimuth">
                <xsl:with-param name="azimuth" select="$azToPoint"/>
                <xsl:with-param name="distance" select="@radius"/>
                <xsl:with-param name="startN" select="$startN"/>
                <xsl:with-param name="startE" select="$startE"/>
              </xsl:call-template>
            </xsl:when>

            <xsl:when test="(name(.) = 'Spiral') and (spiralType = 'EntrySpiral')">
              <xsl:variable name="offsets">
                <xsl:call-template name="CalcSpiral">
                  <xsl:with-param name="smallRadius" select="@radiusEnd"/>
                  <xsl:with-param name="largeRadius" select="@radiusStart"/>
                  <xsl:with-param name="length" select="length"/>
                  <xsl:with-param name="spiralDist" select="$distAlong"/>
                  <xsl:with-param name="spiralType" select="'Clothoid'"/>
                </xsl:call-template>
              </xsl:variable>

              <xsl:variable name="startN">
                <xsl:call-template name="NorthVal">
                  <xsl:with-param name="coordStr" select="Start"/>
                </xsl:call-template>
              </xsl:variable>

              <xsl:variable name="startE">
                <xsl:call-template name="EastVal">
                  <xsl:with-param name="coordStr" select="Start"/>
                </xsl:call-template>
              </xsl:variable>
              
              <xsl:variable name="tempPos">
                <xsl:call-template name="FollowAzimuth">
                  <xsl:with-param name="azimuth" select="inAzimuth"/>
                  <xsl:with-param name="distance" select="msxsl:node-set($offsets)/along"/>
                  <xsl:with-param name="startN" select="$startN"/>
                  <xsl:with-param name="startE" select="$startE"/>
                </xsl:call-template>
              </xsl:variable>

              <xsl:variable name="across">
                <xsl:choose>
                  <xsl:when test="@rot = 'cw'">
                    <xsl:value-of select="msxsl:node-set($offsets)/across"/>
                  </xsl:when>
                  <xsl:otherwise>
                    <xsl:value-of select="msxsl:node-set($offsets)/across * -1.0"/>
                  </xsl:otherwise>
                </xsl:choose>
              </xsl:variable>

              <xsl:call-template name="FollowAzimuth">
                <xsl:with-param name="azimuth" select="inAzimuth + 90.0"/>
                <xsl:with-param name="distance" select="$across"/>
                <xsl:with-param name="startN" select="msxsl:node-set($tempPos)/north"/>
                <xsl:with-param name="startE" select="msxsl:node-set($tempPos)/east"/>
              </xsl:call-template>
            </xsl:when>

            <xsl:when test="(name(.) = 'Spiral') and (spiralType = 'ExitSpiral')">
              <xsl:variable name="offsets">
                <xsl:call-template name="CalcSpiral">
                  <xsl:with-param name="smallRadius" select="@radiusStart"/>
                  <xsl:with-param name="largeRadius" select="@radiusEnd"/>
                  <xsl:with-param name="length" select="length"/>
                  <xsl:with-param name="spiralDist" select="length - $distAlong"/>
                  <xsl:with-param name="spiralType" select="'Clothoid'"/>
                </xsl:call-template>
              </xsl:variable>

              <xsl:variable name="startN">
                <xsl:call-template name="NorthVal">
                  <xsl:with-param name="coordStr" select="End"/>
                </xsl:call-template>
              </xsl:variable>

              <xsl:variable name="startE">
                <xsl:call-template name="EastVal">
                  <xsl:with-param name="coordStr" select="End"/>
                </xsl:call-template>
              </xsl:variable>

              <xsl:variable name="tempPos">
                <xsl:call-template name="FollowAzimuth">
                  <xsl:with-param name="azimuth" select="outAzimuth + 180.0"/>
                  <xsl:with-param name="distance" select="msxsl:node-set($offsets)/along"/>
                  <xsl:with-param name="startN" select="$startN"/>
                  <xsl:with-param name="startE" select="$startE"/>
                </xsl:call-template>
              </xsl:variable>

              <xsl:variable name="across">
                <xsl:choose>
                  <xsl:when test="@rot = 'cw'">
                    <xsl:value-of select="msxsl:node-set($offsets)/across * -1.0"/>
                  </xsl:when>
                  <xsl:otherwise>
                    <xsl:value-of select="msxsl:node-set($offsets)/across"/>
                  </xsl:otherwise>
                </xsl:choose>
              </xsl:variable>

              <xsl:call-template name="FollowAzimuth">
                <xsl:with-param name="azimuth" select="outAzimuth - 90.0"/>
                <xsl:with-param name="distance" select="$across"/>
                <xsl:with-param name="startN" select="msxsl:node-set($tempPos)/north"/>
                <xsl:with-param name="startE" select="msxsl:node-set($tempPos)/east"/>
              </xsl:call-template>
            </xsl:when>
          </xsl:choose>
        </xsl:if>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:for-each>
</xsl:template>


<!-- **************************************************************** -->
<!-- ******* Return the instantaneous radius given the station ****** -->
<!-- **************************************************************** -->
<xsl:template name="InstantaneousRadius">
  <xsl:param name="horizontalDefn"/>
  <xsl:param name="inStation"/>

  <xsl:variable name="startStn" select="msxsl:node-set($horizontalDefn)/startStation"/>
  <xsl:variable name="endStn" select="msxsl:node-set($horizontalDefn)/startStation + msxsl:node-set($horizontalDefn)/length"/>

  <!-- Set to start or end station if very close to either -->
  <xsl:variable name="station">
    <xsl:variable name="absStartDelta" select="concat(substring('-',2 - (($inStation - $startStn) &lt; 0)), '1') * ($inStation - $startStn)"/>
    <xsl:variable name="absEndDelta" select="concat(substring('-',2 - (($inStation - $endStn) &lt; 0)), '1') * ($inStation - $endStn)"/>
    <xsl:choose>
      <xsl:when test="$absStartDelta &lt; 0.0001">
        <xsl:value-of select="$startStn"/>
      </xsl:when>
      <xsl:when test="$absEndDelta &lt; 0.0001">
        <xsl:value-of select="$endStn"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$inStation"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <xsl:choose>
    <xsl:when test="$station &lt; msxsl:node-set($horizontalDefn)/startStation">Out of range</xsl:when>
    <xsl:when test="$station &gt; (msxsl:node-set($horizontalDefn)/startStation + msxsl:node-set($horizontalDefn)/length)">Out of range</xsl:when>
    <xsl:otherwise>
      <xsl:for-each select="msxsl:node-set($horizontalDefn)/*">
        <xsl:variable name="stationToHandle">
          <xsl:call-template name="StationToHandle">
            <xsl:with-param name="station" select="$station"/>
            <xsl:with-param name="endStn" select="$endStn"/>
          </xsl:call-template>
        </xsl:variable>
        <xsl:if test="$stationToHandle = 'true'">
          <xsl:choose>
            <xsl:when test="name(.) = 'Line'">Straight</xsl:when>

            <xsl:when test="name(.) = 'Curve'">
              <xsl:choose>
                <xsl:when test="@rot = 'cw'">
                  <xsl:value-of select="@radius"/>
                </xsl:when>
                <xsl:otherwise>
                  <xsl:value-of select="@radius * -1.0"/>  <!-- Change sign of radius value -->
                </xsl:otherwise>
              </xsl:choose>
            </xsl:when>

            <xsl:when test="(name(.) = 'Spiral') and (spiralType = 'EntrySpiral')">
              <xsl:variable name="distAlong" select="$station - startStn"/>
              <xsl:variable name="radius">
                <xsl:choose>
                  <xsl:when test="$distAlong &lt; 0.00005"> <!-- At the start station of entry spiral - return @radiusStart -->
                    <xsl:choose>
                      <xsl:when test="string(number(@radiusStart)) = 'NaN'">Straight</xsl:when>
                      <xsl:otherwise>
                        <xsl:value-of select="@radiusStart"/>
                      </xsl:otherwise>
                    </xsl:choose>
                  </xsl:when>
                  <xsl:otherwise>  <!-- At some point along the spiral -->
                    <xsl:choose>
                      <xsl:when test="string(number(@radiusStart)) = 'NaN'">  <!-- Spiraling from infinity -->
                        <xsl:choose>
                          <xsl:when test="@spiType = 'bloss'">  <!-- Bloss spirals only spiral from infinity -->
                            <xsl:call-template name="CalcRadiusOnBlossSpiral">
                              <xsl:with-param name="spiralDist" select="$distAlong"/>
                              <xsl:with-param name="smallRadius" select="@radiusEnd"/>
                              <xsl:with-param name="length" select="length"/>
                            </xsl:call-template>
                          </xsl:when>
                          <xsl:otherwise>  <!-- Clothoid or Cubic spiral -->
                            <!-- For a spiral RL = K (constant) for any point on the spiral so R = K / L -->
                            <xsl:value-of select="(length * @radiusEnd) div $distAlong"/>
                          </xsl:otherwise>
                        </xsl:choose>
                      </xsl:when>
                      <xsl:otherwise>  <!-- Spiraling from part way along the spiral (Clothoid or Cubic spiral) -->
                        <!-- First compute the length of the spiral required to spiral from infinity to the start radius -->
                        <xsl:variable name="leadInLen" select="(@radiusEnd * length) div (@radiusStart - @radiusEnd)"/>
                        <xsl:value-of select="(($leadInLen + length) * @radiusEnd) div ($leadInLen + $distAlong)"/>
                      </xsl:otherwise>
                    </xsl:choose>
                  </xsl:otherwise>
                </xsl:choose>
              </xsl:variable>

              <xsl:choose>
                <xsl:when test="$radius = 'Straight'">
                  <xsl:value-of select="$radius"/>
                </xsl:when>
                <xsl:when test="@rot = 'cw'">
                  <xsl:value-of select="$radius"/>
                </xsl:when>
                <xsl:otherwise>  <!-- Change sign of radius value -->
                  <xsl:value-of select="$radius * -1.0"/>
                </xsl:otherwise>
              </xsl:choose>
            </xsl:when>

            <xsl:when test="(name(.) = 'Spiral') and (spiralType = 'ExitSpiral')">
              <xsl:variable name="distAlong" select="length - ($station - startStn)"/>
              <xsl:variable name="radius">
                <xsl:choose>
                  <xsl:when test="$distAlong &lt; 0.00005"> <!-- At the end station of entry spiral - return EndRadius -->
                    <xsl:choose>
                      <xsl:when test="string(number(@radiusEnd)) = 'NaN'">Straight</xsl:when>
                      <xsl:otherwise>
                        <xsl:value-of select="@radiusEnd"/>
                      </xsl:otherwise>
                    </xsl:choose>
                  </xsl:when>
                  <xsl:otherwise>  <!-- At some point along the spiral -->
                    <xsl:choose>
                      <xsl:when test="concat(substring('-',2 - ((startStn - $station) &lt; 0)), '1') * (startStn - $station) &lt; 0.000001">
                        <!-- We are at the start of the exit spiral - simply return the @radiusStart value -->
                        <xsl:value-of select="@radiusStart"/>
                      </xsl:when>
                      <xsl:when test="string(number(@radiusEnd)) = 'NaN'">  <!-- Spiraling to infinity -->
                        <xsl:choose>
                          <xsl:when test="@spiType = 'bloss'">  <!-- Bloss spirals only spiral to infinity -->
                            <xsl:call-template name="CalcRadiusOnBlossSpiral">
                              <xsl:with-param name="spiralDist" select="$distAlong"/>
                              <xsl:with-param name="smallRadius" select="@radiusStart"/>
                              <xsl:with-param name="length" select="length"/>
                            </xsl:call-template>
                          </xsl:when>
                          <xsl:otherwise>  <!-- Clothoid or Cubic spiral -->
                            <!-- For a spiral RL = K (constant) for any point on the spiral so R = K / L -->
                            <xsl:value-of select="(length * @radiusStart) div $distAlong"/>
                          </xsl:otherwise>
                        </xsl:choose>
                      </xsl:when>
                      <xsl:otherwise>  <!-- Spiraling to part way along the spiral (Clothoid or Cubic spiral) -->
                        <!-- First compute the length of the spiral required to spiral from the end radius to infinity -->
                        <xsl:variable name="leadOutLen" select="(@radiusStart * length) div (@radiusEnd - @radiusStart)"/>
                        <xsl:value-of select="(($leadOutLen + length) * @radiusStart) div ($leadOutLen + $distAlong)"/>
                      </xsl:otherwise>
                    </xsl:choose>
                  </xsl:otherwise>
                </xsl:choose>
              </xsl:variable>

              <xsl:choose>
                <xsl:when test="$radius = 'Straight'">
                  <xsl:value-of select="$radius"/>
                </xsl:when>
                <xsl:when test="@rot = 'cw'">
                  <xsl:value-of select="$radius"/>
                </xsl:when>
                <xsl:otherwise>  <!-- Change sign of radius value -->
                  <xsl:value-of select="$radius * -1.0"/>
                </xsl:otherwise>
              </xsl:choose>
            </xsl:when>
          </xsl:choose>
        </xsl:if>
      </xsl:for-each>
    </xsl:otherwise>
  </xsl:choose>

</xsl:template>


<!-- **************************************************************** -->
<!-- ****** Return the instantaneous azimuth given the station ****** -->
<!-- **************************************************************** -->
<xsl:template name="InstantaneousAzimuth">
  <xsl:param name="horizontalDefn"/>
  <xsl:param name="inStation"/>

  <xsl:variable name="startStn" select="msxsl:node-set($horizontalDefn)/startStation"/>
  <xsl:variable name="endStn" select="msxsl:node-set($horizontalDefn)/startStation + msxsl:node-set($horizontalDefn)/length"/>

  <!-- Set to start or end station if very close to either -->
  <xsl:variable name="station">
    <xsl:variable name="absStartDelta" select="concat(substring('-',2 - (($inStation - $startStn) &lt; 0)), '1') * ($inStation - $startStn)"/>
    <xsl:variable name="absEndDelta" select="concat(substring('-',2 - (($inStation - $endStn) &lt; 0)), '1') * ($inStation - $endStn)"/>
    <xsl:choose>
      <xsl:when test="$absStartDelta &lt; 0.0001">
        <xsl:value-of select="$startStn"/>
      </xsl:when>
      <xsl:when test="$absEndDelta &lt; 0.0001">
        <xsl:value-of select="$endStn"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$inStation"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <xsl:choose>
    <xsl:when test="$station &lt; msxsl:node-set($horizontalDefn)/startStation">Out of range</xsl:when>
    <xsl:when test="$station &gt; (msxsl:node-set($horizontalDefn)/startStation + msxsl:node-set($horizontalDefn)/length)">Out of range</xsl:when>
    <xsl:otherwise>
      <xsl:for-each select="msxsl:node-set($horizontalDefn)/*">
        <xsl:variable name="stationToHandle">
          <xsl:call-template name="StationToHandle">
            <xsl:with-param name="station" select="$station"/>
            <xsl:with-param name="endStn" select="$endStn"/>
          </xsl:call-template>
        </xsl:variable>
        <xsl:if test="$stationToHandle = 'true'">
          <xsl:choose>
            <xsl:when test="name(.) = 'Line'">
              <xsl:variable name="coordDeltas">
                <xsl:call-template name="ComputeDeltaCoords">
                  <xsl:with-param name="startCoord" select="Start"/>
                  <xsl:with-param name="endCoord" select="End"/>
                </xsl:call-template>
              </xsl:variable>
              <xsl:call-template name="InverseAzimuth">
                <xsl:with-param name="deltaN" select="msxsl:node-set($coordDeltas)/deltaN"/>
                <xsl:with-param name="deltaE" select="msxsl:node-set($coordDeltas)/deltaE"/>
              </xsl:call-template>
            </xsl:when>

            <xsl:when test="name(.) = 'Curve'">
              <xsl:variable name="deltaAngle" select="(($station - startStn) div @radius) * 180.0 div $Pi"/>
              <xsl:choose>
                <xsl:when test="@rot = 'cw'">
                  <xsl:call-template name="NormalisedAngle">
                    <xsl:with-param name="angle" select="inAzimuth + $deltaAngle"/>
                  </xsl:call-template>
                </xsl:when>
                <xsl:otherwise>
                  <xsl:call-template name="NormalisedAngle">
                    <xsl:with-param name="angle" select="inAzimuth - $deltaAngle"/>
                  </xsl:call-template>
                </xsl:otherwise>
              </xsl:choose>
            </xsl:when>

            <xsl:when test="(name(.) = 'Spiral') and (spiralType = 'EntrySpiral')">
              <xsl:choose>
                <xsl:when test="concat(substring('-',2 - ((startStn - $station) &lt; 0)), '1') * (startStn - $station) &lt; 0.000001">
                  <!-- We are at the start of the spiral - simply return the inAzimuth value -->
                  <xsl:value-of select="inAzimuth"/>
                </xsl:when>
                <xsl:otherwise>
                  <xsl:variable name="spiralType">
                    <xsl:choose>
                      <xsl:when test="@spiType = 'clothoid'">Clothoid</xsl:when>
                      <xsl:when test="@spiType = 'cubic'">Cubic</xsl:when>
                      <xsl:when test="@spiType = 'bloss'">Bloss</xsl:when>
                      <xsl:otherwise>Clothoid</xsl:otherwise>
                    </xsl:choose>
                  </xsl:variable>

                  <xsl:variable name="deltaAngle">
                    <xsl:call-template name="CalcSpiralDeflection">
                      <xsl:with-param name="smallRadius" select="@radiusEnd"/>
                      <xsl:with-param name="largeRadius" select="@radiusStart"/>
                      <xsl:with-param name="length" select="length"/>
                      <xsl:with-param name="spiralDist" select="$station - startStn"/>
                      <xsl:with-param name="spiralType" select="$spiralType"/>
                    </xsl:call-template>
                  </xsl:variable>
                  <xsl:choose>
                    <xsl:when test="@rot = 'cw'">
                      <xsl:call-template name="NormalisedAngle">
                        <xsl:with-param name="angle" select="inAzimuth + $deltaAngle"/>
                      </xsl:call-template>
                    </xsl:when>
                    <xsl:otherwise>
                      <xsl:call-template name="NormalisedAngle">
                        <xsl:with-param name="angle" select="inAzimuth - $deltaAngle"/>
                      </xsl:call-template>
                    </xsl:otherwise>
                  </xsl:choose>
                </xsl:otherwise>
              </xsl:choose>
            </xsl:when>

            <xsl:when test="(name(.) = 'Spiral') and (spiralType = 'ExitSpiral')">
              <xsl:choose>
                <xsl:when test="concat(substring('-',2 - ((startStn - $station) &lt; 0)), '1') * (startStn - $station) &lt; 0.000001">
                  <!-- We are at the start of the spiral - simply return the inAzimuth value -->
                  <xsl:value-of select="inAzimuth"/>
                </xsl:when>
                <xsl:otherwise>
                  <xsl:variable name="spiralType">
                    <xsl:choose>
                      <xsl:when test="@spiType = 'clothoid'">Clothoid</xsl:when>
                      <xsl:when test="@spiType = 'cubic'">Cubic</xsl:when>
                      <xsl:when test="@spiType = 'bloss'">Bloss</xsl:when>
                      <xsl:otherwise>Clothoid</xsl:otherwise>
                    </xsl:choose>
                  </xsl:variable>

                  <xsl:variable name="deltaAngle">
                    <xsl:call-template name="CalcSpiralDeflection">
                      <xsl:with-param name="smallRadius" select="@radiusStart"/>
                      <xsl:with-param name="largeRadius" select="@radiusEnd"/>
                      <xsl:with-param name="length" select="length"/>
                      <xsl:with-param name="spiralDist" select="length - ($station - startStn)"/>
                      <xsl:with-param name="spiralType" select="$spiralType"/>
                    </xsl:call-template>
                  </xsl:variable>
                  <xsl:choose>
                    <xsl:when test="@rot = 'cw'">
                      <xsl:call-template name="NormalisedAngle">
                        <xsl:with-param name="angle" select="outAzimuth - $deltaAngle"/>
                      </xsl:call-template>
                    </xsl:when>
                    <xsl:otherwise>
                      <xsl:call-template name="NormalisedAngle">
                        <xsl:with-param name="angle" select="outAzimuth + $deltaAngle"/>
                      </xsl:call-template>
                    </xsl:otherwise>
                  </xsl:choose>
                </xsl:otherwise>
              </xsl:choose>
            </xsl:when>
          </xsl:choose>
        </xsl:if>
      </xsl:for-each>
    </xsl:otherwise>
  </xsl:choose>

</xsl:template>


<!-- **************************************************************** -->
<!-- ******** Determine if this is a station to be dealt with ******* -->
<!-- **************************************************************** -->
<xsl:template name="StationToHandle">
  <xsl:param name="station"/>
  <xsl:param name="endStn"/>
  
  <xsl:variable name="absDeltaElementStartStn" select="concat(substring('-',2 - ((startStn - $station) &lt; 0)), '1') * (startStn - $station)"/>
  <xsl:variable name="absDeltaEndStn" select="concat(substring('-',2 - (($endStn - $station) &lt; 0)), '1') * ($endStn - $station)"/>

  <xsl:choose>
    <xsl:when test="position() &lt; 3">false</xsl:when>                      <!-- Skip the startStation and length elements -->
    <xsl:when test="$absDeltaElementStartStn &lt; 0.00001">true</xsl:when>  <!-- At the element start station -->
    <xsl:when test="(number(startStn) &lt;= number($station)) and
                    (number(startStn + length - 0.00001) &gt; number($station))">true</xsl:when> <!-- In the station range for this element -->
    <xsl:when test="($absDeltaEndStn &lt; 0.00001) and (position() = last())">true</xsl:when>    <!-- At the horizontal end station and on the last element -->
    <xsl:otherwise>false</xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!-- **************************************************************** -->
<!-- ********* Return coords interpolated between end points ******** -->
<!-- **************************************************************** -->
<xsl:template name="InterpolatedCoords">
  <xsl:param name="distAlong"/>
  <xsl:param name="startCoord"/>
  <xsl:param name="endCoord"/>
  
  <xsl:variable name="startN">
    <xsl:call-template name="NorthVal">
      <xsl:with-param name="coordStr" select="$startCoord"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="startE">
    <xsl:call-template name="EastVal">
      <xsl:with-param name="coordStr" select="$startCoord"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="endN">
    <xsl:call-template name="NorthVal">
      <xsl:with-param name="coordStr" select="$endCoord"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="endE">
    <xsl:call-template name="EastVal">
      <xsl:with-param name="coordStr" select="$endCoord"/>
    </xsl:call-template>
  </xsl:variable>
  
  <xsl:variable name="totalLen">
    <xsl:call-template name="Sqrt">
      <xsl:with-param name="num" select="($startN - $endN) * ($startN - $endN) + ($startE - $endE) * ($startE - $endE)"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:element name="north">
    <xsl:value-of select="($endN - $startN) * $distAlong div $totalLen + $startN"/>
  </xsl:element>

  <xsl:element name="east">
    <xsl:value-of select="($endE - $startE) * $distAlong div $totalLen + $startE"/>
  </xsl:element>
</xsl:template>


<!-- **************************************************************** -->
<!-- ******************* Output value to the file ******************* -->
<!-- **************************************************************** -->
<xsl:template name="OutputFileValue">
  <xsl:param name="value"/>
  <xsl:param name="useCoordNullVal" select="'true'"/>
  <xsl:param name="applyDistUnitsConv" select="'true'"/>

  <xsl:variable name="outputVal">
    <xsl:choose>
      <xsl:when test="string(number($value)) = 'NaN'">
        <xsl:choose>
          <xsl:when test="$useCoordNullVal = 'true'">-999</xsl:when>
          <xsl:otherwise>999999.9</xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      <xsl:otherwise>
        <xsl:choose>
          <xsl:when test="$applyDistUnitsConv = 'true'">
            <xsl:value-of select="$value * $DistConvFactor"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="$value"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>


  <xsl:call-template name="PadLeft">
    <xsl:with-param name="stringWidth" select="23"/>
    <xsl:with-param name="theString" select="format-number($outputVal, $DecPl8, 'Standard')"/>
  </xsl:call-template>

<!--
  <xsl:call-template name="ConvertDoubleToExponentialValue">
    <xsl:with-param name="value" select="$outputVal"/>
  </xsl:call-template>
-->
</xsl:template>


<!-- **************************************************************** -->
<!-- ********* Return North Value From A Coord Pair/Triple ********** -->
<!-- **************************************************************** -->
<xsl:template name="NorthVal">
  <xsl:param name="coordStr"/>
  
  <xsl:value-of select="substring-before(normalize-space($coordStr), ' ')"/>
</xsl:template>

  
<!-- **************************************************************** -->
<!-- ********** Return East Value From A Coord Pair/Triple ********** -->
<!-- **************************************************************** -->
<xsl:template name="EastVal">
  <xsl:param name="coordStr"/>
  
  <!-- Need to allow for potential 3D points -->
  <xsl:variable name="temp" select="substring-after(normalize-space($coordStr), ' ')"/>
  
  <xsl:choose>
    <xsl:when test="contains($temp, ' ')">  <!-- This is a 3D point -->
      <xsl:value-of select="substring-before($temp, ' ')"/>
    </xsl:when>
    <xsl:otherwise>                          <!-- This is a 2D point -->
      <xsl:value-of select="$temp"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

  
<!-- **************************************************************** -->
<!-- ******** Compute coordinate deltas from supplied coords ******** -->
<!-- **************************************************************** -->
<xsl:template name="ComputeDeltaCoords">
  <xsl:param name="startCoord"/>
  <xsl:param name="endCoord"/>

  <xsl:variable name="startNorth">
    <xsl:call-template name="NorthVal">
      <xsl:with-param name="coordStr" select="$startCoord"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="startEast">
    <xsl:call-template name="EastVal">
      <xsl:with-param name="coordStr" select="$startCoord"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="endNorth">
    <xsl:call-template name="NorthVal">
      <xsl:with-param name="coordStr" select="$endCoord"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="endEast">
    <xsl:call-template name="EastVal">
      <xsl:with-param name="coordStr" select="$endCoord"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:element name="deltaN">
    <xsl:value-of select="$endNorth - $startNorth"/>
  </xsl:element>
  <xsl:element name="deltaE">
    <xsl:value-of select="$endEast  - $startEast"/>
  </xsl:element>
</xsl:template>


<!-- **************************************************************** -->
<!-- ****************** Return Computed Arc length ****************** -->
<!-- **************************************************************** -->
<xsl:template name="ComputedArcLength">
  <xsl:param name="startCoord"/>
  <xsl:param name="centreCoord"/>
  <xsl:param name="endCoord"/>
  <xsl:param name="direction"/>

  <xsl:variable name="radius">
    <xsl:variable name="coordDeltas">
      <xsl:call-template name="ComputeDeltaCoords">
        <xsl:with-param name="startCoord" select="$startCoord"/>
        <xsl:with-param name="endCoord" select="$centreCoord"/>
      </xsl:call-template>
    </xsl:variable>
    <xsl:call-template name="InverseDistance">
      <xsl:with-param name="deltaN" select="msxsl:node-set($coordDeltas)/deltaN"/>
      <xsl:with-param name="deltaE" select="msxsl:node-set($coordDeltas)/deltaE"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="azToStart">
    <xsl:variable name="coordDeltas">
      <xsl:call-template name="ComputeDeltaCoords">
        <xsl:with-param name="startCoord" select="$centreCoord"/>
        <xsl:with-param name="endCoord" select="$startCoord"/>
      </xsl:call-template>
    </xsl:variable>
    <xsl:call-template name="InverseAzimuth">
      <xsl:with-param name="deltaN" select="msxsl:node-set($coordDeltas)/deltaN"/>
      <xsl:with-param name="deltaE" select="msxsl:node-set($coordDeltas)/deltaE"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="azToEnd">
    <xsl:variable name="coordDeltas">
      <xsl:call-template name="ComputeDeltaCoords">
        <xsl:with-param name="startCoord" select="$centreCoord"/>
        <xsl:with-param name="endCoord" select="$endCoord"/>
      </xsl:call-template>
    </xsl:variable>
    <xsl:call-template name="InverseAzimuth">
      <xsl:with-param name="deltaN" select="msxsl:node-set($coordDeltas)/deltaN"/>
      <xsl:with-param name="deltaE" select="msxsl:node-set($coordDeltas)/deltaE"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="clockwiseAng">
    <xsl:call-template name="ClockwiseAngle">
      <xsl:with-param name="fromAzimuth" select="$azToStart"/>
      <xsl:with-param name="toAzimuth" select="$azToEnd"/>
    </xsl:call-template>
  </xsl:variable>

  <!-- Now return the computed arc length (the arc angle in radians * radius) -->
  <xsl:choose>
    <xsl:when test="$direction = 'cw'">   <!-- Clockwise arc so use the clockwise angle -->
      <xsl:value-of select="$clockwiseAng * $Pi div 180.0 * $radius"/>
    </xsl:when>
    <xsl:otherwise>     <!-- Anticlockwise arc so use complement of clockwise angle -->
      <xsl:value-of select="(360.0 - $clockwiseAng) * $Pi div 180.0 * $radius"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!-- **************************************************************** -->
<!-- ************ Return the sine of an angle in radians ************ -->
<!-- **************************************************************** -->
<xsl:template name="Sine">
  <xsl:param name="theAngle"/>
  <xsl:variable name="normalisedAngle">
    <xsl:call-template name="RadianAngleBetweenLimits">
      <xsl:with-param name="anAngle" select="$theAngle"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="theSine">
    <xsl:call-template name="sineIter">
      <xsl:with-param name="pX2" select="$normalisedAngle * $normalisedAngle"/>
      <xsl:with-param name="pRslt" select="$normalisedAngle"/>
      <xsl:with-param name="pElem" select="$normalisedAngle"/>
      <xsl:with-param name="pN" select="1"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:value-of select="number($theSine)"/>
</xsl:template>

<xsl:template name="sineIter">
  <xsl:param name="pX2"/>
  <xsl:param name="pRslt"/>
  <xsl:param name="pElem"/>
  <xsl:param name="pN"/>
  <xsl:param name="pEps" select="0.00000001"/>
  <xsl:variable name="vnextN" select="$pN+2"/>
  <xsl:variable name="vnewElem"  select="-$pElem*$pX2 div ($vnextN*($vnextN - 1))"/>
  <xsl:variable name="vnewResult" select="$pRslt + $vnewElem"/>
  <xsl:variable name="vdiffResult" select="$vnewResult - $pRslt"/>
  <xsl:choose>
    <xsl:when test="$vdiffResult > $pEps or $vdiffResult &lt; -$pEps">
      <xsl:call-template name="sineIter">
        <xsl:with-param name="pX2" select="$pX2"/>
        <xsl:with-param name="pRslt" select="$vnewResult"/>
        <xsl:with-param name="pElem" select="$vnewElem"/>
        <xsl:with-param name="pN" select="$vnextN"/>
        <xsl:with-param name="pEps" select="$pEps"/>
      </xsl:call-template>
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="$vnewResult"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!-- **************************************************************** -->
<!-- *********** Return the Cosine of an angle in radians *********** -->
<!-- **************************************************************** -->
<xsl:template name="Cosine">
  <xsl:param name="theAngle"/>

  <!-- Use the sine function after subtracting the angle from halfPi -->
  <xsl:call-template name="Sine">
    <xsl:with-param name="theAngle" select="$halfPi - $theAngle"/>
  </xsl:call-template>
</xsl:template>


<!-- **************************************************************** -->
<!-- *********** Return the Tangent of an angle in radians ********** -->
<!-- **************************************************************** -->
<xsl:template name="Tan">
  <xsl:param name="theAngle"/>
  <xsl:param name="prec" select="0.00000001"/>
  <xsl:param name="abortIfInvalid" select="1"/>

  <xsl:variable name="xDivHalfPi" select="floor($theAngle div $halfPi)"/>
  <xsl:variable name="xHalfPiDiff" select="$theAngle - $halfPi * $xDivHalfPi"/>

  <xsl:choose>  <!-- Check for a solution -->
    <xsl:when test="(-$prec &lt; $xHalfPiDiff) and
                    ($xHalfPiDiff &lt; $prec) and
                    ($xDivHalfPi mod 2 = 1)">
      <xsl:choose>
        <xsl:when test="$abortIfInvalid">
          <xsl:message terminate="yes">
            <xsl:value-of select="concat('Function error: tan() not defined for TheAngle =', $theAngle)"/>
          </xsl:message>
        </xsl:when>

        <xsl:otherwise>Infinity</xsl:otherwise>
      </xsl:choose>
    </xsl:when>

    <!-- Compute the sine and cosine of the angle to get the tangent value -->
    <xsl:otherwise>
      <xsl:variable name="vSin">
        <xsl:call-template name="Sine">
          <xsl:with-param name="theAngle" select="$theAngle"/>
        </xsl:call-template>
      </xsl:variable>

      <xsl:variable name="vCos">
        <xsl:call-template name="Cosine">
          <xsl:with-param name="theAngle" select="$theAngle"/>
        </xsl:call-template>
      </xsl:variable>

      <xsl:value-of select="$vSin div $vCos"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!-- **************************************************************** -->
<!-- ******* Return the arcTan value using a series expansion ******* -->
<!-- **************************************************************** -->
<xsl:template name="ArcTanSeries">
  <xsl:param name="tanVal"/>

  <!-- If the absolute value of tanVal is greater than 1 the work with the -->
  <!-- reciprocal value and return the resultant angle subtracted from Pi. -->
  <xsl:variable name="absTanVal" select="concat(substring('-',2 - ($tanVal &lt; 0)), '1') * $tanVal"/>
  <xsl:variable name="tanVal2">
    <xsl:choose>
      <xsl:when test="$absTanVal &gt; 1.0">
        <xsl:value-of select="1.0 div $tanVal"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$tanVal"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <xsl:variable name="valSq" select="$tanVal2 * $tanVal2"/>

  <xsl:variable name="angVal">
    <xsl:value-of select="$tanVal2 div (1 + ($valSq
                                   div (3 + (4 * $valSq
                                   div (5 + (9 * $valSq
                                   div (7 + (16 * $valSq
                                   div (9 + (25 * $valSq
                                   div (11 + (36 * $valSq
                                   div (13 + (49 * $valSq
                                   div (15 + (64 * $valSq
                                   div (17 + (81 * $valSq
                                   div (19 + (100 * $valSq
                                   div (21 + (121 * $valSq
                                   div (23 + (144 * $valSq
                                   div (25 + (169 * $valSq
                                   div (27 + (196 * $valSq
                                   div (29 + (225 * $valSq))))))))))))))))))))))))))))))"/>
  </xsl:variable>

  <xsl:choose>
    <xsl:when test="$absTanVal &gt; 1.0">
      <xsl:choose>
        <xsl:when test="$tanVal &lt; 0">
          <xsl:value-of select="-$halfPi - $angVal"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$halfPi - $angVal"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="$angVal"/>
    </xsl:otherwise>
  </xsl:choose>

</xsl:template>


<!-- **************************************************************** -->
<!-- *************** Return the square root of a value ************** -->
<!-- **************************************************************** -->
<xsl:template name="Sqrt">
  <xsl:param name="num" select="0"/>       <!-- The number you want to find the square root of -->
  <xsl:param name="try" select="1"/>       <!-- The current 'try'.  This is used internally. -->
  <xsl:param name="iter" select="1"/>      <!-- The current iteration, checked against maxiter to limit loop count - used internally -->
  <xsl:param name="maxiter" select="40"/>  <!-- Set this up to insure against infinite loops - used internally -->

  <!-- This template uses Sir Isaac Newton's method of finding roots -->

  <xsl:choose>
    <xsl:when test="$num &lt; 0"></xsl:when>  <!-- Invalid input - no square root of a negative number so return null -->
    <xsl:when test="$try * $try = $num or $iter &gt; $maxiter">
      <xsl:value-of select="$try"/>
    </xsl:when>
    <xsl:otherwise>
      <xsl:call-template name="Sqrt">
        <xsl:with-param name="num" select="$num"/>
        <xsl:with-param name="try" select="$try - (($try * $try - $num) div (2 * $try))"/>
        <xsl:with-param name="iter" select="$iter + 1"/>
        <xsl:with-param name="maxiter" select="$maxiter"/>
      </xsl:call-template>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!-- **************************************************************** -->
<!-- *********** Convert from double to exponential value *********** -->
<!-- **************************************************************** -->
<xsl:template name="ConvertDoubleToExponentialValue">
  <xsl:param name="value"/>

  <xsl:variable name="absValue" select="concat(substring('-',2 - ($value &lt; 0)), '1') * $value"/>

  <xsl:choose>
    <xsl:when test="$absValue = 0.0">
      <xsl:text>0.00000000000000000e+000</xsl:text>
    </xsl:when>

    <xsl:when test="($absValue &gt;= 1.0) and ($absValue &lt; 10.0)">
      <xsl:value-of select="format-number($value, '0.0000000000000')"/>
      <xsl:text>e+000</xsl:text>
    </xsl:when>

    <xsl:when test="$absValue &gt;= 10.0">
      <xsl:variable name="retVal">
        <xsl:call-template name="GetBaseValueAndExponent">
          <xsl:with-param name="value" select="$absValue"/>
        </xsl:call-template>
      </xsl:variable>
      <xsl:if test="$value &lt; 0.0">-</xsl:if>
      <xsl:value-of select="format-number(msxsl:node-set($retVal)/base, '0.0000000000000000')"/>
      <xsl:text>e</xsl:text>
      <xsl:value-of select="msxsl:node-set($retVal)/exponent"/>
    </xsl:when>

    <xsl:when test="$absValue &lt; 1.0">
      <xsl:variable name="retVal">
        <xsl:call-template name="GetBaseValueAndExponent">
          <xsl:with-param name="value" select="$absValue"/>
        </xsl:call-template>
      </xsl:variable>
      <xsl:if test="$value &lt; 0.0">-</xsl:if>
      <xsl:value-of select="format-number(msxsl:node-set($retVal)/base, '0.0000000000000000')"/>
      <xsl:text>e</xsl:text>
      <xsl:value-of select="msxsl:node-set($retVal)/exponent"/>
    </xsl:when>
  </xsl:choose>
</xsl:template>


<!-- **************************************************************** -->
<!-- ***** Return Angle between 0 and 360 or -180 to 180 degrees **** -->
<!-- **************************************************************** -->
<xsl:template name="NormalisedAngle">
  <xsl:param name="angle"/>
  <xsl:param name="plusMinus180" select="'false'"/>

  <xsl:variable name="fullCircleAngle">
    <xsl:choose>
      <xsl:when test="$angle &lt; 0">
        <xsl:variable name="newAngle">
          <xsl:value-of select="$angle + 360.0"/>
        </xsl:variable>
        <xsl:call-template name="NormalisedAngle">
          <xsl:with-param name="angle" select="$newAngle"/>
        </xsl:call-template>
      </xsl:when>

      <xsl:when test="$angle &gt;= 360.0">
        <xsl:variable name="newAngle">
          <xsl:value-of select="$angle - 360.0"/>
        </xsl:variable>
        <xsl:call-template name="NormalisedAngle">
          <xsl:with-param name="angle" select="$newAngle"/>
        </xsl:call-template>
      </xsl:when>

      <xsl:otherwise>
        <xsl:value-of select="$angle"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <xsl:choose>
    <xsl:when test="$plusMinus180 = 'false'">
      <xsl:value-of select="$fullCircleAngle"/>
    </xsl:when>
    <xsl:otherwise>
      <xsl:choose>
        <xsl:when test="$fullCircleAngle &lt;= 180.0">
          <xsl:value-of select="$fullCircleAngle"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$fullCircleAngle - 360.0"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!-- **************************************************************** -->
<!-- *********** Pad a string to the right with spaces ************** -->
<!-- **************************************************************** -->
<xsl:template name="PadRight">
  <xsl:param name="stringWidth"/>
  <xsl:param name="theString"/>
  <xsl:choose>
    <xsl:when test="$stringWidth = '0'">
      <xsl:value-of select="normalize-space($theString)"/> <!-- Function return value -->
    </xsl:when>
    <xsl:otherwise>
      <xsl:variable name="paddedStr" select="concat($theString, '                                                                                          ')"/>
      <xsl:value-of select="substring($paddedStr, 1, $stringWidth)"/> <!-- Function return value -->
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!-- **************************************************************** -->
<!-- *********** Pad a string to the left with spaces *************** -->
<!-- **************************************************************** -->
<xsl:template name="PadLeft">
  <xsl:param name="stringWidth"/>
  <xsl:param name="theString"/>
  <xsl:choose>
    <xsl:when test="$stringWidth = '0'">
      <xsl:value-of select="normalize-space($theString)"/>
    </xsl:when>
    <xsl:otherwise>
      <xsl:variable name="paddedStr" select="concat('                                                            ', $theString)"/>
      <xsl:value-of select="substring($paddedStr, string-length($paddedStr) - $stringWidth + 1)"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!-- **************************************************************** -->
<!-- ********************** New Line Output ************************* -->
<!-- **************************************************************** -->
<xsl:template name="NewLine">
  <xsl:text>&#10;</xsl:text>
</xsl:template>


<!-- **************************************************************** -->
<!-- ****************** Compute Inverse Distance ******************** -->
<!-- **************************************************************** -->
<xsl:template name="InverseDistance">
  <xsl:param name="deltaN"/>
  <xsl:param name="deltaE"/>

  <!-- Compute the inverse distance from the deltas -->
  <xsl:choose>  <!-- If delta values are both effectively 0 return 0 -->
    <xsl:when test="((concat(substring('-',2 - ($deltaN &lt; 0)), '1') * $deltaN) &lt; 0.000001) and
                    ((concat(substring('-',2 - ($deltaE &lt; 0)), '1') * $deltaE) &lt; 0.000001)">0</xsl:when>
    <xsl:otherwise>
      <!-- Return hypotenuse distance -->
      <xsl:call-template name="Sqrt">
        <xsl:with-param name="num" select="$deltaN * $deltaN + $deltaE * $deltaE"/>
      </xsl:call-template>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!-- **************************************************************** -->
<!-- ******************* Compute Inverse Azimuth ******************** -->
<!-- **************************************************************** -->
<xsl:template name="InverseAzimuth">
  <xsl:param name="deltaN"/>
  <xsl:param name="deltaE"/>
  <xsl:param name="returnInRadians" select="'false'"/>

  <!-- Compute the inverse azimuth from the deltas -->
  <xsl:variable name="absDeltaN" select="concat(substring('-',2 - ($deltaN &lt; 0)), '1') * $deltaN"/>
  <xsl:variable name="absDeltaE" select="concat(substring('-',2 - ($deltaE &lt; 0)), '1') * $deltaE"/>

  <xsl:variable name="flag">
    <xsl:choose>
      <xsl:when test="$absDeltaE &gt; $absDeltaN">1</xsl:when>
      <xsl:otherwise>0</xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <xsl:variable name="adjDeltaN">
    <xsl:choose>
      <xsl:when test="$flag"><xsl:value-of select="$absDeltaE"/></xsl:when>
      <xsl:otherwise><xsl:value-of select="$absDeltaN"/></xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <xsl:variable name="adjDeltaE">
    <xsl:choose>
      <xsl:when test="$flag"><xsl:value-of select="$absDeltaN"/></xsl:when>
      <xsl:otherwise><xsl:value-of select="$absDeltaE"/></xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <!-- Compute the raw angle value -->
  <xsl:variable name="angle">
    <xsl:choose>
      <xsl:when test="$adjDeltaN &lt; 0.000001">
        <xsl:value-of select="0"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:variable name="arcTanAngle">
          <xsl:call-template name="ArcTanSeries">
            <xsl:with-param name="tanVal" select="$adjDeltaE div $adjDeltaN"/>
          </xsl:call-template>
        </xsl:variable>
        <xsl:choose>
          <xsl:when test="$flag">
            <xsl:value-of select="$halfPi - $arcTanAngle"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="$arcTanAngle"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <!-- Assemble the raw angle value into an azimuth -->
  <xsl:variable name="azimuth">
    <xsl:choose>
      <xsl:when test="$deltaE &lt; 0">
        <xsl:choose>
          <xsl:when test="$deltaN &lt; 0">
            <xsl:value-of select="$Pi + $angle"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="$Pi * 2 - $angle"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      <xsl:otherwise>
        <xsl:choose>
          <xsl:when test="$deltaN &lt; 0">
            <xsl:value-of select="$Pi - $angle"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="$angle"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <!-- Return the azimuth value in radians or decimal degrees as requested -->
  <xsl:choose>
    <xsl:when test="$returnInRadians = 'true'">
      <xsl:value-of select="$azimuth"/>
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="$azimuth * 180 div $Pi"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!-- **************************************************************** -->
<!-- ********* Follow Azimuth to Compute New Point Coords *********** -->
<!-- **************************************************************** -->
<xsl:template name="FollowAzimuth">
  <xsl:param name="azimuth"/>  <!-- in degrees -->
  <xsl:param name="distance"/>
  <xsl:param name="startN"/>
  <xsl:param name="startE"/>
  <xsl:param name="startElev" select="''"/>
  <xsl:param name="endElev" select="''"/>
  <xsl:param name="elevInterpLength" select="''"/>
  <xsl:param name="grade"/>

  <xsl:variable name="sineVal">
    <xsl:call-template name="Sine">
      <xsl:with-param name="theAngle" select="$azimuth * $Pi div 180.0"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="cosineVal">
    <xsl:call-template name="Cosine">
      <xsl:with-param name="theAngle" select="$azimuth * $Pi div 180.0"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="newElev">
    <xsl:choose>
      <xsl:when test="(string(number($startElev)) != 'NaN') and (string(number($endElev)) != 'NaN')">
        <xsl:value-of select="($startElev + $endElev) div 2.0"/> <!-- Return average elevation -->
      </xsl:when>
      <xsl:when test="(string(number($startElev)) != 'NaN') and (string(number($grade)) != 'NaN') and
                      (string(number($elevInterpLength)) != 'NaN')">
        <xsl:value-of select="$startElev + $elevInterpLength * $grade"/> <!-- Apply the grade over the interpolation length to the startElev -->
      </xsl:when>
      <xsl:otherwise><xsl:value-of select="$startElev"/></xsl:otherwise> <!-- Return startElev elevation -->
    </xsl:choose>
  </xsl:variable>

  <!-- Return the coords as a node-set variable -->
  <xsl:element name="north" namespace="">
    <xsl:value-of select="$startN + $cosineVal * $distance"/>
  </xsl:element>
  <xsl:element name="east" namespace="">
    <xsl:value-of select="$startE + $sineVal * $distance"/>
  </xsl:element>
  <xsl:element name="elevation" namespace="">
    <xsl:value-of select="$newElev"/>
  </xsl:element>
</xsl:template>


<!-- **************************************************************** -->
<!-- ************* Calculate a spiral deflection angle ************** -->
<!-- **************************************************************** -->
<xsl:template name="CalcSpiralDeflection">
  <xsl:param name="smallRadius"/>
  <xsl:param name="largeRadius"/>
  <xsl:param name="length"/>
  <xsl:param name="spiralDist"/>
  <xsl:param name="along"/>  <!-- Only used for KoreanCubicParabola -->
  <xsl:param name="spiralType" select="'Clothoid'"/>  <!-- Can be 'Clothoid', 'KoreanCubicParabola', 'Cubic' or 'Bloss' -->
  
  <xsl:if test="(string(number($length)) != 'NaN') and (string(number($smallRadius)) != 'NaN') and
                (string(number($spiralDist)) != 'NaN') and ($spiralDist &gt;= 0) and ($spiralDist &lt;= $length)">
    <!-- We have valid input values -->
    <xsl:choose>
      <xsl:when test="($spiralDist = 0.0) or ($smallRadius = 0.0)">
        <xsl:value-of select="0.0"/>  <!-- Return a deflection angle of 0 -->
      </xsl:when>

      <xsl:when test="string(number($largeRadius)) = 'NaN'">  <!-- Standard spiral calculation - Clothoid spiral, Cubic spiral and Cubic parabola -->
        <xsl:choose>
          <xsl:when test="($spiralType = 'Clothoid') or ($spiralType = 'Cubic')">
            <xsl:variable name="K" select="$smallRadius * $length"/>
            <xsl:value-of select="($spiralDist * $spiralDist) div (2.0 * $K) * 180.0 div $Pi"/>
          </xsl:when>

          <xsl:when test="$spiralType = 'Bloss'">  <!-- Bloss spiral -->
            <xsl:value-of select="($spiralDist * $spiralDist * $spiralDist div ($smallRadius * $length * $length) -
                                   $spiralDist * $spiralDist * $spiralDist * $spiralDist div (2.0 * $smallRadius * $length * $length * $length)) * 180.0 div $Pi"/>
          </xsl:when>
          
          <xsl:otherwise>   <!-- KoreanCubicParabola -->
            <xsl:variable name="angle">
              <xsl:call-template name="ArcTanSeries">
                <xsl:with-param name="tanVal" select="$along div (2.0 * $smallRadius)"/>
              </xsl:call-template>
            </xsl:variable>
            <xsl:value-of select="$angle * 180.0 div $Pi"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>

      <xsl:otherwise>
        <xsl:choose>
          <xsl:when test="$largeRadius != $smallRadius">
            <!-- No partial spiral deflection computation for KoreanCubicParabola or Bloss spiral -->
            <xsl:variable name="K" select="($smallRadius * $largeRadius * $length) div ($largeRadius - $smallRadius)"/>
            <xsl:variable name="prevLength" select="concat(substring('-',2 - (($K div $smallRadius) &lt; 0)), '1') * ($K div $smallRadius) - $length"/>
            <xsl:variable name="localDist" select="$prevLength + $spiralDist"/>
            <xsl:variable name="startDeflection" select="($prevLength * $prevLength) div (2.0 * $K)"/>
            <xsl:value-of select="(($localDist * $localDist) div (2.0 * $K) - $startDeflection) * 180.0 div $Pi"/>
          </xsl:when>

          <xsl:otherwise>   <!-- This is a special case of an arc -->
            <xsl:value-of select="($length div $smallRadius) * 180.0 div $Pi"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:if>

</xsl:template>


<!-- **************************************************************** -->
<!-- ******************** Calculate spiral deltas ******************* -->
<!-- **************************************************************** -->
<xsl:template name="CalcSpiral">
  <xsl:param name="smallRadius"/>
  <xsl:param name="largeRadius"/>
  <xsl:param name="length"/>
  <xsl:param name="spiralDist"/>
  <xsl:param name="spiralType" select="'Clothoid'"/>  <!-- Can be 'Clothoid', 'Cubic', 'Bloss', 'KoreanCubicParabola' or 'NSWCubicParabola' -->

  <!-- Returns node-set variable with the elements:  -->
  <!--   along                                       -->
  <!--   across                                      -->
  <!--   deflection                                  -->

  <xsl:if test="(string(number($length)) != 'NaN') and (string(number($smallRadius)) != 'NaN') and
                (string(number($spiralDist)) != 'NaN') and ($spiralDist &gt;= 0) and ($spiralDist &lt;= $length)">
    <!-- We have valid input values -->
    <xsl:choose>
      <xsl:when test="($spiralDist = 0.0) or ($smallRadius = 0.0)">
        <xsl:element name="along" namespace="">0</xsl:element>
        <xsl:element name="across" namespace="">0</xsl:element>
        <xsl:element name="deflection" namespace="">0</xsl:element>
      </xsl:when>

      <xsl:when test="string(number($largeRadius)) = 'NaN'">  <!-- Standard spiral calculation -->
        <xsl:variable name="K" select="$smallRadius * $length"/>

        <xsl:variable name="transitionXc">
          <xsl:if test="$spiralType = 'NSWCubicParabola'">
            <xsl:call-template name="CalcNSWTransitionXc">
              <xsl:with-param name="length" select="$length"/>
              <xsl:with-param name="smallRadius" select="$smallRadius"/>
              <xsl:with-param name="largeRadius" select="$largeRadius"/>
            </xsl:call-template>
          </xsl:if>
        </xsl:variable>

        <xsl:variable name="deflectionNSW">  <!-- Returned in degrees -->
          <xsl:if test="$spiralType = 'NSWCubicParabola'">
            <xsl:call-template name="CalcNSWSpiralDeflection">
              <xsl:with-param name="smallRadius" select="$smallRadius"/>
              <xsl:with-param name="transitionXc" select="$transitionXc"/>
            </xsl:call-template>
          </xsl:if>
        </xsl:variable>

        <xsl:variable name="spiralM">
          <xsl:if test="$spiralType = 'NSWCubicParabola'">
            <xsl:variable name="tanDeflection">
              <xsl:call-template name="Tan">
                <xsl:with-param name="theAngle" select="$deflectionNSW * $Pi div 180.0"/>
              </xsl:call-template>
            </xsl:variable>
            <xsl:value-of select="$tanDeflection div (3.0 * $transitionXc * $transitionXc)"/>
          </xsl:if>
        </xsl:variable>

        <xsl:variable name="spiralVals">
          <xsl:call-template name="CalcSpiralXY">
            <xsl:with-param name="spiralDist" select="$spiralDist"/>
            <xsl:with-param name="K" select="$K"/>
            <xsl:with-param name="spiralM" select="$spiralM"/>
            <xsl:with-param name="spiralType" select="$spiralType"/>
            <xsl:with-param name="smallRadius" select="$smallRadius"/>
            <xsl:with-param name="length" select="$length"/>
          </xsl:call-template>
        </xsl:variable>

        <xsl:element name="along" namespace="">
          <xsl:value-of select="msxsl:node-set($spiralVals)/along"/>
        </xsl:element>

        <xsl:element name="across" namespace="">
          <xsl:value-of select="msxsl:node-set($spiralVals)/across"/>
        </xsl:element>

        <!-- Now compute the spiral deflection - returned in decimal degrees -->
        <xsl:element name="deflection" namespace="">
          <xsl:choose>
            <xsl:when test="$spiralType = 'NSWCubicParabola'">
              <xsl:value-of select="$deflectionNSW"/>
            </xsl:when>

            <xsl:otherwise>
              <xsl:call-template name="CalcSpiralDeflection">
                <xsl:with-param name="smallRadius" select="$smallRadius"/>
                <xsl:with-param name="largeRadius" select="$largeRadius"/>
                <xsl:with-param name="length" select="$length"/>
                <xsl:with-param name="spiralDist" select="$spiralDist"/>
                <xsl:with-param name="along" select="msxsl:node-set($spiralVals)/along"/>
                <xsl:with-param name="spiralType" select="$spiralType"/>
              </xsl:call-template>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:element>
      </xsl:when>

      <xsl:otherwise>  <!-- Spiral between 2 arcs (only applies to Clothoid spiral, Cubic spiral and NSWCubicParabola spirals) -->
        <xsl:choose>
          <xsl:when test="$largeRadius != $smallRadius">
            <xsl:choose>
              <xsl:when test="($spiralType = 'Clothoid') or ($spiralType = 'Cubic')">
                <xsl:variable name="K" select="($smallRadius * $largeRadius * $length) div ($largeRadius - $smallRadius)"/>
                <xsl:variable name="prevLength" select="concat(substring('-',2 - (($K div $smallRadius) &lt; 0)), '1') * ($K div $smallRadius) - $length"/>
                <xsl:variable name="localDist" select="$prevLength + $spiralDist"/>
                <xsl:variable name="startDeflection" select="($prevLength * $prevLength) div (2.0 * $K)"/>

                <xsl:variable name="startDeflectionX">
                  <xsl:call-template name="Cosine">
                    <xsl:with-param name="theAngle" select="$startDeflection"/>
                  </xsl:call-template>
                </xsl:variable>

                <xsl:variable name="startDeflectionY">
                  <xsl:call-template name="Sine">
                    <xsl:with-param name="theAngle" select="$startDeflection"/>
                  </xsl:call-template>
                </xsl:variable>

                <xsl:variable name="prevLenVals">
                  <xsl:call-template name="CalcSpiralXY">
                    <xsl:with-param name="spiralDist" select="$prevLength"/>
                    <xsl:with-param name="K" select="$K"/>
                    <xsl:with-param name="spiralType" select="$spiralType"/>
                    <xsl:with-param name="smallRadius" select="$smallRadius"/>
                    <xsl:with-param name="length" select="$length"/>
                  </xsl:call-template>
                </xsl:variable>

                <xsl:variable name="localDistVals">
                  <xsl:call-template name="CalcSpiralXY">
                    <xsl:with-param name="spiralDist" select="$localDist"/>
                    <xsl:with-param name="K" select="$K"/>
                    <xsl:with-param name="spiralType" select="$spiralType"/>
                    <xsl:with-param name="smallRadius" select="$smallRadius"/>
                    <xsl:with-param name="length" select="$length"/>
                  </xsl:call-template>
                </xsl:variable>

                <xsl:variable name="hX" select="msxsl:node-set($localDistVals)/along - msxsl:node-set($prevLenVals)/along"/>
                <xsl:variable name="hY" select="msxsl:node-set($localDistVals)/across - msxsl:node-set($prevLenVals)/across"/>

                <xsl:variable name="along" select="$hX * $startDeflectionX + $hY * $startDeflectionY"/>

                <xsl:variable name="hX2" select="$hX - $along * $startDeflectionX"/>
                <xsl:variable name="hY2" select="$hY - $along * $startDeflectionY"/>

                <xsl:element name="along" namespace="">  <!-- along value to be returned -->
                  <xsl:value-of select="$along"/>
                </xsl:element>

                <xsl:element name="across" namespace=""> <!-- across value to be returned -->
                  <xsl:call-template name="Sqrt">
                    <xsl:with-param name="num" select="$hX2 * $hX2 + $hY2 * $hY2"/>
                  </xsl:call-template>
                </xsl:element>

                <xsl:element name="deflection" namespace="">
                  <!-- Could call the CalcSpiralDeflection template but we already have all the required values so compute it directly -->
                  <xsl:value-of select="(($localDist * $localDist) div (2.0 * $K) - $startDeflection) * 180.0 div $Pi"/>
                </xsl:element>
              </xsl:when>
              
              <xsl:when test="$spiralType = 'NSWCubicParabola'">  <!-- NSWCubicParabola type transition -->
                <xsl:variable name="transitionXc">
                  <xsl:call-template name="CalcNSWTransitionXc">
                    <xsl:with-param name="length" select="$length"/>
                    <xsl:with-param name="smallRadius" select="$smallRadius"/>
                    <xsl:with-param name="largeRadius" select="$largeRadius"/>
                  </xsl:call-template>
                </xsl:variable>

                <xsl:variable name="fullDeflection">
                  <xsl:call-template name="CalcNSWSpiralDeflection">  <!-- Returned in degrees -->
                    <xsl:with-param name="smallRadius" select="$smallRadius"/>
                    <xsl:with-param name="transitionXc" select="$transitionXc"/>
                  </xsl:call-template>
                </xsl:variable>

                <xsl:variable name="spiralM">
                  <xsl:variable name="tanDeflection">
                    <xsl:call-template name="Tan">
                      <xsl:with-param name="theAngle" select="$fullDeflection * $Pi div 180.0"/>
                    </xsl:call-template>
                  </xsl:variable>
                  <xsl:value-of select="$tanDeflection div (3.0 * $transitionXc * $transitionXc)"/>
                </xsl:variable>

                <xsl:variable name="deflection">
                  <xsl:call-template name="CalcNSWDeflectionAtDistance">
                    <xsl:with-param name="smallRadius" select="$smallRadius"/>
                    <xsl:with-param name="largeRadius" select="$largeRadius"/>
                    <xsl:with-param name="length" select="$length"/>
                    <xsl:with-param name="spiralDist" select="$spiralDist"/>
                    <xsl:with-param name="transitionXc" select="$transitionXc"/>
                  </xsl:call-template>
                </xsl:variable>

                <xsl:variable name="fullyDevelopedLength">
                  <xsl:call-template name="CalcNSWSpiralLength">
                    <xsl:with-param name="transitionXc" select="$transitionXc"/>
                    <xsl:with-param name="spiralM" select="$spiralM"/>
                  </xsl:call-template>
                </xsl:variable>
                
                <xsl:variable name="prevLength" select="$fullyDevelopedLength - $length"/>
                <xsl:variable name="localDist" select="$prevLength + $spiralDist"/>

                <xsl:variable name="localDistVals">
                  <xsl:call-template name="CalcNSWSpiralXY">
                    <xsl:with-param name="spiralDist" select="$localDist"/>
                    <xsl:with-param name="spiralM" select="$spiralM"/>
                  </xsl:call-template>
                </xsl:variable>

                <xsl:variable name="prevLenVals">
                  <xsl:call-template name="CalcNSWSpiralXY">
                    <xsl:with-param name="spiralDist" select="$prevLength"/>
                    <xsl:with-param name="spiralM" select="$spiralM"/>
                  </xsl:call-template>
                </xsl:variable>
                
                <xsl:variable name="startDeflection">
                  <xsl:call-template name="ArcTanSeries">
                    <xsl:with-param name="tanVal" select="3.0 * $spiralM * msxsl:node-set($prevLenVals)/along * msxsl:node-set($prevLenVals)/along"/>
                  </xsl:call-template>
                </xsl:variable>
                
                <xsl:variable name="startDeflectionX">
                  <xsl:call-template name="Cosine">
                    <xsl:with-param name="theAngle" select="$startDeflection"/>
                  </xsl:call-template>
                </xsl:variable>

                <xsl:variable name="startDeflectionY">
                  <xsl:call-template name="Sine">
                    <xsl:with-param name="theAngle" select="$startDeflection"/>
                  </xsl:call-template>
                </xsl:variable>

                <xsl:variable name="hX" select="msxsl:node-set($localDistVals)/along - msxsl:node-set($prevLenVals)/along"/>
                <xsl:variable name="hY" select="msxsl:node-set($localDistVals)/across - msxsl:node-set($prevLenVals)/across"/>

                <xsl:variable name="along" select="$hX * $startDeflectionX + $hY * $startDeflectionY"/>

                <xsl:variable name="hX2" select="$hX - $along * $startDeflectionX"/>
                <xsl:variable name="hY2" select="$hY - $along * $startDeflectionY"/>

                <xsl:element name="along" namespace="">  <!-- along value to be returned -->
                  <xsl:value-of select="$along"/>
                </xsl:element>

                <xsl:element name="across" namespace=""> <!-- across value to be returned -->
                  <xsl:call-template name="Sqrt">
                    <xsl:with-param name="num" select="$hX2 * $hX2 + $hY2 * $hY2"/>
                  </xsl:call-template>
                </xsl:element>
                
                <xsl:element name="deflection" namespace="">
                  <xsl:value-of select="$deflection - $startDeflection"/>
                </xsl:element>
              </xsl:when>
            </xsl:choose>
          </xsl:when>

          <xsl:otherwise>   <!-- This is a special case of an arc -->
            <xsl:variable name="deflectionAngle" select="$length div $smallRadius"/>

            <xsl:variable name="sinDeflection">
              <xsl:call-template name="Sine">
                <xsl:with-param name="theAngle" select="$deflectionAngle"/>
              </xsl:call-template>
            </xsl:variable>

            <xsl:variable name="cosDeflection">
              <xsl:call-template name="Cosine">
                <xsl:with-param name="theAngle" select="$deflectionAngle"/>
              </xsl:call-template>
            </xsl:variable>

            <xsl:element name="along" namespace="">
              <xsl:value-of select="$sinDeflection * $smallRadius"/>
            </xsl:element>

            <xsl:element name="across" namespace="">
              <xsl:value-of select="$smallRadius - $cosDeflection * $smallRadius"/>
            </xsl:element>

            <xsl:element name="deflection" namespace="">
              <xsl:value-of select="$deflectionAngle * 180.0 div $Pi"/>
            </xsl:element>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:if>
</xsl:template>


<!-- **************************************************************** -->
<!-- ***** Calculate radius at given distance along Bloss spiral **** -->
<!-- **************************************************************** -->
<xsl:template name="CalcRadiusOnBlossSpiral">
  <xsl:param name="spiralDist"/>     <!-- Distance along spiral -->
  <xsl:param name="smallRadius"/>
  <xsl:param name="length"/>         <!-- Total length of spiral -->

  <xsl:value-of select="($smallRadius * $length * $length * $length) div
                        (3.0 * $length * $spiralDist * $spiralDist - 2.0 * $spiralDist * $spiralDist * $spiralDist)"/>
</xsl:template>


<!-- **************************************************************** -->
<!-- ********** Returns clockwise angle between 2 azimuths ********** -->
<!-- **************************************************************** -->
<xsl:template name="ClockwiseAngle">
  <xsl:param name="fromAzimuth"/>    <!-- Azimuths in decimal degrees -->
  <xsl:param name="toAzimuth"/>

  <!-- returns the angle from the fromAzimuth direction to the toAzimuth direction in clockwise direction -->
  <xsl:choose>
    <xsl:when test="(string(number($fromAzimuth)) = 'NaN') or (string(number($toAzimuth)) = 'NaN')"/>
    <xsl:otherwise>
      <xsl:variable name="tempAngle" select="$toAzimuth - $fromAzimuth"/>
      <xsl:call-template name="NormalisedAngle">
        <xsl:with-param name="angle" select="$tempAngle"/>
      </xsl:call-template>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!-- **************************************************************** -->
<!-- ********* Return radians angle between Specified Limits ******** -->
<!-- **************************************************************** -->
<xsl:template name="RadianAngleBetweenLimits">
  <xsl:param name="anAngle"/>
  <xsl:param name="minVal" select="0.0"/>
  <xsl:param name="maxVal" select="$Pi * 2.0"/>
  <xsl:param name="incVal" select="$Pi * 2.0"/>

  <xsl:variable name="angle1">
    <xsl:call-template name="AngleValueLessThanMax">
      <xsl:with-param name="inAngle" select="$anAngle"/>
      <xsl:with-param name="maxVal" select="$maxVal"/>
      <xsl:with-param name="incVal" select="$incVal"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="angle2">
    <xsl:call-template name="AngleValueGreaterThanMin">
      <xsl:with-param name="inAngle" select="$angle1"/>
      <xsl:with-param name="minVal" select="$minVal"/>
      <xsl:with-param name="incVal" select="$incVal"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:value-of select="$angle2"/>
</xsl:template>


<!-- **************************************************************** -->
<!-- * Base & exponent vals used by ConvertDoubleToExponentialValue * -->
<!-- **************************************************************** -->
<xsl:template name="GetBaseValueAndExponent">
  <xsl:param name="value"/>
  <xsl:param name="exponent" select="0"/>

  <xsl:choose>
    <xsl:when test="$value &gt;= 10.0">
      <xsl:call-template name="GetBaseValueAndExponent">
        <xsl:with-param name="value" select="$value div 10.0"/>
        <xsl:with-param name="exponent" select="$exponent + 1"/>
      </xsl:call-template>
    </xsl:when>

    <xsl:when test="$value &lt; 1.0">
      <xsl:call-template name="GetBaseValueAndExponent">
        <xsl:with-param name="value" select="$value * 10.0"/>
        <xsl:with-param name="exponent" select="$exponent - 1"/>
      </xsl:call-template>
    </xsl:when>

    <xsl:otherwise>
      <xsl:element name="base" namespace="">
        <xsl:value-of select="$value"/>
      </xsl:element>
      <xsl:element name="exponent" namespace="">
        <xsl:choose>
          <xsl:when test="$exponent &gt;= 0.0">
            <xsl:text>+</xsl:text>
            <xsl:value-of select="format-number($exponent, '000')"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>-</xsl:text>
            <xsl:value-of select="format-number($exponent * -1.0, '000')"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:element>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!-- **************************************************************** -->
<!-- *** Calculate spiral deflection angle for NSWCubicParabola ***** -->
<!-- **************************************************************** -->
<xsl:template name="CalcNSWSpiralDeflection">
  <xsl:param name="smallRadius"/>
  <xsl:param name="transitionXc"/>

  <xsl:variable name="sqRoot3" select="1.73205080756887729"/>
  
  <xsl:variable name="arcCosVal">
    <xsl:call-template name="ArcCos">
      <xsl:with-param name="cosVal" select="-3.0 * $sqRoot3 * $transitionXc div (4.0 * $smallRadius)"/>
    </xsl:call-template>
  </xsl:variable>
  
  <xsl:variable name="arcCosValInDeg" select="$arcCosVal * (180.0 div $Pi)"/>
  
  <xsl:variable name="cosVal">
    <xsl:call-template name="Cosine">
      <xsl:with-param name="theAngle" select="($arcCosValInDeg div 3.0 + 240.0) * $Pi div 180.0"/>
    </xsl:call-template>
  </xsl:variable>
  
  <xsl:variable name="radiansVal">
    <xsl:call-template name="ArcSin">
      <xsl:with-param name="sinVal" select="2.0 div $sqRoot3 * $cosVal"/>
    </xsl:call-template>
  </xsl:variable>
  
  <xsl:value-of select="$radiansVal * 180.0 div $Pi"/>  <!-- Return deflection value in degrees -->
</xsl:template>


<!-- **************************************************************** -->
<!-- ** Calculate NSWCubicParabola deflection at dist along spiral ** -->
<!-- **************************************************************** -->
<xsl:template name="CalcNSWDeflectionAtDistance">
  <xsl:param name="smallRadius"/>
  <xsl:param name="largeRadius"/>
  <xsl:param name="length"/>
  <xsl:param name="spiralDist"/>
  <xsl:param name="transitionXc"/>

  <xsl:variable name="deflectionAtLargeRadius">
    <xsl:choose>
      <xsl:when test="string(number($largeRadius)) = 'NaN'">0</xsl:when>  <!-- Standard spiral to infinity -->
      <xsl:otherwise>
        <xsl:variable name="largeRadiusVals">  <!-- Get the along value at the large radius (spiralDist = 0) -->
          <xsl:call-template name="CalcNSWRadiusAtDistance">
            <xsl:with-param name="smallRadius" select="$smallRadius"/>
            <xsl:with-param name="largeRadius" select="$largeRadius"/>
            <xsl:with-param name="length" select="$length"/>
            <xsl:with-param name="spiralDist" select="0"/>
            <xsl:with-param name="transitionXc" select="$transitionXc"/>
          </xsl:call-template>
        </xsl:variable>

        <xsl:call-template name="CalcNSWSpiralDeflection">  <!-- Returned in degrees -->
          <xsl:with-param name="smallRadius" select="$largeRadius"/>
          <xsl:with-param name="transitionXc" select="msxsl:node-set($largeRadiusVals)/along"/>
        </xsl:call-template>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <xsl:choose>
    <xsl:when test="$spiralDist &lt; 0.00005">0</xsl:when>

    <xsl:when test="concat(substring('-',2 - (($length - $spiralDist) &lt; 0)), '1') * ($length - $spiralDist) &lt; 0.00005">
      <!-- At the smallest radius for the spiral -->
      <xsl:variable name="deflection">
        <xsl:call-template name="CalcNSWSpiralDeflection">  <!-- Returned in degrees -->
          <xsl:with-param name="smallRadius" select="$smallRadius"/>
          <xsl:with-param name="transitionXc" select="$transitionXc"/>
        </xsl:call-template>
      </xsl:variable>
      
      <!-- Return the deflection less the deflectionAtLargeRadius -->
      <xsl:value-of select="$deflection - $deflectionAtLargeRadius"/>
    </xsl:when>

    <xsl:otherwise>
      <!-- First get the radius at this position -->
      <xsl:variable name="radiusVals">
        <xsl:call-template name="CalcNSWRadiusAtDistance">
          <xsl:with-param name="smallRadius" select="$smallRadius"/>
          <xsl:with-param name="largeRadius" select="$largeRadius"/>
          <xsl:with-param name="length" select="$length"/>
          <xsl:with-param name="spiralDist" select="$spiralDist"/>
          <xsl:with-param name="transitionXc" select="$transitionXc"/>
        </xsl:call-template>
      </xsl:variable>

      <xsl:variable name="deflection">
        <xsl:call-template name="CalcNSWSpiralDeflection">  <!-- Returned in degrees -->
          <xsl:with-param name="smallRadius" select="msxsl:node-set($radiusVals)/radius"/>
          <xsl:with-param name="transitionXc" select="msxsl:node-set($radiusVals)/along"/>
        </xsl:call-template>
      </xsl:variable>

      <!-- Return the deflection less the deflectionAtLargeRadius -->
      <xsl:value-of select="$deflection - $deflectionAtLargeRadius"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!-- **************************************************************** -->
<!-- ********* Calculate NSWCubicParabola transitionXc value ******** -->
<!-- **************************************************************** -->
<xsl:template name="CalcNSWTransitionXc">
  <xsl:param name="length"/>
  <xsl:param name="smallRadius"/>
  <xsl:param name="largeRadius"/>

  <xsl:choose>
    <xsl:when test="string(number($largeRadius)) = 'NaN'">  <!-- Standard fully developed transition -->
      <xsl:call-template name="CalcNSWIterateXcStandard">
        <xsl:with-param name="transitionXc" select="$length * 0.99"/>  <!-- Initial estimate of TransitionXc value -->
        <xsl:with-param name="length" select="$length"/>
        <xsl:with-param name="smallRadius" select="$smallRadius"/>
      </xsl:call-template>
    </xsl:when>
    
    <xsl:otherwise>  <!-- Compound transition between 2 arcs -->
      <xsl:call-template name="CalcNSWIterateXcCompound">
        <xsl:with-param name="transitionXc" select="$smallRadius * 0.68"/>  <!-- Initial estimate of TransitionXc value -->
        <xsl:with-param name="length" select="$length"/>
        <xsl:with-param name="smallRadius" select="$smallRadius"/>
        <xsl:with-param name="largeRadius" select="$largeRadius"/>
      </xsl:call-template>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!-- **************************************************************** -->
<!-- **************** Calculate spiral X and Y values *************** -->
<!-- **************************************************************** -->
<xsl:template name="CalcSpiralXY">
  <xsl:param name="spiralDist"/>
  <xsl:param name="K"/>           <!-- Used for Clothoid spiral, Cubic spiral and KoreanCubicParabola transitions -->
  <xsl:param name="spiralM"/>     <!-- Used for NSWCubicParabola transition -->
  <xsl:param name="spiralType"/>
  <xsl:param name="smallRadius"/> <!-- Used for Bloss spiral transition -->
  <xsl:param name="length"/>      <!-- Used for Bloss spiral transition -->

  <!-- Returns node-set variable with the elements:  -->
  <!--   along                                       -->
  <!--   across                                      -->

  <xsl:variable name="squaredDist" select="$spiralDist * $spiralDist"/>
  <xsl:variable name="t1" select="($squaredDist div (2.0 * $K)) * ($squaredDist div (2.0 * $K)) * -1.0"/>

  <xsl:choose>
    <xsl:when test="($spiralType = 'Clothoid') or ($spiralType = 'KoreanCubicParabola') or ($spiralType = 'Cubic')"> <!-- Clothoid spiral, KoreanCubicParabola or Cubic spiral -->
      <!-- Calculate displacement along entry azimuth -->
      <!-- Computation of displacement along the entry azimuth is identical for Clothoid spiral, KoreanCubicParabola and Cubic spiral -->
      <xsl:variable name="alongTerm" select="$spiralDist"/>

      <xsl:variable name="along">
        <xsl:call-template name="alongTermIter">
          <xsl:with-param name="count" select="1"/>
          <xsl:with-param name="t1" select="$t1"/>
          <xsl:with-param name="alongTerm" select="$alongTerm"/>
          <xsl:with-param name="along" select="$alongTerm"/>
        </xsl:call-template>
      </xsl:variable>

      <xsl:element name="along" namespace="">
        <xsl:value-of select="$along"/>
      </xsl:element>

      <!-- Calculate displacement normal to entry azimuth -->
      <xsl:element name="across" namespace="">
        <xsl:choose>
          <xsl:when test="$spiralType = 'Clothoid'">  <!-- Clothoid spiral -->
            <xsl:variable name="acrossTerm" select="$squaredDist * $spiralDist div (6.0 * $K)"/>

            <xsl:call-template name="acrossTermIter">
              <xsl:with-param name="count" select="1"/>
              <xsl:with-param name="t1" select="$t1"/>
              <xsl:with-param name="acrossTerm" select="$acrossTerm"/>
              <xsl:with-param name="across" select="$acrossTerm"/>
            </xsl:call-template>
          </xsl:when>

          <xsl:when test="$spiralType = 'Cubic'">
            <xsl:value-of select="$squaredDist * $spiralDist div (6.0 * $K)"/>
          </xsl:when>

          <xsl:otherwise>  <!-- KoreanCubicParabola transition -->
            <xsl:value-of select="$along * $along div (6.0 * $K div $spiralDist)"/>  <!-- No iteration required -->
          </xsl:otherwise>
        </xsl:choose>
      </xsl:element>
    </xsl:when>

    <xsl:when test="$spiralType = 'NSWCubicParabola'">  <!-- NSWCubicParabola type transition -->
      <xsl:call-template name="CalcNSWSpiralXY">
        <xsl:with-param name="spiralDist" select="$spiralDist"/>
        <xsl:with-param name="spiralM" select="$spiralM"/>
      </xsl:call-template>
    </xsl:when>
    
    <xsl:when test="$spiralType = 'Bloss'">
      <xsl:call-template name="CalcBlossSpiralXY">
        <xsl:with-param name="spiralDist" select="$spiralDist"/>
        <xsl:with-param name="smallRadius" select="$smallRadius"/>
        <xsl:with-param name="length" select="$length"/>
      </xsl:call-template>
    </xsl:when>
  </xsl:choose>

</xsl:template>


<!-- **************************************************************** -->
<!-- ***** Calculate spiral X and Y values for NSWCubicParabola ***** -->
<!-- **************************************************************** -->
<xsl:template name="CalcNSWSpiralXY">
  <xsl:param name="spiralDist"/>
  <xsl:param name="spiralM"/>

  <!-- Returns node-set variable with the elements:  -->
  <!--   along                                       -->
  <!--   across                                      -->

  <xsl:variable name="squaredDist" select="$spiralDist * $spiralDist"/>
  <xsl:variable name="t1" select="($spiralM * $squaredDist) * ($spiralM * $squaredDist) * -1"/>

  <xsl:variable name="along">
    <xsl:call-template name="alongNSWTermIter">
      <xsl:with-param name="count" select="1"/>
      <xsl:with-param name="t1" select="$t1"/>
      <xsl:with-param name="t1Start" select="$t1"/>
      <xsl:with-param name="spiralDist" select="$spiralDist"/>
      <xsl:with-param name="alongTerm" select="$spiralDist"/>
      <xsl:with-param name="along" select="$spiralDist"/>
    </xsl:call-template>
  </xsl:variable>
  
  <xsl:element name="along" namespace="">
    <xsl:value-of select="$along"/>
  </xsl:element>
  
  <xsl:element name="across" namespace="">
    <xsl:value-of select="$spiralM * $along * $along * $along"/>
  </xsl:element>
</xsl:template>


<!-- **************************************************************** -->
<!-- ********** Calculate NSWCubicParabola spiral length ************ -->
<!-- **************************************************************** -->
<xsl:template name="CalcNSWSpiralLength">
  <xsl:param name="transitionXc"/>
  <xsl:param name="spiralM"/>

  <xsl:variable name="squaredDist" select="$transitionXc * $transitionXc"/>
  <xsl:variable name="t1" select="($spiralM * $squaredDist) * ($spiralM * $squaredDist)"/>

  <xsl:call-template name="lengthNSWTermIter">
    <xsl:with-param name="count" select="1"/>
    <xsl:with-param name="t1" select="$t1"/>
    <xsl:with-param name="t1Start" select="$t1 * -1"/>
    <xsl:with-param name="transitionXc" select="$transitionXc"/>
    <xsl:with-param name="lengthTerm" select="$transitionXc"/>
    <xsl:with-param name="length" select="$transitionXc"/>
  </xsl:call-template>
</xsl:template>


<!-- **************************************************************** -->
<!-- ******* Return radians angle less than Specificed Maximum ****** -->
<!-- **************************************************************** -->
<xsl:template name="AngleValueLessThanMax">
  <xsl:param name="inAngle"/>
  <xsl:param name="maxVal"/>
  <xsl:param name="incVal"/>

  <xsl:choose>
    <xsl:when test="$inAngle &gt; $maxVal">
      <xsl:variable name="newAngle">
        <xsl:value-of select="$inAngle - $incVal"/>
      </xsl:variable>
      <xsl:call-template name="AngleValueLessThanMax">
        <xsl:with-param name="inAngle" select="$newAngle"/>
      </xsl:call-template>
    </xsl:when>

    <xsl:otherwise>
      <xsl:value-of select="$inAngle"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!-- **************************************************************** -->
<!-- ************* Return radians angle greater than Zero *********** -->
<!-- **************************************************************** -->
<xsl:template name="AngleValueGreaterThanMin">
  <xsl:param name="inAngle"/>
  <xsl:param name="minVal"/>
  <xsl:param name="incVal"/>

  <xsl:choose>
    <xsl:when test="$inAngle &lt; $minVal">
      <xsl:variable name="newAngle">
        <xsl:value-of select="$inAngle + $incVal"/>
      </xsl:variable>
      <xsl:call-template name="AngleValueGreaterThanMin">
        <xsl:with-param name="inAngle" select="$newAngle"/>
      </xsl:call-template>
    </xsl:when>

    <xsl:otherwise>
      <xsl:value-of select="$inAngle"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!-- **************************************************************** -->
<!-- ********** Compute ArcCosine value using an expansion ********** -->
<!-- **************************************************************** -->
<xsl:template name="ArcCos">
  <xsl:param name="cosVal"/>

  <xsl:choose>
    <xsl:when test="($cosVal &gt;= -1.0) and ($cosVal &lt;= 1.0)">  <!-- We can compute a solution -->
      <!-- Use the ArcSin expansion to return the ArcCos value -->
      <xsl:variable name="arcSineVal">
        <xsl:call-template name="ArcSin">
          <xsl:with-param name="sinVal" select="$cosVal"/>       
        </xsl:call-template>
      </xsl:variable>
      <xsl:value-of select="$halfPi - $arcSineVal"/>
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="''"/>  <!-- Return null value -->
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!-- **************************************************************** -->
<!-- *********** Compute ArcSine value using an expansion *********** -->
<!-- **************************************************************** -->
<xsl:template name="ArcSin">
  <xsl:param name="sinVal"/>
  
  <xsl:choose>
    <xsl:when test="($sinVal &gt;= -1.0) and ($sinVal &lt;= 1.0)">  <!-- We can compute a solution -->
      <xsl:variable name="valToPowerOf3" select="$sinVal * $sinVal * $sinVal"/>
      <xsl:variable name="valToPowerOf5" select="$valToPowerOf3 * $sinVal * $sinVal"/>
      <xsl:variable name="valToPowerOf7" select="$valToPowerOf5 * $sinVal * $sinVal"/>
      <xsl:variable name="valToPowerOf9" select="$valToPowerOf7 * $sinVal * $sinVal"/>
      <xsl:variable name="valToPowerOf11" select="$valToPowerOf9 * $sinVal * $sinVal"/>
      <xsl:variable name="valToPowerOf13" select="$valToPowerOf11 * $sinVal * $sinVal"/>
      <xsl:variable name="valToPowerOf15" select="$valToPowerOf13 * $sinVal * $sinVal"/>
      <xsl:variable name="valToPowerOf17" select="$valToPowerOf15 * $sinVal * $sinVal"/>
      <xsl:variable name="valToPowerOf19" select="$valToPowerOf17 * $sinVal * $sinVal"/>
      <xsl:variable name="valToPowerOf21" select="$valToPowerOf19 * $sinVal * $sinVal"/>
      <xsl:variable name="valToPowerOf23" select="$valToPowerOf21 * $sinVal * $sinVal"/>
      <xsl:variable name="valToPowerOf25" select="$valToPowerOf23 * $sinVal * $sinVal"/>
      <xsl:variable name="valToPowerOf27" select="$valToPowerOf25 * $sinVal * $sinVal"/>
      <xsl:variable name="valToPowerOf29" select="$valToPowerOf27 * $sinVal * $sinVal"/>
      <xsl:variable name="valToPowerOf31" select="$valToPowerOf29 * $sinVal * $sinVal"/>
      <xsl:variable name="valToPowerOf33" select="$valToPowerOf31 * $sinVal * $sinVal"/>
      <xsl:variable name="valToPowerOf35" select="$valToPowerOf33 * $sinVal * $sinVal"/>
      <xsl:variable name="valToPowerOf37" select="$valToPowerOf35 * $sinVal * $sinVal"/>

      <xsl:value-of select="$sinVal + $valToPowerOf3 div 6.0
                                    + 3.0          * $valToPowerOf5  div 40.0
                                    + 5.0          * $valToPowerOf7  div 112.0
                                    + 35.0         * $valToPowerOf9  div 1152.0
                                    + 63.0         * $valToPowerOf11 div 2816
                                    + 231.0        * $valToPowerOf13 div 13312
                                    + 143.0        * $valToPowerOf15 div 10240
                                    + 6435.0       * $valToPowerOf17 div 557056.0
                                    + 12155.0      * $valToPowerOf19 div 1245184.0
                                    + 46189.0      * $valToPowerOf21 div 5505024.0
                                    + 88179.0      * $valToPowerOf23 div 12058624.0
                                    + 676039.0     * $valToPowerOf25 div 104857600.0
                                    + 1300075.0    * $valToPowerOf27 div 226492416.0
                                    + 5014575.0    * $valToPowerOf29 div 973078528.0
                                    + 9694845.0    * $valToPowerOf31 div 2080374784.0
                                    + 100180065.0  * $valToPowerOf33 div 23622320128.0
                                    + 116680311.0  * $valToPowerOf35 div 30064771072.0
                                    + 2268783825.0 * $valToPowerOf37 div 635655159808.0"/>
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="''"/>  <!-- Return null value -->
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!-- **************************************************************** -->
<!-- **** Calculate NSWCubicParabola radius at dist along spiral **** -->
<!-- **************************************************************** -->
<xsl:template name="CalcNSWRadiusAtDistance">
  <xsl:param name="smallRadius"/>
  <xsl:param name="largeRadius"/>
  <xsl:param name="length"/>
  <xsl:param name="spiralDist"/>
  <xsl:param name="transitionXc"/>

  <!-- Return a node set variable that provides: -->
  <!--   radius  -->
  <!--   along   -->
  
  <xsl:variable name="deflection">  <!-- Returned in degrees -->
    <xsl:call-template name="CalcNSWSpiralDeflection">
      <xsl:with-param name="smallRadius" select="$smallRadius"/>
      <xsl:with-param name="transitionXc" select="$transitionXc"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="spiralM">
    <xsl:variable name="tanDeflection">
      <xsl:call-template name="Tan">
        <xsl:with-param name="theAngle" select="$deflection * $Pi div 180.0"/>
      </xsl:call-template>
    </xsl:variable>
    <xsl:value-of select="$tanDeflection div (3.0 * $transitionXc * $transitionXc)"/>
  </xsl:variable>

  <xsl:variable name="spiralVals">
    <xsl:choose>
      <xsl:when test="string(number($largeRadius)) = 'NaN'"> <!-- This is a standard spiral to infinity -->
        <xsl:call-template name="CalcNSWSpiralXY">
          <xsl:with-param name="spiralDist" select="$spiralDist"/>
          <xsl:with-param name="spiralM" select="$spiralM"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>  <!-- This is a spiral between 2 arcs -->
        <xsl:variable name="totalLength">  <!-- Spiral length from infinity to smallRadius -->
          <xsl:call-template name="CalcNSWSpiralLength">
            <xsl:with-param name="transitionXc" select="$transitionXc"/>
            <xsl:with-param name="spiralM" select="$spiralM"/>
          </xsl:call-template>
        </xsl:variable>

        <xsl:variable name="lengthToPoint" select="$totalLength - ($length - $spiralDist)"/>

        <xsl:call-template name="CalcNSWSpiralXY">
          <xsl:with-param name="spiralDist" select="$lengthToPoint"/>
          <xsl:with-param name="spiralM" select="$spiralM"/>
        </xsl:call-template>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <xsl:variable name="x" select="msxsl:node-set($spiralVals)/along"/>
  <xsl:variable name="term" select="1.0 + 9.0 * $spiralM * $spiralM * $x * $x * $x * $x"/>
  <xsl:variable name="num">
    <xsl:call-template name="Sqrt">
      <xsl:with-param name="num" select="$term * $term * $term"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:element name="radius" namespace="">
    <xsl:value-of select="$num div (6.0 * $spiralM * $x)"/>
  </xsl:element>
  
  <xsl:element name="along" namespace="">
    <xsl:value-of select="$x"/>
  </xsl:element>
</xsl:template>


<!-- **************************************************************** -->
<!-- ***** Recursive func to calc NSWCubicParabola transitionXc ***** -->
<!-- **************************************************************** -->
<xsl:template name="CalcNSWIterateXcStandard">  <!-- Standard fully defined case -->
  <xsl:param name="transitionXc"/>
  <xsl:param name="length"/>
  <xsl:param name="smallRadius"/>

  <xsl:variable name="deflection">  <!-- Returned in degrees -->
    <xsl:call-template name="CalcNSWSpiralDeflection">
      <xsl:with-param name="smallRadius" select="$smallRadius"/>
      <xsl:with-param name="transitionXc" select="$transitionXc"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="spiralM">
    <xsl:variable name="tanDeflection">
      <xsl:call-template name="Tan">
        <xsl:with-param name="theAngle" select="$deflection * $Pi div 180.0"/>
      </xsl:call-template>
    </xsl:variable>
    <xsl:value-of select="$tanDeflection div (3.0 * $transitionXc * $transitionXc)"/>
  </xsl:variable>

  <!-- Get the calculated spiral length based on the current Xc value -->
  <xsl:variable name="calcLength">
    <xsl:call-template name="CalcNSWSpiralLength">
      <xsl:with-param name="transitionXc" select="$transitionXc"/>
      <xsl:with-param name="spiralM" select="$spiralM"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="absLengthDelta" select="concat(substring('-',2 - (($calcLength - $length) &lt; 0)), '1') * ($calcLength - $length)"/>

  <xsl:choose>
    <xsl:when test="$absLengthDelta &gt; 0.0000001">
      <!-- Recurse function adjusting transitionXc value by known length to computed length proportion -->
      <xsl:call-template name="CalcNSWIterateXcStandard">
        <xsl:with-param name="transitionXc" select="$transitionXc * $length div $calcLength"/>
        <xsl:with-param name="length" select="$length"/>
        <xsl:with-param name="smallRadius" select="$smallRadius"/>
      </xsl:call-template>
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="$transitionXc"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!-- **************************************************************** -->
<!-- ***** Recursive func to calc NSWCubicParabola transitionXc ***** -->
<!-- **************************************************************** -->
<xsl:template name="CalcNSWIterateXcCompound">  <!-- Compound transition between 2 arcs case -->
  <xsl:param name="transitionXc"/>
  <xsl:param name="length"/>
  <xsl:param name="smallRadius"/>
  <xsl:param name="largeRadius"/>

  <xsl:variable name="deflection">  <!-- Returned in degrees -->
    <xsl:call-template name="CalcNSWSpiralDeflection">
      <xsl:with-param name="smallRadius" select="$smallRadius"/>
      <xsl:with-param name="transitionXc" select="$transitionXc"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="spiralM">
    <xsl:variable name="tanDeflection">
      <xsl:call-template name="Tan">
        <xsl:with-param name="theAngle" select="$deflection * $Pi div 180.0"/>
      </xsl:call-template>
    </xsl:variable>
    <xsl:value-of select="$tanDeflection div (3.0 * $transitionXc * $transitionXc)"/>
  </xsl:variable>

  <xsl:variable name="transitionXl">
    <xsl:call-template name="CalcNSWIterateXl">
      <xsl:with-param name="transitionXl" select="1.0 div (6.0 * $spiralM * $largeRadius)"/>
      <xsl:with-param name="largeRadius" select="$largeRadius"/>
      <xsl:with-param name="spiralM" select="$spiralM"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="lenLargeRad">
    <xsl:call-template name="CalcNSWSpiralLength">
      <xsl:with-param name="transitionXc" select="$transitionXl"/>
      <xsl:with-param name="spiralM" select="$spiralM"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="totalLength">
    <xsl:call-template name="CalcNSWSpiralLength">
      <xsl:with-param name="transitionXc" select="$transitionXc"/>
      <xsl:with-param name="spiralM" select="$spiralM"/>
    </xsl:call-template>
  </xsl:variable>
  
  <xsl:variable name="absLengthDelta" select="concat(substring('-',2 - ((($totalLength - $lenLargeRad) - $length) &lt; 0)), '1') * (($totalLength - $lenLargeRad) - $length)"/>

  <xsl:choose>
    <xsl:when test="$absLengthDelta &gt; 0.0000001">
      <!-- Recurse function adjusting transitionXc value by known length to computed length proportion -->
      <xsl:call-template name="CalcNSWIterateXcCompound">
        <xsl:with-param name="transitionXc" select="$transitionXc * $length div ($totalLength - $lenLargeRad)"/>
        <xsl:with-param name="length" select="$length"/>
        <xsl:with-param name="smallRadius" select="$smallRadius"/>
        <xsl:with-param name="largeRadius" select="$largeRadius"/>
      </xsl:call-template>
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="$transitionXc"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<xsl:template name="acrossTermIter">
  <xsl:param name="count"/>
  <xsl:param name="maxIter" select="30"/>
  <xsl:param name="t1"/>
  <xsl:param name="acrossTerm"/>
  <xsl:param name="across"/>

  <xsl:variable name="resolution" select="0.0000001"/>

  <xsl:choose>
    <xsl:when test="((concat(substring('-',2 - ($acrossTerm &lt; 0)), '1') * $acrossTerm) &gt; $resolution) and
                    ($count &lt; $maxIter)">
      <xsl:variable name="t2" select="((4 * $count + 3) * $count * (4 * $count + 2))"/>
      <xsl:variable name="term" select="$acrossTerm * ($t1 * ((4 * $count - 1) div $t2))"/>
      <!-- Now recurse function -->
      <xsl:call-template name="acrossTermIter">
        <xsl:with-param name="count" select="$count + 1"/>
        <xsl:with-param name="maxIter" select="$maxIter"/>
        <xsl:with-param name="t1" select="$t1"/>
        <xsl:with-param name="acrossTerm" select="$term"/>
        <xsl:with-param name="across" select="$across + $term"/>
      </xsl:call-template>
    </xsl:when>

    <xsl:otherwise>
      <xsl:value-of select="$across"/>  <!-- Return the iterated across value -->
    </xsl:otherwise>
  </xsl:choose>

</xsl:template>


<xsl:template name="alongTermIter">
  <xsl:param name="count"/>
  <xsl:param name="maxIter" select="30"/>
  <xsl:param name="t1"/>
  <xsl:param name="alongTerm"/>
  <xsl:param name="along"/>

  <xsl:variable name="resolution" select="0.0000001"/>

  <xsl:choose>
    <xsl:when test="((concat(substring('-',2 - ($alongTerm &lt; 0)), '1') * $alongTerm) &gt; $resolution) and
                    ($count &lt; $maxIter)">
      <xsl:variable name="t2" select="((4 * $count + 1) * $count * (4 * $count - 2))"/>
      <xsl:variable name="term" select="$alongTerm * ($t1 * ((4 * $count - 3) div $t2))"/>
      <!-- Now recurse function -->
      <xsl:call-template name="alongTermIter">
        <xsl:with-param name="count" select="$count + 1"/>
        <xsl:with-param name="maxIter" select="$maxIter"/>
        <xsl:with-param name="t1" select="$t1"/>
        <xsl:with-param name="alongTerm" select="$term"/>
        <xsl:with-param name="along" select="$along + $term"/>
      </xsl:call-template>
    </xsl:when>

    <xsl:otherwise>
      <xsl:value-of select="$along"/>  <!-- Return the iterated along value -->
    </xsl:otherwise>
  </xsl:choose>

</xsl:template>


<xsl:template name="alongNSWTermIter">
  <xsl:param name="count"/>
  <xsl:param name="maxIter" select="5"/>
  <xsl:param name="t1"/>
  <xsl:param name="t1Start"/>
  <xsl:param name="spiralDist"/>
  <xsl:param name="alongTerm"/>
  <xsl:param name="along"/>

  <xsl:variable name="resolution" select="0.0000001"/>

  <xsl:choose>
    <xsl:when test="((concat(substring('-',2 - ($alongTerm &lt; 0)), '1') * $alongTerm) &gt; $resolution) and
                    ($count &lt; $maxIter)">
      <xsl:variable name="t2">
        <xsl:choose>
          <xsl:when test="$count = 1">0.9</xsl:when>
          <xsl:when test="$count = 2">5.175</xsl:when>
          <xsl:when test="$count = 3">43.1948</xsl:when>
          <xsl:when test="$count = 4">426.0564</xsl:when>
        </xsl:choose>
      </xsl:variable>

      <xsl:variable name="term" select="$t1 * $t2 * $spiralDist"/>

      <!-- Now recurse function -->
      <xsl:call-template name="alongNSWTermIter">
        <xsl:with-param name="count" select="$count + 1"/>
        <xsl:with-param name="maxIter" select="$maxIter"/>
        <xsl:with-param name="t1" select="$t1 * $t1Start"/>
        <xsl:with-param name="t1Start" select="$t1Start"/>
        <xsl:with-param name="spiralDist" select="$spiralDist"/>
        <xsl:with-param name="alongTerm" select="$term"/>
        <xsl:with-param name="along" select="$along + $term"/>
      </xsl:call-template>
    </xsl:when>
    
    <xsl:otherwise>
      <xsl:value-of select="$along"/>  <!-- Return the iterated along value -->
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<xsl:template name="lengthNSWTermIter">
  <xsl:param name="count"/>
  <xsl:param name="maxIter" select="5"/>
  <xsl:param name="t1"/>
  <xsl:param name="t1Start"/>
  <xsl:param name="transitionXc"/>
  <xsl:param name="lengthTerm"/>
  <xsl:param name="length"/>

  <xsl:variable name="resolution" select="0.0000001"/>

  <xsl:choose>
    <xsl:when test="((concat(substring('-',2 - ($lengthTerm &lt; 0)), '1') * $lengthTerm) &gt; $resolution) and
                    ($count &lt; $maxIter)">
      <xsl:variable name="t2">
        <xsl:choose>
          <xsl:when test="$count = 1">0.9</xsl:when>
          <xsl:when test="$count = 2">1.125</xsl:when>   <!-- 9.0 / 8.0 -->
          <xsl:when test="$count = 3"><xsl:value-of select="729.0 div 208.0"/></xsl:when>
          <xsl:when test="$count = 4"><xsl:value-of select="32805.0 div 2176.0"/></xsl:when>
        </xsl:choose>
      </xsl:variable>

      <xsl:variable name="term" select="$t1 * $t2 * $transitionXc"/>

      <!-- Now recurse function -->
      <xsl:call-template name="lengthNSWTermIter">
        <xsl:with-param name="count" select="$count + 1"/>
        <xsl:with-param name="maxIter" select="$maxIter"/>
        <xsl:with-param name="t1" select="$t1 * $t1Start"/>
        <xsl:with-param name="t1Start" select="$t1Start"/>
        <xsl:with-param name="transitionXc" select="$transitionXc"/>
        <xsl:with-param name="lengthTerm" select="$term"/>
        <xsl:with-param name="length" select="$length + $term"/>
      </xsl:call-template>
    </xsl:when>

    <xsl:otherwise>
      <xsl:value-of select="$length"/>  <!-- Return the iterated length value -->
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!-- **************************************************************** -->
<!-- ******* Calculate spiral X and Y values for Bloss spiral ******* -->
<!-- **************************************************************** -->
<xsl:template name="CalcBlossSpiralXY">
  <xsl:param name="spiralDist"/>     <!-- Distance along spiral -->
  <xsl:param name="smallRadius"/>
  <xsl:param name="length"/>         <!-- Total length of spiral -->

  <!-- Returns node-set variable with the elements:  -->
  <!--   along                                       -->
  <!--   across                                      -->

  <xsl:variable name="t1" select="$spiralDist * $spiralDist * $spiralDist div ($smallRadius * $length * $length)"/>
  <xsl:variable name="t2" select="$t1 * $spiralDist div $length"/>
  
  <xsl:element name="along" namespace="">
    <xsl:call-template name="alongBlossTermIter">
      <xsl:with-param name="count">1</xsl:with-param>
      <xsl:with-param name="t1" select="$t1"/>
      <xsl:with-param name="t2" select="$t2"/>
      <xsl:with-param name="spiralDist" select="$spiralDist"/>
      <xsl:with-param name="along">0.0</xsl:with-param>
    </xsl:call-template>
  </xsl:element>

  <xsl:element name="across" namespace="">
    <xsl:call-template name="acrossBlossTermIter">
      <xsl:with-param name="count">1</xsl:with-param>
      <xsl:with-param name="t1" select="$t1"/>
      <xsl:with-param name="t2" select="$t2"/>
      <xsl:with-param name="spiralDist" select="$spiralDist"/>
      <xsl:with-param name="across">0.0</xsl:with-param>
    </xsl:call-template>
  </xsl:element>
</xsl:template>


<!-- **************************************************************** -->
<!-- ***** Recursive func to calc NSWCubicParabola transitionXl ***** -->
<!-- **************************************************************** -->
<xsl:template name="CalcNSWIterateXl">
  <xsl:param name="transitionXl"/>
  <xsl:param name="largeRadius"/>
  <xsl:param name="spiralM"/>

  <xsl:variable name="calcLargeRadius">
    <xsl:call-template name="CalcNSWRadiusAtTangentDistance">
      <xsl:with-param name="transitionXc" select="$transitionXl"/>
      <xsl:with-param name="spiralM" select="$spiralM"/>
    </xsl:call-template>
  </xsl:variable>
  
  <xsl:variable name="absRadDelta" select="concat(substring('-',2 - (($calcLargeRadius - $largeRadius) &lt; 0)), '1') * ($calcLargeRadius - $largeRadius)"/>

  <xsl:choose>
    <xsl:when test="$absRadDelta &gt; 0.0000001">
      <!-- Recurse function adjusting transitionXl value by computed large radius to known large radius proportion -->
      <xsl:call-template name="CalcNSWIterateXl">
        <xsl:with-param name="transitionXl" select="$transitionXl * $calcLargeRadius div $largeRadius"/>
        <xsl:with-param name="largeRadius" select="$largeRadius"/>
        <xsl:with-param name="spiralM" select="$spiralM"/>
      </xsl:call-template>
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="$transitionXl"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<xsl:template name="alongBlossTermIter">
  <xsl:param name="count"/>
  <xsl:param name="maxIter" select="3"/>
  <xsl:param name="t1"/>
  <xsl:param name="t2"/>
  <xsl:param name="spiralDist"/>
  <xsl:param name="along"/>

  <xsl:choose>
    <xsl:when test="$count &lt;= $maxIter">
      <xsl:variable name="t3">
        <xsl:choose>
          <xsl:when test="$count = 1">1.0</xsl:when>
          <xsl:when test="$count = 2">
            <xsl:value-of select="- $t1 * $t1 div 14.0 +
                                    $t1 * $t2 div 16.0 -
                                    $t2 * $t2 div 72.0"/>
          </xsl:when>
          <xsl:when test="$count = 3">
            <xsl:value-of select="$t1 * $t1 * $t1 * $t1 div 312.0 -
                                  $t1 * $t1 * $t1 * $t2 div 168.0 +
                                  $t1 * $t1 * $t2 * $t2 div 240.0 -
                                  $t1 * $t2 * $t2 * $t2 div 768.0 +
                                  $t2 * $t2 * $t2 * $t2 div 6528.0"/>
          </xsl:when>
        </xsl:choose>
      </xsl:variable>

      <xsl:variable name="term" select="$spiralDist * $t3"/>

      <!-- Now recurse function -->
      <xsl:call-template name="alongBlossTermIter">
        <xsl:with-param name="count" select="$count + 1"/>
        <xsl:with-param name="maxIter" select="$maxIter"/>
        <xsl:with-param name="t1" select="$t1"/>
        <xsl:with-param name="t2" select="$t2"/>
        <xsl:with-param name="spiralDist" select="$spiralDist"/>
        <xsl:with-param name="along" select="$along + $term"/>
      </xsl:call-template>
    </xsl:when>

    <xsl:otherwise>
      <xsl:value-of select="$along"/>  <!-- Return the iterated along value -->
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<xsl:template name="acrossBlossTermIter">
  <xsl:param name="count"/>
  <xsl:param name="maxIter" select="2"/>
  <xsl:param name="t1"/>
  <xsl:param name="t2"/>
  <xsl:param name="spiralDist"/>
  <xsl:param name="across"/>

  <xsl:choose>
    <xsl:when test="$count &lt;= $maxIter">
      <xsl:variable name="t3">
        <xsl:choose>
          <xsl:when test="$count = 1">
            <xsl:value-of select="$t1 div 4.0 - $t2 div 10.0"/>
          </xsl:when>
          <xsl:when test="$count = 2">
            <xsl:value-of select="- $t1 * $t1 * $t1 div 60.0 +
                                    $t1 * $t1 * $t2 div 44.0 -
                                    $t1 * $t2 * $t2 div 96.0 +
                                    $t2 * $t2 * $t2 div 624.0"/>
          </xsl:when>
        </xsl:choose>
      </xsl:variable>

      <xsl:variable name="term" select="$spiralDist * $t3"/>

      <!-- Now recurse function -->
      <xsl:call-template name="acrossBlossTermIter">
        <xsl:with-param name="count" select="$count + 1"/>
        <xsl:with-param name="maxIter" select="$maxIter"/>
        <xsl:with-param name="t1" select="$t1"/>
        <xsl:with-param name="t2" select="$t2"/>
        <xsl:with-param name="spiralDist" select="$spiralDist"/>
        <xsl:with-param name="across" select="$across + $term"/>
      </xsl:call-template>
    </xsl:when>

    <xsl:otherwise>
      <xsl:value-of select="$across"/>  <!-- Return the iterated across value -->
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!-- **************************************************************** -->
<!-- **** Calculate NSWCubicParabola radius at dist along tangent *** -->
<!-- **************************************************************** -->
<xsl:template name="CalcNSWRadiusAtTangentDistance">
  <xsl:param name="transitionXc"/>
  <xsl:param name="spiralM"/>
  
  <xsl:variable name="param1" select="1.0 + 9.0 * $spiralM * $spiralM * $transitionXc * $transitionXc * $transitionXc * $transitionXc"/>
  <!-- Variable numerator is param1 to power of 3/2 (square root of cube of value) -->
  <xsl:variable name="numerator">
    <xsl:call-template name="Sqrt">
      <xsl:with-param name="num" select="$param1 * $param1 * $param1"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:value-of select="$numerator div (6.0 * $spiralM * $transitionXc)"/>
</xsl:template>


</xsl:stylesheet>
